#
# This is the package for  DifferentialGeometry  Diagonistics>




DifferentialGeometryDiagnostics:=proc(ProcedureList,{tracy:=off}) 

local EqualTransform,Check, TF,BadProcedureList,P;


EqualTransform:=proc(phi,psi) 
	LinearAlgebra:-Equal(JacobianMatrix(phi),JacobianMatrix(psi)) 
and 	op(phi)[2]=op(psi)[2] 
and 	op(phi)[1][1..3]=op(phi)[1][1..3]; 
end; 

#--------------------------

Check[DifferentialGeometry:-`&plus`]:=proc()
local a,b,c,ans;

coord_init([x,y,z],DGChk);

a:=D_x;
b:=D_y;
c:=a &plus b;
ans:=_VESSIOT([[vect, DGChk, []], [[[1], 1], [[2], 1]]]);
if c <> ans then RETURN(false) fi;

a:=dx;
b:=dz;
c:=a &plus b;
ans:=_VESSIOT([[form, DGChk, 1], [[[1], 1], [[3], 1]]]);
if c <> ans then RETURN(false) fi;

a:=dx &wedge dy;
b:=dx &wedge dz;
c:=a &plus b;
ans:=_VESSIOT([[form, DGChk, 2], [[[1, 2], 1], [[1, 3], 1]]]);
if c <> ans then RETURN(false)fi;

a:=dx &tensor D_y;
b:=dy &tensor D_z;
c:=a &plus b;
ans:=_VESSIOT([[tensor, DGChk, [[cov_bas, con_bas], []]], [[[1, 2], 1], [[2, 3], 1]]]);
if c <> ans then RETURN(false)fi;

true:
end:

#-----------------------------------

Check[DifferentialGeometry:-`&minus`]:=proc()
local a,b,c,ans;
coord_init([x,y,z],DGChk);

a:=D_x;
b:=D_y;
c:=a &minus b;
ans:=_VESSIOT([[vect, DGChk, []], [[[1], 1], [[2], -1]]]);
if c <> ans then RETURN(false) fi;

a:=dx;
b:=dz;
c:=a &minus b;
ans:=_VESSIOT([[form, DGChk, 1], [[[1], 1], [[3], -1]]]);
if c <> ans then RETURN(false) fi;

a:=dx &wedge dy;
b:=dx &wedge dz;
c:=a &minus b;
ans:=_VESSIOT([[form, DGChk, 2], [[[1, 2], 1], [[1, 3], -1]]]);
if c <> ans then RETURN(false)fi;

a:=dx &tensor D_y;
b:=dy &tensor D_z;
c:=a &minus b;
ans:=_VESSIOT([[tensor, DGChk, [[cov_bas, con_bas], []]], [[[1, 2], 1], [[2, 3], -1]]]);
if c <> ans then RETURN(false)fi;

true;
end:
#---------------------------------

Check[DifferentialGeometry:-`&mult`]:=proc()

local a,c,ans;

coord_init([x,y,z],DGChk);

a:=D_x;
c:=5 &mult a;
ans:=_VESSIOT([[vect, DGChk, []], [[[1], 5]]]);
if c <> ans then RETURN(false) fi;

a:=dx;
c:=(-2) &mult a;
ans:=_VESSIOT([[form, DGChk, 1], [[[1], -2] ]]);
if c <> ans then RETURN(false) fi;

a:=dx &wedge dy;
c:=(1/z) &mult a;
ans:=_VESSIOT([[form, DGChk, 2], [[[1, 2], 1/z] ]]);
if c <> ans then RETURN(false)fi;

a:=dx &tensor D_y;
c:= (-2) &mult a;
ans:=_VESSIOT([[tensor, DGChk, [[cov_bas, con_bas], []]], [[[1, 2], -2]]]);
if c <> ans then RETURN(false)fi;

true;
end:

#---------------------
   
Check[DifferentialGeometry:-`&wedge`]:=proc()
local a,b,c,ans;
coord_init([x,y,z],DGChk);
a:=dx; b:=dx;
c:= a &wedge  b;
ans:=_VESSIOT([[form, DGChk, 2], [[[1, 2], 0]]]);
if c <> ans then RETURN(false) fi;

a:=evalV(dx +2*dy -3*dz); 
b:=evalV(dx+ dz); 
c:= a &wedge b;
ans:=_VESSIOT([[form, DGChk, 2], [[[1, 2], -2], [[1, 3], 4], [[2, 3], 2]]]);
if c <> ans then RETURN(false) fi;

a:=dx &wedge dy;
b:=evalV(dx +dz);c:= a &wedge b;
ans:=_VESSIOT([[form, DGChk, 3], [[[1, 2,3], 1] ]]);
if c <> ans then RETURN(false)fi;

true;
end:

#--------------------------


Check[DifferentialGeometry:-`&tensor`]:=proc()

local a,b,c,ans;

coord_init([x,y,z],DGChk);

a:=dx; b:=dx;
c:= a &tensor  b;
ans:=_VESSIOT([[tensor, DGChk, [[cov_bas, cov_bas], []]], [[[1, 1], 1]]]);
if c <> ans then RETURN(false) fi;

a:=evalV(dx +2*dy -3*dz); 
b:=evalV(D_x+ 2*D_y); 
c:= a &tensor b;
ans:=_VESSIOT([[tensor, DGChk, [[cov_bas, con_bas], []]], [[[1, 1], 1], 
[[1, 2], 2], [[2, 1], 2], [[2, 2], 4], [[3, 1], -3], [[3, 2], -6]]]);
if c <> ans then RETURN(false) fi;

a:=evalV(dx &t D_y -dz &t D_z);
b:=evalV(dz &t D_x  + dy&t D_y); 
c:= a &tensor b;
ans:=_VESSIOT([[tensor, DGChk, [[cov_bas, con_bas, cov_bas, con_bas], []]], 
[[[1, 2, 2, 2], 1], [[1, 2, 3, 1], 1], [[3, 3, 2, 2], -1], [[3, 3, 3, 1], -1]]]);
if c <> ans then RETURN(false)fi;

true;
end:

#----------------------------------------------

Check[DifferentialGeometry:-change_frame_to]:=proc()

coord_init([x],M1);
coord_init([y],M2);

change_frame_to(M1);

true
end:


#-----------------------------------------------


Check[DifferentialGeometry:-coord_init]:=proc()

coord_init([x]);

coord_init([y],M1);

coord_init([x],[u],M2);

true
end:

#------------------------------------------------

Check[DifferentialGeometry:-coframe_data]:=proc()

local a,b,f, ans; 
coord_init([x,y,z], DGChk);

f:=[x,y,z];
a:=[dx,dy,dz];
b:=coframe_data(f,a);
ans := [[moving_frame, Cartan, [3], DGChk, [x, y, z]], 
	[[[1, 1], 1], [[2, 2], 1], [[3, 3], 1]], []];
if  b <> ans then RETURN(false) fi;

f:=[x,y,z];
a:=evalV([y*dx,z*dy,y*dz]);
b:=coframe_data(f,a);
ans := [[moving_frame, Cartan, [3], DGChk, [x, y, z]], 
	[[[1, 1], 1/y], [[2, 2], 1/z], [[3, 3], 1/y]], [[[1, 2, 1], 1/(y*z)], 
	 [[2, 3, 2], 1/(y*z)], [[2, 3, 3], -1/(y*z)]]];

if b <> ans then RETURN(false) fi;

true;
end:







#-----------------------------------------------

Check[DifferentialGeometry:-DiffGeomMain]:=proc()
#
# What to do here

true;
end:

#---------------------------------------------

Check[DifferentialGeometry:-frame_init]:=proc()
#
# Additional tests of this command are  given by  Repere_mobile_diagnostics




true;
end:

#------------------------------------

















Check[DifferentialGeometry:-helmsimp]:=proc()
local  a,b,c, ans;

coord_init([x,y,z],DGChk);

a:= evalV(x*D_x +D_y);
b:=subs(x=0,a);
c:=helmsimp(b);
ans:=D_y;
if ans <> c then RETURN(false) fi;

a:=evalV(x*dx &w dy  + y * dz&w dy);
b:=subs({x=0,y=0},a);
c:=helmsimp(b);
ans:=DifferentialGeometryTools:-zero_form(2);
if ans <> c then RETURN(false) fi;


a:=evalV(x*dx &t dy  + y * dz&t dy);
b:=subs({x=0,y=0},a);
c:=helmsimp(b);
ans:=DifferentialGeometryTools:-zero_tens([[cov_bas,cov_bas],[]]);
if ans <> c then RETURN(false) fi;
true;

end:



#---------------------------------------------



Check[DifferentialGeometry:-remove_frame]:=proc()

coord_init([x],M1);
coord_init([y],[u],M2);

remove_frame(M1);
remove_frame(M2);




true;
end:

#-------------------------
Check[DifferentialGeometry:-evalV]:=proc()
local a,ans;
coord_init([x,y,z], DGChk);
a:=evalV(y*D_x + z*D_z);
ans:=_VESSIOT([[vect, DGChk, []], [[[1], y], [[3], z]]]);
if   a<> ans then RETURN(false) fi;

a:=evalV( dx &w dy  - dx &w dz);
ans:=_VESSIOT([[form, DGChk, 2], [[[1, 2], 1], [[1, 3], -1]]]);
if   a<> ans then RETURN(false) fi;

a:=evalV(y*D_x&t dx  + z*D_z&t dz);
ans:=_VESSIOT([[tensor, DGChk, [[con_bas, cov_bas], []]], [[[1, 1], y], [[3, 3], z]]]);

if   a<> ans then RETURN(false) fi;
true; 

end:








#---------------------------

Check[DifferentialGeometry:-compose_transforms]:= proc() 

local phi,psi, Phi,Psi;

coord_init([x,y,z],M1);
coord_init([t],M2);
coord_init([u,v],M3);

phi:=transform(M2,M1,[x=t,y=t^2, z=t^3]);
psi:=transform(M1,M3,[u=x^2+y^2,v= x*z]);
Phi:=compose_transforms(psi,phi);
Psi:=[[helm_map, [[M2, 0], [M3, 0]], [], [Matrix([[2*t+4*t^3], [4*t^3]])]], [[t^2+t^4, u], [t^4, v]]];
if  not EqualTransform(Phi,  _VESSIOT(Psi))  then RETURN(false) fi;

true;

end:

#----------------------------

Check[DifferentialGeometry:-dual_basis]:=proc()

local F,Omega, ans;
coord_init([x,y,z],DGChk);

Omega:=[dx,dy,dz];
F:=dual_basis(Omega);
ans:=[D_x,D_y,D_z];
if F <> ans then RETURN(false) fi;

F:=[D_x, D_y,D_z];
Omega:=dual_basis(F);
ans:=[dx,dy,dz];
if Omega <> ans then RETURN(false) fi;

F:=evalV([D_x +3*D_y -D_z,D_x+D_y+2*D_z,D_y-D_z]);
Omega:=dual_basis(F);
ans:=evalV([3*dx-dy-dz, -2*dx+dy+dz, -7*dx+3*dy+2*dz]);
if Omega <> ans then RETURN(false) fi;

true;
end:






#----------------------------
Check[DifferentialGeometry:-ext_d]:=proc () 

local alpha,beta,delta;

coord_init([x,y,z],DGChk);

alpha:= dy;
beta:= 0 &mult (dx &wedge dy);
delta:=ext_d(alpha);
if  beta <> delta then RETURN(false) fi;

alpha:=evalV(x*sin(z)*dx -z*cos(y)*dy + y*exp(x)*dz);
beta:=evalV((y*exp(x)-x*cos(z))*dx &w dz+(exp(x)+cos(y))*dy &w dz); 
delta:=ext_d(alpha):
if delta <> beta then RETURN(false)fi;

alpha:= evalV(log(z)*dx &w dy + log(x)*dy&w dz);
beta:=evalV((z+x)/x/z*dx &w dy &w dz); 
delta:=ext_d(alpha);
if  delta<> beta then RETURN(false) fi;

true:
end:

#--------------------------------------------

Check[homotopy_ext_d]:=proc()
local  alpha,beta,delta;

coord_init([x,y,z],GDChk);
alpha:=evalV(x*dx&w dy);
beta:=homotopy_ext_d(alpha);
delta:=evalV(-1/3*x*y*dx +1/3*x^2*dy);
if beta<>delta then RETURN(false) fi;

alpha:=evalV(x*dx&w dy);
beta:=homotopy_ext_d(alpha,path=straightline,lineparameters=[0,1,[x=1,y=2,z=3]]);
delta:=evalV((-1/3*x*y+2/3*x-1/6*y+1/3)*dx+(1/3*x^2-1/6*x-1/6)*dy);
if beta<>delta then RETURN(false) fi;

alpha:=evalV(x*dx&w dy);
beta:=homotopy_ext_d(alpha, path=zigzag);
delta:=evalV(-x*y*dx);
if beta<>delta then RETURN(false) fi;


alpha:=evalV(x*dx&w dy);
beta:=homotopy_ext_d(alpha, path=zigzag,variableorder=[y,x,z],initialpoint=[x=2,y=3,z=5]);
delta:=evalV((1/2*x^2-9/2)*dy);
if beta<>delta then RETURN(false) fi;

alpha:=evalV( 1/x^2 *dx);
beta:=homotopy_ext_d(alpha, lineparameters=[1,infinity,[x=0,y=0,z=0]]);
delta:=evalV(1/x*DifferentialGeometryTools:-scalar_form() );
if beta<>delta then RETURN(false) fi;

alpha:=evalV( 1/x *dx);
beta:=homotopy_ext_d(alpha,lineparameters=[0,1,[x=1,y=0,z=0]]);
delta:=evalV(ln(x)*DifferentialGeometryTools:-scalar_form() );
if beta<>delta then RETURN(false) fi;
true;

alpha:=evalV( x*dy&w dz - y*dx&w dz -2*z*dx &w dy);
beta:=homotopy_ext_d(alpha);
delta:=evalV(z*y*dx-x*z*dy);
if beta<>delta then RETURN(false) fi;

alpha:=evalV( x*dy&w dz - y*dx&w dz -2*z*dx &w dy);
beta:=homotopy_ext_d(alpha,path=zigzag,variableorder=[y,z,x]); 
delta:=evalV(-2*x*z*dy-x*y*dz);
if beta<>delta then RETURN(false) fi;

alpha:=evalV(x^2*exp(y)*sin(w)*dx &w dy &w dz);
beta:=homotopy_ext_d(alpha,path=zigzag,variableorder=[x,y,z]); 
delta:=evalV(x^2*exp(y)*sin(w)*z*dx &w dy);
if beta<>delta then RETURN(false) fi;

true:
end:















#--------------------------------------------

Check[DifferentialGeometry:-hook]:=proc()

local X,X1,X2,alpha,beta, gamma;
coord_init([x,y,z],DGchk);

X:= D_x;
alpha:= evalV(4*dx + dy - dz):
beta:=hook(X,alpha);
gamma:=DifferentialGeometryTools:-scalar_form(4);
if gamma <> beta then RETURN(false) fi;

X:=evalV(x*D_x -z*D_y +y*D_z);
alpha:= evalV(4*dx&w dy  + 2*dy&w dz  - 3*dx &w dz):
beta:=hook(X,alpha);
gamma:=evalV((3*y+4*z)*dx+(-2*y+4*x)*dy+(-2*z-3*x)*dz);
if gamma <> beta then RETURN(false) fi;

X1:=D_x;
X2:=D_z;
alpha:=dx &wedge dy &wedge dz;
beta:=hook([X1,X2],alpha);
gamma:=evalV(-dy);
if gamma <> beta then RETURN(false) fi;

true;
end:



#-----------------------------------------------

Check[DifferentialGeometry:-inverse_transform]:=proc()  

local phi,psi, Phi;

coord_init([x,y,z],M1);

phi:=transform(M1,M1,[x=x+y,y=2*x+y+z, z=x-y-3*z]);
psi:=inverse_transform(phi);
Phi:=[[helm_map, [[M1, 0], [M1, 0]], [], [Matrix([[-2/5, 3/5, 1/5], [7/5, -3/5, -1/5], [-3/5, 2/5, -1/5]])]], [[-2/5*x+3/5*y+1/5*z, x], [-3/5*y+7/5*x-1/5*z, y], [-1/5*z-3/5*x+2/5*y, z]]];
if  not EqualTransform(psi,  _VESSIOT(Phi))  then RETURN(false) fi;

true;
end;

#-----------------------------------------------
Check[DifferentialGeometry:-Lie_bracket]:=proc() 

local X,Y,Z,W;

coord_init([x,y,z],DGChk);
 
X:=D_x; 
Y:=D_y;
Z:=0 &mult D_x;
W:=Lie_bracket(X,Y);
if W=Z then else RETURN(false) fi;

X:=evalV(x*y*D_x +x^2*D_y +z^2*D_z); 
Y:=evalV(y^2*D_y -z^2*D_x -x^2*D_z);
Z:=evalV((z^2*y-y^2*x-2*z^3)*D_x+(2*z^2*x+2*x^2*y)*D_y+(2*x^2*z-2*x^2*y)*D_z);
W:=Lie_bracket(X,Y);
if W=Z then else RETURN(false) fi;

true

end:

#-----------------------------------------------------------------


Check[DifferentialGeometry:-Lie_derivative]:=proc() 

local X, alpha,beta, delta;

coord_init([x,y,z]);


X:=x*D_x;
alpha:=dx;
beta:=alpha;
delta:=Lie_derivative(X,alpha);
if delta<> beta then RETURN(false) fi;

X:=x*D_x;
alpha:=D_x;
beta:=evalV((-1)*D_x);
delta:=Lie_derivative(X,alpha);
if delta<> beta then RETURN(false) fi;

X:=evalV(y*D_x);
alpha:=evalV(x*dy&t dx &t D_y);
beta:=evalV(-x*dy &t dx &t D_x +y*dy &t dx &t D_y+x*dy &t dy &t D_y);
delta:=Lie_derivative(X,alpha);
if delta<> beta then RETURN(false) fi;

X:=evalV(1/x*D_x + 1/y*D_y +1/z*D_z);
alpha:= x*y*z;
beta:=(y^2*z^2+x^2*z^2+x^2*y^2)/(x*y*z);
delta:=Lie_derivative(X,alpha);
if delta<> beta then RETURN(false) fi;

X:=evalV(x^2*D_x + y^2*D_y + z^2*D_z);
alpha:= DifferentialGeometryTools:-vol_form();
beta:=(2*x+2*y +2*z) &mult DifferentialGeometryTools:-vol_form();
delta:=Lie_derivative(X,alpha);
if delta<> beta then RETURN(false) fi;

X:=evalV(x^2*D_x + y^2*D_y + z^2*D_z);
alpha:=zero_tens([[cov_bas,con_bas,cov_bas],[]]);
# ADD CHECK FOR CONNECTIONS


true;
end:

#------------------------------------------------


Check[DifferentialGeometry:-linear_combo]:=proc() 

local a,B,C,L,TF;

coord_init([x,y,z],DGChk);
a:=dx;
B:=[dx,dy,dz];
C:=[1,0,0];
L:=linear_combo(a,B);
if L<>C then RETURN(false) fi; 
TF:=linear_combo(a,B,check=true); 
if TF <> true then RETURN(false) fi;


a:=evalV(dx &t D_y - x * dy&t D_z);
B:=evalV([dx &t D_x, dx &t D_y, dy &t D_x, dy&t D_z]);
C:=[0,1,0,-x];
L:=linear_combo(a,B);
if L<>C then RETURN(false) fi; 
TF:=linear_combo(a,B,check=true); 
if TF <> true then RETURN(false) fi;


a:=evalV( z*dx&w dy + y*dz&w dx + x *dy &w dz); 
B:=evalV([dx&w dy, dx&w dy  + dy &w dz]);L:=linear_combo(a,B);
C:=[];
if L<>C then RETURN(false) fi; 
TF:=linear_combo(a,B,check=true); 
if TF <> false then RETURN(false)fi;


a:=evalV([ D_x +2*D_y,  3*D_y +4*D_z]);
B:=evalV([D_x,D_y,D_z]);
C:=[[1,2,0],[0,3,4]];
L:=linear_combo(a,B);
if L<>C then RETURN(false) fi; 
TF:=linear_combo(a,B,check=true); 
if TF <> true then RETURN(false) fi;


a:=Matrix([[x,y],[0,z]]);
B:=[Matrix([[1,0],[0,0]]), Matrix([[0,1],[0,0]]),Matrix([[1,0],[0,1]])];
C:=[x-z,y,z]:
L:=linear_combo(a,B);
if L<>C then RETURN(false) fi; 
TF:=linear_combo(a,B,check=true); 
if TF <> true then RETURN(false) fi;


a:=[Matrix([[0,1],[0,0]]),    Matrix([[0,0],[1,0]])];
B:=[Matrix([[1,0],[0,0]]), Matrix([[0,1],[0,0]]),Matrix([[1,0],[0,1]])];
C:=[]:
L:=linear_combo(a,B);
if L<>C then RETURN(false) fi; 
TF:=linear_combo(a,B,check=true); 
if TF <> false then RETURN(false) fi;

true;
end:

#----------------------------------


Check[DifferentialGeometry:-pullback]:=proc()

local  phi, inv_phi,alpha,beta,gamma;

coord_init([x,y,z],M1);
coord_init([t],M2);
coord_init([u,v],M3);
coord_init([r,theta],M4):

phi:=transform(M1,M3,[u=x,v=y]);
alpha:=evalV(du);
beta:=pullback(phi,alpha);
gamma:=evalV(dx);
if beta<>gamma then RETURN(false)  fi;

phi:=transform(M2,M1,[x=t^2,y=t^3 ,z=t^4]);
alpha:=evalV(x*dx +y*dy + z*dz);
beta:=pullback(phi,alpha);
gamma:=evalV( (2*t^3 + 3*t^5 + 4*t^7)*dt);
if beta<>gamma then RETURN(false)  fi;

phi:=transform(M3,M1,[x= sin(u)*cos(v),y=sin(u)*sin(v) ,z=cos(u)]);
alpha:=hook(x*D_x +y*D_y +z*D_z, dx&w dy &w dz);
beta:=pullback(phi,alpha);
gamma:=evalV(sin(u)*du &w dv);
if beta<>gamma then RETURN(false)  fi;
true;

end:

#----------------------------------


Check[DifferentialGeometry:-pushforward]:=proc()

local  phi, inv_phi,X,Y,Z;

coord_init([x,y,z],M1);
coord_init([t],M2);
coord_init([u,v],M3);
coord_init([r,theta],M4):

phi:=transform(M1,M3,[u=x,v=y]);
X:=evalV(z*D_x);
Y:=pushforward(phi,X);
Z:=evalV(z*D_u);
if Y<>Z then RETURN(false)  fi;


phi:=transform(M2,M1,[x=cos(t) ,y=sin(t), z=t^2+1]);
X:=evalV(D_t);
Y:=pushforward(phi,X);
Z:=evalV(-sin(t)*D_x +cos(t)*D_y +2*t*D_z );
if Y<>Z then RETURN(false)  fi;

phi:=transform(M4,M3,[u=r*cos(theta) ,v=r*sin(theta)]);_EnvExplict:=true;
inv_phi:=inverse_transform(phi);
X:= r*D_r;
Y:=pushforward(phi,inv_phi,X);
Z:=evalV(u*D_u +v*D_v);
if Y<>Z then RETURN(false)  fi;

X:=evalV(u*D_v -v*D_u);
Y:=pushforward(inv_phi,phi,X);
Z:=evalV(D_theta);
if Y<>Z then RETURN(false)  fi;

true;

end:


#-----------------------------------


Check[DifferentialGeometry:-pullback_vect]:=proc()

local X,Y,phi,ans;
coord_init([u,v],M1);
coord_init([x,y,z],M2);

phi:=transform(M1,M2, [x=u, y=v, z=0]);
X:=evalV(x^2*D_x - y*x *D_y  +  z*D_z);
Y:=pullback_vect(phi,X,[D_u,D_v]);
ans:=evalV(u^2*D_u - u*v *D_v);
if ans <> Y then RETURN(false) fi;


phi:=transform(M1,M2, [x=cos(u)*sin(v), y=sin(u)*sin(v), z=cos(v)]);
X:=evalV(z*y*D_x +z*x*D_y -(2*x*y)*D_z );
Y:=pullback_vect(phi,X);
ans:=evalV(cos(v)*(-1+2*cos(u)^2)*D_u+2*cos(u)*sin(v)*sin(u)*D_v);
if ans <> Y then RETURN(false) fi;

true; 
end:


#-----------------------------------

Check[DifferentialGeometry:-transform]:=proc() 

local phi,psi;


coord_init([x,y,z],M1);
coord_init([t],M2);
coord_init([u,v],M3);


phi:=transform(M2,M1,[x=t,y=t^2, z=t^3]);
psi:=[[helm_map, [[M2, 0], [M1, 0]], [], [Matrix([[1], [2*t],[3*t^2]])]], [[t, x], [t^2, y], [t^3, z]]];
if   not EqualTransform(phi,  _VESSIOT(psi))  then RETURN(false) fi;

phi:=transform(M1,M3,[u=x^2+y^2,v= x*z]);
psi:=[[helm_map, [[M1, 0], [M3, 0]], [], [Matrix([[2*x, 2*y, 0], [z, 0, x]])]], [[x^2+y^2, u], [x*z, v]]];
if  not EqualTransform(phi,  _VESSIOT(psi))  then RETURN(false) fi;

true
end:


#-------------------------------------

Check[DifferentialGeometry:-transform_to_vect]:=proc() 

local phi, X,Y;
coord_init([x,y,z],DGchk);

phi:=transform(DGchk,DGchk, [x=x+t, y=y+2*t, z=z+3*t]);
X:=transform_to_vect(phi,[t]);
Y:=evalV([D_x + 2*D_y + 3*D_z]);
if X<>Y then RETURN(false) fi;


phi:=transform(DGchk,DGchk, [x=t*x, y=t^2*y, z=t^3*z]);
X:=transform_to_vect(phi,[t],initialpoint=[t=1]);
Y:=evalV([x*D_x + 2*y*D_y + 3*z*D_z]);
if X<>Y then RETURN(false) fi;


phi:=transform(DGchk,DGchk, [x=t*cos(s)*x +t*sin(s)*y, y=-t*sin(s)*x + t*cos(s)*y,z=z]);
X:=transform_to_vect(phi,[t,s],initialpoint=[t=1,s=0]);
Y:=evalV([x*D_x +y*D_y, -x*D_y + y*D_x]);
if X<>Y then RETURN(false) fi;

true
end:

#------------------------------------------

Check[DifferentialGeometry:-vect_to_Lie_alg]:=proc() 

local  Gamma, A, B;
coord_init([x,y,z]);

Gamma:=evalV([D_x,D_y, x*D_y]);
A:=vect_to_Lie_alg(Gamma);
B:=[[Lie_alg, Koszul, [3]], [[[1, 3, 2], 1]]];
if A<>B then RETURN(false) fi:
Gamma:=evalV([D_x,D_y, x*D_y]);

A:=vect_to_Lie_alg(Gamma,bob);
B:=[[Lie_alg, bob, [3]], [[[1, 3, 2], 1]]];
if A<>B then RETURN(false) fi:

Gamma:=evalV([D_x,D_y, exp(x)*D_y]);
A:=vect_to_Lie_alg(Gamma,inititalpoints=[[x=0,y=1]]);
B := [[Lie_alg, Koszul, [3]], [[[1, 3, 3], 1]]];
if A<>B then RETURN(false) fi:
true;

end:


#------------------------------------------

Check[DifferentialGeometry:-vect_to_transform]:=proc()

local X,phi, psi;
coord_init([x,y,z],DGChk);

X:=D_x;
phi:=vect_to_transform(X,s);
psi:=transform(DGChk,DGChk,[x=x+s,y=y,z=z]);
if not EqualTransform(phi,psi) then RETURN(false) fi;
 
X:=evalV(x^2*D_x +x*D_y);
phi:=simplify(vect_to_transform(X,s));
psi:=transform(DGChk,DGChk,[x = -x/(s*x-1), y = -ln((s*x-1)/x)+ln(-1/x)+y, z = z]);
if not EqualTransform(phi,psi) then RETURN(false) fi;


X:=evalV(y*D_x -x*D_y +x*y*D_z);
phi:=simplify(vect_to_transform(X,s));
 psi:=transform(DGChk,DGChk,[x = y*sin(s)+x*cos(s), y = -x*sin(s)+y*cos(s), z = x*sin(s)*y*cos(s)-1/2*y^2*cos(s)^2+1/2*y^2+z-1/2*x^2+1/2*x^2*cos(s)^2]);
if not EqualTransform(phi,psi) then RETURN(false) fi;
 true end:


#---------------------------------------


Check[DifferentialGeometry:-v_zip]:=proc()
local a,b,c,ans:
coord_init([x,y,z,w],DGChk);

a:=[x,y,z,w];
b:=[D_x,D_y,D_z, D_w];
c:=v_zip(a,b,plus);
ans:=evalV(x*D_x + y*D_y + z*D_z + w*D_w);
if  ans <> c then RETURN(false) fi;

a:=[1,2,3,4];
b:=evalV([dx &w dy,  dx&w dz,  dy &w dz, dz &w dw]);
c:=v_zip(a,b,plus);
ans:=evalV(dx &w dy + 2*dx&w dz +  3*dy &w dz +4* dz &w dw);
if  ans <> c then RETURN(false) fi;

a:=[1,2,3,4];
b:=evalV([dx &t D_y,  dx&t D_z,  dy &t D_z, dz &t D_w]);
c:=v_zip(a,b,plus);
ans:=evalV(dx &t D_y + 2*dx&t D_z +  3*dy &t D_z +4* dz &t D_w);
if  ans <> c then RETURN(false) fi;

a:=[2,3];
b:=evalV([dx &w dy, dz &w dw]);
c:=v_zip(a,b,wedge);
ans:=evalV(6*dx &w dy &w dz &w dw);
if  ans <> c then RETURN(false) fi;

a:=evalV([dx &t D_y,  dx&t D_z,  dy &t D_z]);
c:=v_zip(a,tensor);
ans:=evalV(dx &t D_y &t dx&t D_z &t  dy &t D_z);
if  ans <> c then RETURN(false) fi;


true; 
end:








#
#-----------------------------------------
#

#Main Procedure

 
BadProcedureList:=[];

for P in ProcedureList do 
	
	if tracy=on then 
		trace(Check[P]) 
	fi;

	TF:=Check[P]();

      if TF=true then print(cat(P,"  OK")) fi;

	untrace(Check[P]);

	if TF=true  then  else  
		BadProcedureList:=[op(BadProcedureList),P] 
	fi;

od;

if BadProcedureList=[] then 
	print(`All Differential Geometry Procedures Gave Correct Answers`) 
else  
	print(`The Following Procedures Failed To Give The Correct Answer`);
	print(BadProcedureList);
fi:

writeline(default,cat(StringTools:-FormatTime("%n%n%c"),"/IA")):
end:





