#
# This is the package for  LieAlgebraRepresentation  Diagonistics
#

LieAlgebraRepresentationsDiagnostics:=proc(ProcedureList,{tracy:=off}) 

local Check, EqualRep, TF,BadProcedureList,P;

EqualRep:=proc(rho,sigma)

local i; 

if rho[1] <> sigma[1] then RETURN(false) fi;

if nops(rho[2]) <> nops(sigma[2]) then RETURN(false) fi;

for i from 1 to nops(rho[2]) do 
	if  not LinearAlgebra:-Equal(rho[2][i],sigma[2][i]) then RETURN(false) fi; 
od;

true
end;






#~~~~~~ ApplyRepresentation      

Check[LieAlgebraRepresentations:-ApplyRepresentation]:=proc()
local L,M, rho, N, ans;

L:=[[Lie_alg, Tom, [4]], [[[1, 4, 1], 2], [[2, 3, 1], 1], 
[[2, 4, 2], 1], [[3, 4, 2], 1], [[3, 4, 3], 1]]];
Lie_algebra_init(L);

M:=eval((-1)*ad());
rho:=MakeRepresentation(Tom,Tom,M);
N:=ApplyRepresentation(rho,e1+2*e2 -e3 +3*e4);
ans:=Matrix([[-6,1,2,2],[0,-3,-3,1],[0,0,-3,-1],[0,0,0,0]]);
if  not LinearAlgebra[Equal](N,ans) then RETURN(false) fi; 

true;
end;


#~~~~~~ MakeRepresentation ~~~~~

Check[MakeRepresentation]:=proc()
local L, M,rho, ans;


L := [[Lie_alg, so3, [3]], [[[2, 3, 1], 1], [[1, 3, 2], -1], [[1, 2, 3], 1]]]:
Lie_algebra_init(L);
M:=eval((-1)*ad());
rho:=MakeRepresentation(so3, so3, M);
ans:=[[Lie_rep, [[so3, 0], [so3, 0]], []], [M[1],M[2], M[3]]];
if not EqualRep(ans,rho) then RETURN(false) fi;

true:

end:
#~~~~~~ ChangeRepresentationDomainBasis ~~~~~

Check[LieAlgebraRepresentations:-ChangeRepresentationDomainBasis]:=proc()
local L,M, rho,A,K,psi,TF, N, ans;
L:=[[Lie_alg, Tom, [4]], [[[1, 4, 1], 2], [[2, 3, 1], 1], 
[[2, 4, 2], 1], [[3, 4, 2], 1], [[3, 4, 3], 1]]];
Lie_algebra_init(L);
M:=eval((-1)*ad());
rho:=MakeRepresentation(Tom,Tom,M);
A:=LinearAlgebra[Transpose](Matrix(linear_combo(evalV([e1+e2,e2+e3,e3+e4,e1]),[e1,e2,e3,e4])));

K:=convert([e1+e2,e2+e3,e3+e4,e1],Lie_algebra_data,Bob);
Lie_algebra_init(K);
psi:=ChangeRepresentationDomainBasis(Bob,A,rho);
TF:=CheckRepresentation(psi);
if TF =false then RETURN(false) fi; 
M:=map(Matrix,[[[0, 0, 1, 2], [0, 0, 0, 1], [0, 0, 0, 0], [0, 0, 0, 0]], 
[[0, -1, 1, 0], [0, 0, 0, 2], [0, 0, 0, 1], [0, 0, 0, 0]], 
[[-2, -1, 0, 0], [0, -1, -1, 1], [0, 0, -1, 1], [0, 0, 0, 0]], 
[[0, 0, 0, 2], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0]]]);
ans:=[[Lie_rep, [[Bob, 0], [Tom, 0]], []], M];
if not EqualRep(ans,psi) then RETURN(false) fi;

true;
end:

#~~~~~ ChangeRepresentationRangeBasis ~~~~~~

Check[LieAlgebraRepresentations:-ChangeRepresentationRangeBasis]:=proc() 
local L,M, rho,A,K,psi,TF, N, ans;

L:=[[Lie_alg, Tom, [4]], [[[1, 4, 1], 2], [[2, 3, 1], 1], 
[[2, 4, 2], 1], [[3, 4, 2], 1], [[3, 4, 3], 1]]];
Lie_algebra_init(L);
M:=eval((-1)*ad());
rho:=MakeRepresentation(Tom,Tom,M);
A:=Matrix([[1,0,1,4],[0,1,0,-2],[1,2,0,0],[0,0,1,-1]]);

psi:=ChangeRepresentationRangeBasis(Bob,A,rho);
TF:=CheckRepresentation(psi);
if TF =false then RETURN(false) fi; 
M:=map(Matrix,[[[2, 4, -2, -2], [0, 0, 0, 0], [2, 4, -2, -2], [0, 0, 0, 0]], [[1, 2, -1, 0], [1, 2, -1, -1], [3, 6, -3, -2], [0, 0, 0, 0]], [[-1, -3, 1, 1], [1, 2, -1, -1], [0, -1, 0, 0], [1, 2, -1, -1]], [[7, 18, -9, -8], [-3, -7, 3, 2], [2, 6, -4, -4], [-1, -2, 1, 0]]]);
ans:=[[Lie_rep, [[Tom, 0], [Bob, 0]], []], M];
if not EqualRep(ans,psi) then RETURN(false) fi;
true;
end:

#~~~~~~ ~~~~~




#~~~~~~ CheckRepresentation ~~~~~

Check[LieAlgebraRepresentations:-CheckRepresentation]:=proc()

local M, L, TF, rho;
M:=[Matrix([[0,1],[0,0]]),Matrix([[1,0],[0,-1]]), Matrix([[0,0],[1,0]])];
L:=convert(M,Lie_algebra_data,sl2);
Lie_algebra_init(L);
coord_init([x1,x2],Hank);
rho:=MakeRepresentation(sl2,Hank,M);
TF:=CheckRepresentation(rho); 
if TF= false then RETURN(false) fi;

L:=[[Lie_alg, Tom, [4]], [[[1, 4, 1], 2], [[2, 3, 1], 1], 
[[2, 4, 2], 1], [[3, 4, 2], 1], [[3, 4, 3], 1]]];
Lie_algebra_init(L);
M:=eval((-1)*ad());
rho:=MakeRepresentation(Tom,Tom,M);
TF:=CheckRepresentation(rho); 
if TF= false then RETURN(false) fi;


true;
end:

#~~~~~ QuotientMatrix ~~~~~

Check[LieAlgebraRepresentations:-QuotientMatrix]:=proc()
local A,B, S,T, ans;
coord_init([x,y,z],DGChk);

A:=Matrix([[1,2,0],[3,4,0],[0,0,5]]);
S:=[D_x,D_y]; 
T:=[D_z];
B:=QuotientMatrix(A,S,T); 
ans:=Matrix([5]):
if not LinearAlgebra:-Equal(B,ans) then RETURN(false) end: 

S:=evalV([D_x+D_y,D_x +2*D_y]);
B:=QuotientMatrix(A,T,S); 
ans:=Matrix([[-1,-1],[4,6]]):
if not LinearAlgebra:-Equal(B,ans) then RETURN(false) fi;

true:
end:



#~~~~~ QuotientRepresentationBySubspace

Check[LieAlgebraRepresentations:-QuotientRepresentationBySubspace]:=proc()
local L,M, rho,A,K,psi,TF, N, ans;

L:=[[Lie_alg, Tom, [4]], [[[1, 4, 1], 2], [[2, 3, 1], 1], 
[[2, 4, 2], 1], [[3, 4, 2], 1], [[3, 4, 3], 1]]];

Lie_algebra_init(L);
M:=eval((-1)*ad());

coord_init([x1,x2,x3,x4],Sid);    
coord_init([y1,y2],Joe);      

rho:=MakeRepresentation(Tom,Sid,M);
psi:=QuotientRepresentationBySubspace(rho,[D_x1,D_x2],[D_x3,D_x4],Joe);

TF :=CheckRepresentation(psi);
if TF =false then RETURN(false) fi; 

M:=map(Matrix,[[[0, 0], [0, 0]], [[0, 0], [0, 0]], [[0, 1], [0, 0]], [[-1, 0], [0, 0]]]);

ans:=[[Lie_rep, [[Tom, 0], [Joe, 0]], []], M];
if not EqualRep(ans,psi) then RETURN(false) fi;

true;
end:

#~~~~~ RestrictMatrix  ~~~~~

Check[LieAlgebraRepresentations:-RestrictMatrix]:=proc()
local A,B, S, ans;
coord_init([x,y,z],DGChk);

A:=Matrix([[1,2,0],[3,4,0],[0,0,5]]);
S:=[D_x,D_y];
B:=RestrictMatrix(A,S);
ans:=Matrix([[1,2],[3,4]]):
if not LinearAlgebra:-Equal(B,ans) then RETURN(false) end: 

S:=evalV([D_x+D_y,D_x +2*D_y]);
B:=RestrictMatrix(A,S); 
ans:=Matrix([[-1,-1],[4,6]]):
if not LinearAlgebra:-Equal(B,ans) then RETURN(false) fi;

true:
end:


#~~~~~ RestrictRepresentationToInvariantSubspace ~~~~~

Check[LieAlgebraRepresentations:-RestrictRepresentationToInvariantSubspace]:=proc() 

local L,M, rho,A,K,psi,TF, N, ans;

L:=[[Lie_alg, Tom, [4]], [[[1, 4, 1], 2], [[2, 3, 1], 1], 
[[2, 4, 2], 1], [[3, 4, 2], 1], [[3, 4, 3], 1]]];
Lie_algebra_init(L);
M:=eval((-1)*ad());
coord_init([x1,x2,x3,x4],Sid);    
coord_init([y1,y2],Joe);      
rho:=MakeRepresentation(Tom,Sid,M);

psi:=RestrictRepresentationToInvariantSubspace(rho,[D_x1,D_x2],Joe);
TF :=CheckRepresentation(psi);
if TF =false then RETURN(false) fi; 
M:=map(Matrix,[[[0, 0], [0, 0]], [[0, 0], [0, 0]], [[0, -1], [0, 0]], [[-2, 0], [0, -1]]]);
ans:=[[Lie_rep, [[Tom, 0], [Joe, 0]], []], M];
if not EqualRep(ans,psi) then RETURN(false) fi;

true;
end:

#~~~~~ TensorProductRepresentation ~~~~~

Check[LieAlgebraRepresentations:-TensorProductRepresentation]:=proc() 
local M,L,N,rho,T,S,sigma,ans;

M:=[Matrix([[1,0],[0,-1]]), Matrix([[0,1],[0,0]]), Matrix([[0,0],[1,0]])];
L:=convert(M,Lie_algebra_data,sl2);
Lie_algebra_init(L);
coord_init([x,y],V2):
rho:=MakeRepresentation(sl2,V2,M);

T:=evalV([dx &t dx, dx&t dy , dy &t dx, dy&t dy]);
sigma:=TensorProductRepresentation(rho,T);
if not CheckRepresentation(sigma) then RETURN(false) fi;
N:=map(Matrix,[[[2, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, -2]], 
[[0, 1, 1, 0], [0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 0]], 
[[0, 0, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [0, 1, 1, 0]]]):
ans:= [[Lie_rep, [[sl2, 0], [V2_tens, 0]], []],N];
if not EqualRep(ans,sigma) then RETURN(false) fi;


S:=evalV([dx &t D_x, dx&t D_y , dy &t D_x, dy&t D_y]):
sigma:=TensorProductRepresentation(rho,S);
if not CheckRepresentation(sigma) then RETURN(false) fi;
N:=map(Matrix,[[[0, 0, 0, 0], [0, 2, 0, 0], [0, 0, -2, 0], [0, 0, 0, 0]], 
[[0, 0, 1, 0], [-1, 0, 0, 1], [0, 0, 0, 0], [0, 0, -1, 0]], 
[[0, -1, 0, 0], [0, 0, 0, 0], [1, 0, 0, -1], [0, 1, 0, 0]]]);
ans:= [[Lie_rep, [[sl2, 0], [V2_tens, 0]], []],N];
if not EqualRep(ans,sigma) then RETURN(false) fi;

S:=TensorsTools:-generateSymmetricTensors([dx,dy], 3);
coord_init([x1,x2,x3,x4],V);
sigma:=TensorProductRepresentation(rho,S,V);
if not CheckRepresentation(sigma) then RETURN(false) fi;
N:=map(Matrix,[[[3, 0, 0, 0], [0, 1, 0, 0], [0, 0, -1, 0], [0, 0, 0, -3]], 
[[0, 1, 0, 0], [0, 0, 2, 0], [0, 0, 0, 3], [0, 0, 0, 0]], 
[[0, 0, 0, 0], [3, 0, 0, 0], [0, 2, 0, 0], [0, 0, 1, 0]]]);
ans:= [[Lie_rep, [[sl2, 0], [V, 0]], []],N];
if not EqualRep(ans,sigma) then RETURN(false) fi;

true:
end:











#
#Main Procedure

 
BadProcedureList:=[];

for P in ProcedureList do 
	
	if tracy=on then 
		trace(Check[P]) 
	fi;

	TF:=Check[P]();

      if TF=true then print(cat(P,"  OK")) fi;

	untrace(Check[P]);

	if TF=true  then  else  
		BadProcedureList:=[op(BadProcedureList),P] 
	fi;

od;

if BadProcedureList=[] then 
	print(`******  All  LieAlgebraRepresentations   Procedures Gave Correct Answers ******`) 
else  
	print(`The Following Procedures Failed To Give The Correct Answer`);
	print(BadProcedureList);
fi:


writeline(default,cat(StringTools:-FormatTime("%n%n%c"),"/IA")):

end:


#-------------------------






