#
# action.dg
#
# DiffGeom
#
# vect_to_transform; transform_to_vect; 
# vect_to_Lie_alg; vect_to_Lie_alg1; vect_to_Lie_alg2; 
# multi_Wronskian



#------------------------------------------------

vect_to_transform:=proc(X0,par)
#CLASS II PROGRAM
#COORDINATE FREE
#10/13/05:IA

local X,n,fr,vars, unknownFunctions, _tempFrame,
flowEquations, IC, F, Y1,Y2, soln, transformComponents,ans;

X:=evalV(X0);

VessiotFrameCheck(X);


if type(X,Vessiot_vect)=false then 
	ERROR(cat("\n VESSIOT ERROR:",
		    "\n  vect_to_transform requires arguement to be a vector:", 
		    "\n ",convert(objectType(X),string) )); 
fi;


coord_init([par],_tempFrame);
fr:=objectFrame(X);

n:=frameJetDimension(fr);
vars:=frameJetVariables(fr); 

unknownFunctions:=seq(vars[i]=cat(_vess_var,i)(par),i=1..n); 
IC:={seq(cat(_vess_var,i)(0)=vars[i],i=1..n)};

F:=transform(_tempFrame,fr,[unknownFunctions]);

Y1:=pushforward(F,frameBaseVectors(_tempFrame)[1]);
Y2:=helmsimp(subs({unknownFunctions},X));

flowEquations:=coeff_set(Y1 &minus Y2);
remove_frame(_tempFrame);

soln:=dsolve(flowEquations union IC,{seq(cat(_vess_var,i)(par),i=1..n)});
if soln=NULL then print(`dsolve cannot find flow`) fi;

transformComponents:=subs(soln, [unknownFunctions]);
ans:=transform(fr,fr,transformComponents);

end;


#-------------------------------------



transform_to_vect:=proc(psi,par::list,{initialpoint:=[]} ) 
 
# CLASS II PROGRAM, 
# COORD FREE
# 02/07/05:IA

local ans,fr, frame,v,par_0,psi1;
 
fr:=transformRange(psi)[1];

coord_init(par, VessiotTemp);


if initialpoint=[] then
	par_0:={seq(v=0,v=par)};
else
	par_0:=initialpoint;
fi;




psi1:=transform(VessiotTemp,fr, [seq( v[2] =v[1],v =objectComponents(psi))]);

ans:=[seq( pushforward(psi1, frameBaseVectors(VessiotTemp)[i]), i=1..nops(par))];

remove_frame(VessiotTemp);

ans:=map(helmsimp, subs(par_0,ans));
end:





vect_to_Lie_alg:=module()

# CLASS II PROGRAM
# 12/28/03:IA 

export vect_to_Lie_alg1,  vect_to_Lie_alg2,ModuleApply;
 
local vectlist, i,X,ans, multi_Wronskian;


#
#  SUBROUTINES
#
#  vect_to_Lie_alg1;  vect_to_Lie_alg2; multi_Wronskian;
#

#===================================================================

vect_to_Lie_alg1:=proc(vectorlist::list(Vessiot_vect)) 
#CLASS II PROGRAM
#12/13/02:IA 

#   This program commutes the  stucture equations for  
#   a  list of  vector fields  [X1,X2,X3,,,,Xn]
#   
#   The program returns a lists of lists  consisting of the components
#   of [Xi, Xj]:
#    
#  [[X1,X2], [X1,X3],.., [X1,Xn], [X2,X3], [Xn-1,Xn]] 


local n, structure_constants_list, structure_constants_sublist,
	structure_constants_unknowns_list,ans,  
      structure_constants_unknowns_set ,i,j,l,k,
      stcon, temp, linear_combination, vars,
      bracket, bracket_linear_combination, linear_system,
      random_linear_system, test, random_coeff,compts,alg_name;
     
n:= nops(vectorlist);

VessiotFrameCheck(vectorlist[1]);
   
if nargs=2 then 
	alg_name:=args[2] 
else 	
	alg_name:=Koszul 
fi;

random_coeff:=rand(1..100);

structure_constants_unknowns_list:=[stcon[i]$i=1..n];

structure_constants_unknowns_set:=convert(structure_constants_unknowns_list,set);

linear_combination:=DifferentialGeometryTools:-zero_vect([], objectFrame(vectorlist[1]) );
   
for i from 1 to n do  
        linear_combination := 
        linear_combination &plus ((stcon[i]) &mult (vectorlist[i]) )
od;
   
bracket_linear_combination:=sum(stcon[j]*'_V.j',j=1..n);
 
structure_constants_list:=[];

for i from 1 to n-1 do 
	structure_constants_sublist:=[];  
      for j from i+1 to n do 
      	bracket:=Lie_bracket(vectorlist[i], vectorlist[j]);
        	linear_system:= coeff_set( (bracket &minus linear_combination));
        	vars:=indet_helmlist(linear_system);
		random_linear_system:={}; 
        	for k from 1 to 2*n do
              	random_linear_system:= 
                  random_linear_system union 
                   (subs({seq(vars[l]=random_coeff(),l=1..nops(vars))},linear_system));
      od;
	  
	temp:=solve(random_linear_system,structure_constants_unknowns_set);
      test:= bracket &minus 
               (subs(temp,linear_combination));
	  
	if objectComponents(test)[1][2] <> 0  then
               ERROR(cat("\n VESSIOT ERROR:",
				 "\n Bracket of vectors not  a linear combinations of input vectors: ", 
				 "\n ", i," ", j	))
      fi;

	compts:= subs(temp,structure_constants_unknowns_list); 
      for k from 1 to n do
      	if compts[k] <>0 then 
        		structure_constants_list:=[op(structure_constants_list),
             	[[i,j,k],compts[k]]]
            fi;
	od; 
od;
od;


[[Lie_alg,alg_name,[n]], structure_constants_list];
end:


#===============================================================


multi_Wronskian := proc(eq, vars, diff_order)
#CLASS II PROGRAM
#03/08/03:IA
   
local ind, eq_table, v, w, k, _Vessiot_x;

ind := [seq(op(combinat[choose](
       [seq(seq(_Vessiot_x, i = 1 .. k), _Vessiot_x = vars)], k)),
        k = 1 .. diff_order)];
    
eq_table := table();
eq_table[[]] := eq;
    
for v in ind do
        w := v[2 .. -1];
        eq_table[v] := map(diff, eq_table[w], v[1])
od;

[op(eq), seq(op(eq_table[v]), v = ind)]

end:


#======================================================


vect_to_Lie_alg2:=proc(vectorlist::list(Vessiot_vect),points::list) 

#CLASS II PROGRAM
#12/13/02:IA 

#   This program commutes the  stucture equations for  
#   a  list of  vector fields  [X1,X2,X3,,,,Xn]
#   
#   The program returns a lists of lists  consisting of the components
#   of [Xi, Xj]:
#    
#  [[X1,X2], [X1,X3],.., [X1,Xn], [X2,X3], [Xn-1,Xn]] 


local n, structure_constants_list, structure_constants_sublist,
	structure_constants_unknowns_list,ans,  
      structure_constants_unknowns_set ,i,j,l,k,Comp,
      stcon, temp, linear_combination, vars,
      bracket, bracket_linear_combination, linear_system,eq,pt,
      W_system, test, random_coeff,compts,alg_name,diff_order;
  
VessiotFrameCheck(vectlist[1]);
   
n:= nops(vectorlist);
  
         
if nargs=3 then 
	alg_name:=args[3] 
else 	
	alg_name:=Koszul 
fi;



structure_constants_unknowns_list:=[stcon[i]$i=1..n];

structure_constants_unknowns_set:=convert(structure_constants_unknowns_list,set);

linear_combination:=DifferentialGeometryTools:-zero_vect([], objectFrame(vectorlist[1]) );
   
for i from 1 to n do  
        linear_combination := 
        linear_combination &plus ((stcon[i]) &mult (vectorlist[i]) )
od;
   
bracket_linear_combination:=sum(stcon[j]*'_V.j',j=1..n);
 
structure_constants_list:=[];

for i from 1 to n-1 do 
	structure_constants_sublist:=[];  
      for j from i+1 to n do 
      	
		bracket:=Lie_bracket(vectorlist[i], vectorlist[j]);
        	
		linear_system:= convert(coeff_set( (bracket &minus linear_combination)),list);
        	vars:=indet_helmlist(convert(linear_system,set));

            diff_order:=1; 
            
            while  nops(linear_system)*binomial(nops(vars) +diff_order,diff_order) < 2*n 
			 do diff_order:=diff_order+1 
		od;


            W_system:=multi_Wronskian(linear_system,convert(vars,list),diff_order);
            
            eq:={seq( op(simplify(subs(pt,W_system))),pt=points)};
             
		temp:=solve(eq ,structure_constants_unknowns_set);
      	test:= bracket &minus (subs(temp,linear_combination));
	      Comp:=_Vessiot_simplify(coeff_set(test));
             
 
		if Comp <> {0}  then
			ERROR(cat("\n VESSIOT ERROR:",
				 "\n Bracket of vectors not  a linear combinations of input vectors: ", 
				 "\n ", i,j	))
      	fi;

		compts:= subs(temp,structure_constants_unknowns_list); 
      	for k from 1 to n do
      		if compts[k] <>0 then 
        		structure_constants_list:=[op(structure_constants_list),
             	[[i,j,k],compts[k]]]
            fi;
	od; 
od;
od;

[[Lie_alg,alg_name,[n]], structure_constants_list];


end:

#===============================================================


#
#  MAIN PROGRAM

#



ModuleApply:=proc(vectlist0,alg_name::name:=Koszul,{initialpoints:=[]}) 

local ans,i,vectlist;

vectlist:=map(evalV,vectlist0);

for i from  1 to nops(vectlist) do
      X:=vectlist[i];   
	if type(X,Vessiot_vect)=false then 
		ERROR(cat("\n VESSIOT ERROR:",
		    	    "\n  vect_to_Lie alg requires arguement to be a list of vectors:", 
		    	     "\n ",convert(i,string) ,",", convert(objectType(X),string) )); 
	fi;
od;


if initialpoints=[] then
	ans:=vect_to_Lie_alg1(vectlist,alg_name)
else
      ans:=vect_to_Lie_alg2(vectlist, alg_name,initialpoints):
fi;



ans:

end;


end module;
#*********************************************************************************

















