#
# LieDerivative.ms
#
# DifferentialGeometry Package
#
# Lie_derivative 
#


Lie_derivative:=module() 

# CLASS II PROGRAM
# 08/26/1991:CH
# 12/17/2002:IA         


local LieDerivativeRuleTable, LieDerivativeProcedureTable,  ans, vectr,geomObjField,frame, vec,objType,Lie_conn,indxType,
newdegree, term,vecType;

export  addRule, addProcedure, ModuleApply;



ModuleApply:=proc(vectr0,geomObjField0)

local procType;    

vectr:=evalV(vectr0):
geomObjField:=evalV(geomObjField0):

if type(vectr,Vessiot_vect)=false then 
	ERROR(cat("\n VESSIOT ERROR:",
		    "\n  Lie derivative requires first arguement to be a vector field,", 
		    "\n ",convert(objectType(vectr),string) )); 
fi;

if type(geomObjField,{Vessiot_scalar,Vessiot_bftv,Vessiot_connection})=false then 
	ERROR(cat("\n VESSIOT ERROR:",
		    "\n  Lie_derivative requires second arguement to be one of:",  
		    "\n  Maple expression, biform,form,tensor, vector:", 
		    "\n ",convert(objectType(geomObjField),string) )); 
fi;


if  type(geomObjField, Vessiot_scalar)=false and
	objectFrame(vectr)<> objectFrame(geomObjField) then  
    	ERROR(cat("\n VESSIOT  ERROR:",
              "\n frame of vector must equal frame of geometric object field",
		  "\n ", convert(objectFrame(vectr),string),  
			   convert(objectFrame(geomObjField),string) ))
fi;


#ERROR(cat("\n VESSIOT ERROR:",  
#          "\n Invalid arguements.  Second arguement must be one of:", 
#	    "\n, expression, vector, form, biform, tensor, connection.",
#         "\n If  second arguement is  biform, then  first arguement must be a vector of type:",
#          "\n projectable, evol, tot.") ); 

VessiotFrameCheck(vectr):

frame := frameProtocol();

objType:=objectType(geomObjField);

if objType = NULL then 
	objType:=Vessiot_scalar
else
   objType:=cat(Vessiot_, objType);
fi;

procType:=LieDerivativeRuleTable[objType](vectr,geomObjField);

LieDerivativeProcedureTable[op(procType)](vectr,geomObjField);

end:


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


addRule:=proc(r,p) LieDerivativeRuleTable[op(r)]:= p end:


#~~~~~~~~~~~~~~~

addProcedure:=proc(r,p) LieDerivativeProcedureTable[op(r)]:= p end:


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#  Vessiot_scalar rules
#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



LieDerivativeRuleTable[Vessiot_scalar]:=proc(X,f) local fr;
	fr:=frameProtocol(); 
	[Vessiot_scalar,fr]
end;

LieDerivativeProcedureTable[Vessiot_scalar,vst_euc]:=proc(vectr,expr)  	
	#CLASS I PROGRAM
	#08/26/1991:CH
	#12/17/2002:IA         

	local ans, term;

	ans := 0;

	for term in op(vectr)[2] do
		ans := ans + term[2]*diff(expr, index_to_jet_label(term[1][1]))
	od;

	_Vessiot_simplify(ans)
end:



LieDerivativeProcedureTable[Vessiot_scalar,repere_mobile]:=proc(v,exp) 
	# Class I program
	# 01/12/03       

	local  ans,fr,varlist,var,term;
	global _Vessiot_current_frame_name;

	fr:=_Vessiot_current_frame_name;
	varlist:=indet_helmlist(exp):

	ans:=0;

	for  var in varlist do 
     		for term in op(v)[2]  do
     			ans:= ans +   term[2] *
     			_Vessiot_frame_data[fr][4][1][term[1][1],tuple_to_index(var)]*diff(exp,var)
     		od;
	od;

ans

end:




#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#  Vessiot_form rules
#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

LieDerivativeRuleTable[Vessiot_form]:=proc(vectr,geomObjField)
      local fr;
	fr:=frameProtocol(); 
      [Vessiot_form,fr];	
end:


LieDerivativeProcedureTable[Vessiot_form,vst_euc]:=proc(vectr,geomObjField)

local PHI, Z;

PHI:=proc(expr, flag,vec)


	#CLASS I PROGRAM
	#08/26/1991:CH
	#12/17/2002:IA         

	local vectr, i, ans, vars, v,Lie_f;

	vectr:=op(vec)[2]; 

	if flag='bftv' then
		i:=1;
		while (i<nops(vectr)) and (vectr[i][1][1] < expr) do i:=i+1  
		od;

      	if vectr[i][1][1] = expr then
           		ans := [];
    	    		vars := indet_helmlist(vectr[i][2]);
    		
			for v in vars do
        			ans := [op(ans), [[tuple_to_index(v)], diff(vectr[i][2], v)]]
    			od;
    		
		Lie_f:=_VESSIOT([[form, _Vessiot_current_frame_name, 1], ans])
      
		else
           	
		Lie_f := DifferentialGeometryTools:-zero_form(1)
      	fi;

	elif  flag='coeff'  then
       	ans:= [[[], Lie_derivative(vec, expr)] ];
       	Lie_f:=_VESSIOT([[form, _Vessiot_current_frame_name,0], ans])

	else
		ERROR(`VESSIOT ERROR: insufficient input`)
	fi;

	Lie_f
end:


newdegree:=formDegree(geomObjField);

Z:= DifferentialGeometryTools:-zero_form(newdegree);

derivation[formBiform](PHI,deriv,geomObjField,Z, vectr);


end:




LieDerivativeProcedureTable[Vessiot_form,repere_mobile]:=proc(vectr,geomObjField)
	
local PHI, Z;

PHI:=proc(expr, flag,v)

	# Class I program
	# 01/12/03       

	local d_ans,fr,A,B;

	fr:=_Vessiot_current_frame_name;

	if flag = 'coeff' then
         	d_ans :=  _VESSIOT([[form, fr,0], [ [ [],Lie_derivative(v, expr)]]]);
	fi;

	if flag='bftv' then
		A:=  HOOK[form,deriv](v, _Vessiot_frame_data[fr][3][1][expr]);
		B:=  DifferentialGeometry:-ext_d(HOOK[form,deriv](v, DifferentialGeometryTools:-Form(expr)));
		d_ans:=DiffGeomMain:-Plus(A,B);
        
	fi;

	d_ans:

end:




newdegree:=formDegree(geomObjField);

Z:= DifferentialGeometryTools:-zero_form(newdegree);

derivation[formBiform](PHI,deriv,geomObjField,Z, vectr);
end:


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#  Vessiot_vect rules
#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


LieDerivativeRuleTable[Vessiot_vect] := proc()   
Vessiot_vect end:


LieDerivativeProcedureTable[Vessiot_vect]:=proc(vec1,vec2) Lie_bracket(vec1,vec2) end:

#---------------------------------------

end module:


















