#
#
#
#


`convert/Vessiot_connection`:=proc(a_tensor::Vessiot_tensor)
# CLASS I PROGRAM
# 05/01/99:CH
# 01/14/03:IA


#    assigns the type connection to a tensor.
#    a_tensor: a type [cov_bas,con_bas,cov_bas] or type [cov_hor,con_vrt,cov_vrt] tensor.
#     *****  ALSO SHOULD ALLOW FOR A CONNECTION ON  A VECTOR BUNDLE. ****


local ans, fr,tensIndType,tensDensType; 

fr:=objectFrame(a_tensor):
VessiotFrameCheck(a_tensor):

tensIndType := tensorIndexType(a_tensor);
tensDensType := tensorDensityType(a_tensor) ;

if tensDensType <> [] then
	ERROR(cat("\n VESSIOT ERROR:",
                                     "\n A connection must have zero weight, \n",tensDensType)); 
fi;

if  not (tensIndType = [cov_bas, con_bas, cov_bas] or tensIndType = [cov_vrt, con_vrt, cov_bas]) 
then 
	ERROR(cat("\n VESSIOT ERROR:",
	                      "\n the connection must be of type [cov_bas,con_bas,cov_bas] or    [cov_vrt,con_vrt,cov_hor]\n",tensIndType)); 
fi; 

VessiotFrameCheck(a_tensor);

ans:=op(a_tensor);

ans[1][1]:=connection;


_VESSIOT(ans);
        
end:



















`convert/Vessiot_tensor`:=proc(geomObjField0)

local A, geomObjField, ans,vect_to_tens,form_to_tens,array_to_tens,connection_to_tens,objType;


geomObjField:=evalV(geomObjField0);


if objectType(geomObjField) =tensor then RETURN(geomObjField) fi;




#
#  ERROR checks
#


#=====================================================

vect_to_tens:=proc(a_vect::Vessiot_vect)

local index_char, ans, C, fr, max_mv_index, min_mv_index;

#    This routine converts a Vessiot vector to a Vessiot tensor.
#     
#    a_vect: Vessiot vector.
#    optional args: con_bas,con_vrt,con_jet,con_usr - range of space variables.

C:=objectComponents(a_vect);

VessiotFrameCheck(a_vect);

min_mv_index:=C[1][1][1];
max_mv_index:=C[-1][1][1];


if nargs = 1 then
	index_char:=[con_jet];

	if min_mv_index > frameBaseDimension() and
         max_mv_index <= frameFiberDimension()+frameBaseDimension() 
	then
		index_char:=[con_vrt]  
      fi;
	
	if max_mv_index <= frameBaseDimension() then
      	index_char:=[con_bas]
      fi;
 	
fi;


if nargs = 2 then
	if member(args[2],[con_bas,con_vrt,con_jet,con_usr]) then
      	index_char:=[args[2]]
      else
	      ERROR(cat("\n VESSIOT ERROR:",  
                      "\n a vector is a contravariant tensor.  Tensor type ",
				args[2], " is not admissable"))
      fi; 

      if  index_char[1]=con_bas and max_mv_index > frameBaseDimension() then 
		 ERROR(cat("\n VESSIOT ERROR:",  
                       "\n a vector element is not in the base space."))
      fi;

	if  index_char[1]=con_vrt and (max_mv_index > frameBaseDimension()+ frameFiberDimension()
         or min_mv_index <= frameBaseDimension()) and C<> [ [[1],0] ]
	then 
		ERROR(cat("\n VESSIOT ERROR:",   
                      "\n a vector element is not in the vertical space."))
	fi;
fi;

index_char:=[index_char,[]];
     
     
#    construct the tensor:

fr:=_Vessiot_current_frame_name;

ans:=_VESSIOT([[tensor,fr,index_char],C]);
 

ans

end:

#==============================================


form_to_tens:=proc(a_form::Vessiot_form)
# CLASS I PROGRAM
# 05/01/99:CH
# 01/04/03:IA
    
#    This routine converts a Vessiot form to a Vessiot tensor.
#     
#    a_form: Vessiot vector.
#    optional args: con_bas,con_vrt,con_jet,con_usr - range of space variables.


local index_char, comp_list, index_roster, indx, compSet,p,formComp, 
	 ans,  ans0,compt_list,fr, max_mv_index, min_mv_index, term;

compSet:=objectGenerators(a_form);
formComp:=objectComponents(a_form);


VessiotFrameCheck(a_form):

p:=formDegree(a_form);

min_mv_index:=min(op(compSet));
max_mv_index:=max(op(compSet));


if nargs = 1 then
	index_char:=[cov_jet$p];

	if min_mv_index > frameBaseDimension() and
         max_mv_index <= frameFiberDimension()+frameBaseDimension() 
	then
		index_char:=[cov_vrt$p]  
      fi;
	
	if max_mv_index <= frameBaseDimension() then
      	index_char:=[cov_bas$p]
      fi;
 	
fi;


if nargs = 2 then
	if member(args[2],[cov_bas,cov_vrt,cov_jet,cov_usr]) then
      	index_char:=[args[2]$p]
      else
	      ERROR(cat("\n VESSIOT ERROR: ", 
                      "\n a form is a covariant tensor.  Tensor type ",
		          args[2], "is not admissable."))
      fi; 

      if  index_char[1]=cov_bas and max_mv_index > frameBaseDimension() then 
		 ERROR(cat("\n VESSIOT ERROR: ", 
                       "\n a form element is not in the horizontal space."))
      fi;

	if  index_char[1]=cov_vrt and (max_mv_index > frameBaseDimension()+ frameFiberDimension()
         or min_mv_index <= frameBaseDimension()) 
	then 
		ERROR(cat("\n VESSIOT ERROR: ", 
                      "\n a form element is not in the vertical space."))
	fi;
fi;

index_char:=[index_char,[]];
     
     
#    construct the tensor:

compt_list:=objectComponents(a_form);
fr:=_Vessiot_current_frame_name;

ans0:=_VESSIOT([[tensor,fr,index_char],compt_list]);
 
ans:= (p!)&mult tensors:-symm_skew_indices(ans0,[all],skew);

end:

#========================================


array_to_tens:=proc (an_array::array, tensor_type::list) 

local ans, array_ranges, indx, index_roster, n, m, check, i, new_comps, 
      char_list, new_index; 

optionalFrameChange([args],2); 

n := frameBaseDimension(); 
m := frameFiberDimension(); 

char_list := tensor_type[1]; 

array_ranges := [op(2,eval(an_array))]; 

for indx to nops(char_list) do 
	check := 1; 
	if tensorIndexPart2(char_list[indx]) = bas 
		and op(1,array_ranges[indx]) = 1 
		and op(2,array_ranges[indx]) = n 
		then 
		check := 2 
	elif tensorIndexPart2(char_list[indx]) = vrt 
		and op(1,array_ranges[indx]) = 1 
		and op(2,array_ranges[indx]) = m 
		then 
		check := 2 
	elif tensorIndexPart2(char_list[indx]) = jet 
		and op(1,array_ranges[indx]) = 1 then 
		check := 2 
	elif tensorIndexPart2(char_list[indx]) = usr then 
		check := 2 
	fi; 
	
	if check = 1 then 
		ERROR(cat("\n VESSIOT ERROR:",
			    "\n The tensor index type ",char_list[indx],
			    "is not compatible with the array range",array_ranges[index])) 
	fi 
od; 

index_roster := [indices(an_array)]; 
new_comps := []; 

for indx in index_roster do 
	if an_array[op(indx)] <> 0 then 
		new_index := indx; 
		for i to nops(indx) do 
			if tensorIndexPart2(char_list[i]) = vrt then 
				new_index[i] := indx[i]+n 
			fi 
		od; 
		new_comps := [op(new_comps), [new_index, an_array[op(indx)]]] 
fi 
od; 

ans := _VESSIOT([[tensor, _Vessiot_current_frame_name, [char_list, tensor_type[2]]], new_comps]); 

helmsimp(ans) 

end:
#===============================



connection_to_tens:=proc(a_connection::Vessiot_connection)
# CLASS I PROGRAM
# 05/01/99:CH
# 01/14/03:IA

#    This routine assigns the type tensor to a connection, for the purposes of manipulation.
#    It should be noted that a connection is NOT a tensor.

local ans; 

VessiotFrameCheck(a_connection);
  	
ans:=op(a_connection);
ans[1][1]:=tensor;

_VESSIOT(ans)
        
end:

#-----------------------------------------




#
# MAIN PROGRAM
#

objType:=objectType(geomObjField);

if objType='vect'  then	
	ans:=vect_to_tens(geomObjField);
               RETURN(ans);
fi;


if objType ='form' then	
	ans:=form_to_tens(geomObjField);
               RETURN(ans);
fi;



if  type(geomObjField,array) then	
	ans:=array_to_tens(geomObjField,args[2]);
               RETURN(ans);
fi;

if  type(geomObjField,Matrix) then
      A:=convert(geomObjField,array);
	ans:=array_to_tens(A,args[2]);
               RETURN(ans);
fi;


if  type(geomObjField,Vessiot_connection) then	
	ans:=connection_to_tens(geomObjField);
               RETURN(ans);
fi;

end:

#------------------------------------------------------------------------------------------


`convert/Vessiot_form`:=proc(x)

local  ans,  tens_to_form,  biform_to_form;

biform_to_form := proc(bform0)

#CLASS II PROGRAM
#12/13/02:IA      
  
local ans, frame, newdegree, bform;

bform:=evalV(bform0);


if type(bform,Vessiot_biform)=false then 
	ERROR(cat("\n VESSIOT ERROR:",
		    "\n  biform_to_form requires arguement to be a biform:", 
		    "\n ",objectType(bform))); 
fi;

VessiotFrameCheck(bform):

frame:=frameProtocol();

newdegree :=  biformDegree(bform)[1] +biformDegree(bform)[2];

ans:=homomorphism[formBiform](frame[biform_to_form],bform, DifferentialGeometryTools:-zero_form(newdegree), 'form');



ans

end:


tens_to_form:=proc(a_tensor::Vessiot_tensor)
# CLASS I PROGRAM
# 05/01/99:CH
# 01/04/03:IA
     
#    This routine converts a Vessiot tensor to a Vessiot form.
#
#     IS THE  NORMALIZATION CORRECT>>>>>>> 

local ans, indexType,densType, fr,p,i,temp, comp_list,C,term;  


indexType:=tensorIndexType(a_tensor):
p:=nops(indexType);
densType:= tensorDensityType(a_tensor):

VessiotFrameCheck(a_tensor);

C:=objectComponents(a_tensor):


if  densType<>[] then
	ERROR(cat("\n VESSIOT ERROR: ",  
                "\n the tensor must have no weight. The given tensor has weight ", densType))
fi; 

for i from 1 to nops(indexType) do
	if not member(indexType[i],[cov_bas,cov_vrt,cov_jet,cov_usr]) then
       	ERROR(cat("\n VESSIOT ERROR:",
                      "\n the tensor must be of covariant type. The given tensor has index type ", 
                      indexType))
	fi;  
od;

temp:=tensors:-symm_skew_indices(a_tensor,[$1..p],skew);    
     
if temp <> a_tensor then
       ERROR(cat("\n VESSIOT ERROR:",
                 "\n the tensor must be skew symmetric over all indices."))
fi;

comp_list:=[]; 

for term in C do
	if term[1]=sort(term[1]) then
      	comp_list:=[op(comp_list),[term[1],p!*term[2]]];
      fi;
od; 


fr:= _Vessiot_current_frame_name;

ans:=_VESSIOT([[form,fr,p],comp_list]);
  
ans

end:

if   objectType(x)=tensor   
	then  ans:=tens_to_form(x)
fi;

if objectType(x)=biform
	then  ans:=biform_to_form(x)
fi;


ans;


end:

#========================================================


`convert/Vessiot_vect`:=proc(X)

local tens_to_vect;

tens_to_vect:=proc(a_tensor::Vessiot_tensor)
#Class I Program
#
#     

#    This routine converts a Vessiot tensor to a Vessiot vector.

local ans, indexType,densType, fr, C;  


indexType:=tensorIndexType(a_tensor):
densType:= tensorDensityType(a_tensor):

fr:=objectFrame(a_tensor):
VessiotFrameCheck(a_tensor):

C:=objectComponents(a_tensor):


if  densType<>[] then
	ERROR(cat("\n VESSIOT ERROR:",  
                "\n the tensor must have no weight. ", 
                " The given tensor has weight ", densType))
fi; 

if 1 < nops(indexType) then 
	ERROR(cat("\n VESSIOT ERROR:", 
                "\n the tensor must have a single index. ",
                "The given tensor has index type ",indexType)) 
fi;

if not member(indexType[1],[con_bas,con_vrt,con_jet,con_usr]) then
       ERROR(cat("\n VESSIOT ERROR:", 
                 "\n the tensor must be of contravariant type.",
	           " The given tensor has index type ", indexType))
fi;  


ans:=_VESSIOT([[vect,fr,[]],C]);


  
ans

end:

tens_to_vect(X)
end:
#------------------------------


#----------------------------------------------------------------------------------

`convert/Vessiot_array`:=proc(X)

local tens_to_array;

tens_to_array:=proc(a_tensor::Vessiot_tensor)
# CLASS II PROGRAM
# 05/01/99:CH
# 01/04/03:IA
     
#    This routine takes a Vessiot tensor to a maple array. 

local ans, i, indexType,tensComponents, new_index, array_ranges, 
	max_jet_term, term,n,m;

n:=frameBaseDimension();
m:=frameFiberDimension();
max_jet_term:=frameJetDimension();

indexType:=tensorIndexType(a_tensor);
tensComponents:=objectComponents(a_tensor);

array_ranges:=[];

if indexType=[] then
	ans:=array(1..1);
      ans[1]:=tensComponents[1][2]; 
	RETURN(eval(ans))
fi;

for term in indexType do
	
	if tensorIndexPart2(term)=`bas` then
		array_ranges:=[op(array_ranges),1..n];
      fi;

	if  tensorIndexPart2(term)=`vrt` then
		array_ranges:=[op(array_ranges),1..m];
	fi; 
		   
	if  tensorIndexPart2(term)=`jet` then
		array_ranges:=[op(array_ranges),1..max_jet_term];
	fi 

od;

ans:=array(op(array_ranges),'sparse');

for term in tensComponents do
      new_index:=term[1];
      	for i from 1 to nops(term[1]) do
		 	if tensorIndexPart2(indexType[i])=vrt then
                  	new_index[i]:= term[1][i] -n
                  fi;
           od; 


	ans[op(new_index)]:=term[2];
od;
	
eval(ans);

end:
tens_to_array(X)

end:

#-------------------------------------------------------------


