#
#
#





MatrixRepresentationToMultiplicationProcedure:=proc(M,vars)

local n,A,B,C,E,eq,multiplication1, multiplication2,ans;

n:=nops(vars);

A:=map2(subs, {seq(vars[i]=X[i],i=1..n)},M);
B:=map2(subs, {seq(vars[i]=Y[i],i=1..n)},M);
C:=map2(subs, {seq(vars[i]= cat(_z,i),i=1..n)},M);

E:=eval(C- LinearAlgebra[MatrixMatrixMultiply](A,B));
eq:=convert(E,set);

eq:=map(combine,eq);


multiplication1:=solve(eq,{seq(cat(_z,i),i=1..n)});
multiplication2:=subs(multiplication1,[seq(cat(_z,i),i=1..n)]);

proc(A,B) 
	subs({seq(X[i]=A[i],i=1..nops(A)),seq(Y[i]=B[i],i=1..nops(B))}, multiplication2) 
end: 


end:




LeftRightInvariantVectorFieldsAndForms:=module()

option package;

export ModuleApply, MaurerCartanGroupMult, MaurerCartanMatrixGroup, MaurerCartanUSU2 ;


#
#

ModuleApply:=proc()

local ans;

if args[-1] = group_mult then
	ans:=MaurerCartanGroupMult(args[1..-2]);
      RETURN(ans):
fi;

if args[-1]=matrix_group then
	ans:=MaurerCartanMatrixGroup(args[1..-2]);
	RETURN(ans):
fi;
      
if args[-1]=USU2 then
	ans:=MaurerCartanUSU2(args[1..-2]);
      RETURN(ans);
fi;

ans:

end:




MaurerCartanGroupMult:=proc(gr_mult,id, gr_frame) 

local vars, n,a_vars, b_vars, i,rightM, J,
      leftM, muR, muL,invR, invL, OmegaR, OmegaL,XR,XL; 

change_frame_to(gr_frame):

vars:=frameIndependentVariables(); 
n:=frameBaseDimension(); 

a_vars:=[seq(cat(a,i), i=1..n)]; 
b_vars:=[seq(cat(b,i), i=1..n)]; 

rightM:=gr_mult(vars,b_vars);
leftM:=gr_mult(a_vars,vars);

muR:=transform(gr_frame,gr_frame,[seq(vars[i]=rightM[i], i=1..n)]);
muL:=transform(gr_frame,gr_frame,[seq(vars[i]=leftM[i],i=1..n)]);

invR:=solve({seq(rightM[i]=id[i], i=1..n)},convert(b_vars,set));
invL:=solve({seq(leftM[i]=id[i], i=1..n)},convert(a_vars,set));




OmegaR:= map(helmsimp,subs(invR,map2(pullback,muR,frameBaseForms() ) ));
OmegaL:= map(helmsimp,subs(invL,map2(pullback,muL,frameBaseForms() ) ));

if nargs=4 then
        J:=convert(args[4],listlist);
	  OmegaR:=map(v_zip,J,OmegaR,plus);
	  OmegaL:=map(v_zip,J,OmegaL,plus);
fi;        



XR:= dual_basis(OmegaR);
XL:= dual_basis(OmegaL);

[[XL, OmegaL], [XR, OmegaR]]

end:


#~~~~~~~~


 MaurerCartanMatrixGroup:=proc(gp_rep, alg_rep,gr_frame,alg_rep_parameters) 
 
 local vars, n,a_vars, b_vars, T, i,rightM, m,S0,S,OmegaR0, OmegaL0,A,P,rho, eq,soln,J,
       leftM, muR, muL,invR, invL, OmegaR, OmegaL,XR,XL; 
 
 change_frame_to(gr_frame):
 

 m:=frameBaseDimension();

 T:=LinearAlgebra[MatrixInverse](gp_rep);

 n:=LinearAlgebra[RowDimension](T);

 
 S0:=map(scalar_form,gp_rep);
 
 
 S:=map(ext_d,S0);
 
 
 OmegaR0:=  S &mmult T;
 OmegaL0:=  T &mmult S;
 
 A:=[seq(map(diff,alg_rep,s),s=alg_rep_parameters)];
 
 
 P:=[ seq( v_zip([seq(_p[i,j],i=1..m)],frameBaseForms(),plus), j=1..m)];
 
 rho:=Matrix(n,n,(i,j)->DifferentialGeometryTools:-zero_form(1));

 for i from 1 to m do 
 rho:= rho &mplus ( A[i] &mmult P[i]);
 od;
 
 eq:=map(coeff_set,convert(OmegaR0 &mminus rho,set));
 eq:=map(op, eq);
 
 soln:=solve(eq, {seq(seq(_p[i,j],i=1..m), j=1..m)});
 
 OmegaR:= map(helmsimp, subs(soln, P));
 
 
 eq:=map(coeff_set,convert(OmegaL0 &mminus rho,set));
 eq:=map(op, eq);
 
 soln:=solve(eq, {seq(seq(_p[i,j],i=1..m), j=1..m)});
 
 
 OmegaL:= map(helmsimp,subs(soln, P));
 
 
 if nargs=5 then
         J:=convert(args[5],listlist);
 	  OmegaR:=map(v_zip,J,OmegaR,plus);
 	  OmegaL:=map(v_zip,J,OmegaL,plus);
 fi;        
 
 
 
 XR:= dual_basis(OmegaR);
 XL:= dual_basis(OmegaL);
 
 [[XL, OmegaL,eval(OmegaL0)], [XR, OmegaR,eval(OmegaR0)]]
 
 end:


#~~~~~~~~~~~~~~



MaurerCartanUSU2:=proc(alg,frame_name)

local gp_mult,M,E,G,Orig,newG,C,J,i,N,A,U;

n:=alg[1];

if alg[2]>0 then 
	gp_mult:=eval(USU2LieGroups()[alg][G_dot]);
	M:=Matrix(USU2LieGroups()[alg][representation]);
	E:=USU2LieGroups()[alg][identity];
	U:=Matrix(USU2Representation()[alg]);
else
	gp_mult:=eval(USU2DLieGroups()[alg][G_dot]);
	M:=Matrix(USU2DLieGroups()[alg][representation]);
	E:=USU2DLieGroups()[alg][identity];
	U:=Matrix(USU2D_Representation()[alg]);
fi;

G:=[seq(map(diff,M,cat(x,i)),i=1..n)];

Orig:={seq(cat(x,i)=E[i],i=1..n)};
newG:=[seq(map(eval,  map2(subs,Orig,G[i])),i=1..n)];

A:=[seq(map(diff,U,s[i]),i=1..n)];
C:=linear_combo(newG,A);
J:=Matrix(C);

MaurerCartanMatrixGroup(gp_mult,E,frame_name,LinearAlgebra[MatrixInverse](J));
end:



end module:
