#
#
#

LieAlgebraToMaurerCartanForms:=module() 

local form_solve1;

export  ModuleApply, LieAlgebraToMaurerCartanForms1, LieAlgebraToMaurerCartanForms2, 
SolvableLieAlgebraToMaurerCartanForms;


ModuleApply:=proc(alg_name, fr_name, method)

local ans;

if method=Cartan then 
	ans:=LieAlgebraToMaurerCartanForms1(alg_name,fr_name);
fi;

if method=solvable then 
      ans:=SolvableLieAlgebraToMaurerCartanForms(alg_name,fr_name)
fi;


if method=Levi then 
	ans:=LieAlgebraToMaurerCartanForms2(args[1], fr_name):
fi;

ans:
end:


#~~~~~~  

LieAlgebraToMaurerCartanForms1:=proc(alg_name, fr_name)

# Class II program
# IA  10/02/2004

description `Implement of Cartans proof of Lie's Third Theorem-- See H. Flanders :Diff Forms...`;

option  `Copyright  Ian Anderson  02 2004`;

local i,j,k,vars, n, f,F,H,Id,DE,IC, de, soln, H1,Omega; 

if nargs <>2 then  
	ERROR(cat("\n GroupAction:",
                "\n  LiesThird requires  2 arguments:",
                "\n  Initialized Lie algebra name, frame name."));
fi;

if  member(alg_name,listOfFrameNames()) =false then 
	ERROR(cat("\n GroupAction:",
                "\n First argument to LiesThird must be an initialized Lie algebra name."));
fi;


if  member(fr_name,listOfFrameNames()) =false then 
	ERROR(cat("\n GroupAction:",
                "\n Second argument to LiesThird must be an initialized frame name."));
fi;

if frameBaseDimension(alg_name) <> frameBaseDimension(fr_name) then
	ERROR(cat("\n GroupAction:",
                "\n Dimensions of  Lie algebra and coordinate frame do not agree:",
                "\n", frameBaseDimension(alg_name) ,",", frameBaseDimension(fr_name) ));
fi;
	

vars:=frameIndependentVariables(fr_name);

n:=frameBaseDimension();

change_frame_to(alg_name):

f:=proc(i,j) 
	local k ; 
	add(structure_constant(i,k,j)*vars[k], k=1..n) 
end;


F:=Matrix(n,n,f);

H:=Matrix(n,n,  (i,j)-> cat(h,i,j)(t));

Id:=LinearAlgebra[IdentityMatrix](n);

DE:= map(diff,H,t) - Id - LinearAlgebra[MatrixMatrixMultiply](H,F);

IC:={seq(seq(cat(h,i,j)(0)=0, i=1..n),j=1..n)};

de:=convert(DE,set) union IC;

soln:=dsolve(de);

H1:=map2(subs,{t=1},map2(subs,soln,H));

change_frame_to(fr_name);

Omega:=convert(Matrix(1,n,frameBaseForms()) &mmult H1,listlist);

op(Omega);
end:


#~~~~~~

SolvableLieAlgebraToMaurerCartanForms:=proc(alg,fr) 

local n,vars, ans, k, alpha, beta, i, j,tau,C,gamma1, sigma, gamma2,omega, omegabar,TF, rho; 


change_frame_to(alg):

rho:=LieAlgebraRepresentations:-MakeRepresentation(alg,alg,LieAlgebras:-ad());

TF:=LieEngelTheorems:-CheckSolvableRepresentation(rho);

if TF <> true then 
	ERROR("Ad representation not solvable"); 
fi;



n:=frameBaseDimension(alg); 

vars:=frameIndependentVariables(fr);

ans:=[seq(zero_form(1,fr),i=1..n-1), frameBaseForms(fr)[n]];

for k from n-1 to 1 by -1 do

	alpha:=v_zip([seq( -structure_constant(k,i,k,alg),i=k+1..n)], 
		         ans[k+1..n],plus);

	beta:=v_zip([seq( -structure_constant(k-1,i,k,alg),i=k+1..n)], 
		         ans[k+1..n],plus);
	
      tau:=zero_form(2,fr);

	for i from k+1 to n do 
		for j from i+1 to n do
            	tau:=tau &minus (structure_constant(i,j,k,alg) 
			      &mult (ans[i] &wedge ans[j])) 
		od; 
	od;

	C:=coeff_set(beta);

	if C={0} then 

# CASE I
 
      	ans[k]:=form_solve1(alpha,tau,vars[k..n])

      else 
      
      	sigma:=zero_form(2);
  
		for i from k+1 to n do 
			for j from i+1 to n do
            
				sigma:=sigma &minus (structure_constant(i,j,k-1,alg) 
			      &mult (ans[i] &wedge ans[j])) 
			od; 
		od;

      	gamma1:= evalV(alpha + I*beta);
      	gamma2:= evalV(sigma + I*tau);

      	omega:=form_solve1(gamma1,gamma2,[vars[k-1] +I*vars[k], op(vars[k+1..n])]);
      	
		omegabar:=Vessiot_map(2,evalc,Vessiot_map(2,conjugate,omega));
      
      	ans[k-1]:= evalV(1/2*(omega + omegabar));
      	ans[k]:=   evalV(-I/2*(omega -omegabar));
      	k:=k-1;
      fi;
od;

ans; 

end:




form_solve1:=proc(alpha,beta,vars)

local i,n,F,theta,sigma,eq,pde_soln_parser,
soln,expressions,arb_functions,S,soln1,ans,v; 

#
# Local Procedure
#

pde_soln_parser:=proc(exp) 
local ans, v;
ans:=[]; 

for v in exp do 
	if member(op(0,v),[seq(cat(_F,i), i=1..n)]) then 
		ans:=[op(ans),v] 
	fi; 
od; 
	
ans; 
end:

#
#
#

n:=nops(vars); 

F:=seq(cat(a,i)(op(indets(vars))),i=2..n);
theta:= v_zip([1, F],map(ext_d,map(scalar_form,vars)),plus);

sigma:=ext_d(theta) &minus ( (theta &wedge alpha)  &plus beta);
eq:=coeff_set(sigma);

if eq<>{0} then 
	soln:=pdsolve(eq,{F}) ;
	expressions:=indets(map(rhs,soln)); 
	arb_functions:=pde_soln_parser(expressions);
	S:={seq(v=0,v=arb_functions)};
	soln1:=eval(convert(subs(S,soln),int)) 
else 
	soln1:={} 
fi; 

helmsimp(subs(soln1,theta));
end:

#~~~~~~ LieAlgebraToMauerCartanForms2 


LieAlgebraToMaurerCartanForms2:=proc(LeviDecomposition::list(list(Vessiot_vect)),GroupFrame)

local SS, Rad, Vars, n, AlgData0, W, newComponents, newSSBasis, MCcomponents, OmegaSS0, phi1, 
OmegaSS, AlgData1, adMatrices, rho, LieBasis, LieBasisComponents, newRadBasis, AlgData2, OmegaRad0, phi2, 
OmegaRad1,Restricted_ad,OmegaLevi,newOmegaRad,fr,m,k,RS;


SS:=LeviDecomposition[2]:
Rad:=LeviDecomposition[1];
fr:=objectFrame(Rad[1]):


Vars:=frameIndependentVariables(GroupFrame);
n:=nops(Rad):
m:=nops(SS):
if nops(SS) <>3 then ERROR(`program only works for sl2`) fi;

AlgData0:=convert(SS, Lie_algebra_data,`_ss1`);
Lie_algebra_init(AlgData0):
W:=LieAlgebraClassifier:-ClassifyLieAlgebra():

if W[2] <>[[USU, [3, 5, 0]]] then  ERROR(`program only works for sl2`) fi;

newComponents:=convert(LinearAlgebra[Transpose](W[1]^(-1)),listlist):
newSSBasis:=map(v_zip,newComponents,SS,plus);
coord_init([x1,x2,x3],`temp1`);

MCcomponents:=[[x2/x1^2*(1+x3*x2), 1/x1, -x2^2/x1], [-2/x1*(1+x3*x2), 0, 2*x2], [-x3, 0, x1]];
OmegaSS0:=map(v_zip,MCcomponents,[dx1,dx2,dx3],plus);

phi1:=transform(GroupFrame,temp1,[x1=Vars[1],x2=Vars[2], x3=Vars[3]]);
OmegaSS:=map2(pullback,phi1,OmegaSS0);


AlgData1:=convert(Rad, Lie_algebra_data,`_rad1`);
Lie_algebra_init(AlgData1):


adMatrices:=ad():
rho:=MakeRepresentation(_rad1,_rad1,adMatrices);

#
#  Watch for problems with asc ideal basis
#

LieBasis:=LieEngelTheorems:-SolvableRepresentation(rho)[1]:

LieBasisComponents:=linear_combo(LieBasis,frameBaseVectors());
newRadBasis:=map(v_zip,LieBasisComponents,Rad,plus):

AlgData2:=convert(LieBasis, Lie_algebra_data,`_rad2`);
Lie_algebra_init(AlgData2);

coord_init([seq(cat(x,i),i=1..n)],`RadGp`);
OmegaRad0:=SolvableLieAlgebraToMaurerCartanForms(_rad2,RadGp):
phi2:=transform(GroupFrame,RadGp,[seq(cat(x,i) =Vars[i+3], i=1..n)]):
OmegaRad1:=map2(pullback,phi2,OmegaRad0);

change_frame_to(fr);
Restricted_ad:= [seq(ad(v,Rad),v=newSSBasis)];

change_frame_to(GroupFrame);
OmegaLevi:=Matrix(n,n, (i,j)->zero_form(1));

for k from 1 to m do
RS:=(Restricted_ad[k] &mmult OmegaSS[k]);
OmegaLevi:= OmegaLevi &mplus RS;
od; 

W:=MaurerCartan_matrix_to_matrix_rep(right,OmegaLevi,METHOD=pdsolve,INIT_POINT=[Vars[1]=1,Vars[2]=0,Vars[3]=0]);

newOmegaRad:=convert(W &mmult Vector(OmegaRad1),list):





[newRadBasis,newSSBasis],[newOmegaRad,OmegaSS]
end:












end module: