    #
# LinalgUtilities.utl
#
# 
#
# 
# 
# canonical_subspace_basis, left_inverse;
# 
# annihilator; linear_combo; Vessiot_basis; 
# check_linear_combo
#
# complementary_basis
#


IntersectSubspaces:=proc(subspace_list0)

# CLASS II PROGRAM
# 06/27/2005 IA

option `Copyright Ian M. Anderson 2005`; 

description `Computes the intersection of a list of subspaces`;

local S,s,n,X,vars,A,N,C,i,j,eq,v,basis,sub_vars,subspace_list,Z,Coeff;
 
if nargs=0 then
	ERROR(cat("\n DiffGeom Error:",
                "\n IntersectSubspaces requires 1 argument:",
		    "\n a list of list of biforms, forms, vectors, tensors."))
fi;

if type(subspace_list0, list(list))=false then 
	ERROR(cat("\n DiffGeom Error:",
                "\n The argument for IntersectSubspaces must be a",
		    "\n a list of list of biforms, forms, vectors, tensors."))
fi;

subspace_list:=map2(map,evalV,subspace_list0); 


if type(subspace_list, list(list(Vessiot_bftv)))=false then 
	ERROR(cat("\n DiffGeom Error:",
                "\n The argument for IntersectSubspaces must be a",
		    "\n a list of list of of biforms, forms,vectors, tensors."))
fi;


n:=nops(subspace_list);
S:=subspace_list[1];
s:=nops(S):
 
vars:=[seq(seq(C[i,j],j=1..nops(subspace_list[i])),i=1..n)]; 

eq:={}:

for i from 2 to n do 	
	
	sub_vars:=[seq(C[1,j],j=1..s),seq(C[i,j],
		   j=1..nops(subspace_list[i]))];  
 	
	X:=v_zip( sub_vars,[op(S), op(subspace_list[i])],plus);
 	eq:=eq union  coeff_set(X); 
od; 

A:=LinearAlgebra[GenerateMatrix](expand(eq),vars);
N:=Vessiot_nullspace(A[1]);

basis:=[];
Z:=[0$s];
 
for v in N do
      Coeff:=[seq(v[i],i=1..s)];
      if Coeff<> Z then         
  		basis:=[op(basis), v_zip(Coeff,S,plus)] 
	fi;
od;
 
basis;

end:

#-----------------------------


DiffGeom_basis:=proc(a) 
#CLASS II PROGRAM
#11/20/02:IA

local Vessiot_basis_bftv, Vessiot_basis_matrix, ans;

#
# LOCAL SUBROUTINES
#

Vessiot_basis_bftv:=proc(v_list::list(Vessiot_bftv))
#CLASS II PROGRAM
#11/18/02:IA

# v_list:  alist of Vessiot objects 
# bas:     a basis for the span of v_list
# November 20, 2002: IA
# 03/30/03:IA
 
global _Vessiot_xlinalg_conditions;
local n, a,Z, A,bas,i,j,vars,w;

n:=nops(v_list);
vars:=[seq(a[i],i=1..n)];

Z:= coeff_set(v_zip(vars, v_list,plus));
Z:=collect(Z,vars);

A:=Vessiot_rref(LinearAlgebra[GenerateMatrix](Z, vars)[1]):


bas:=[];

for  i from 1 to LinearAlgebra[RowDimension](A) do 
	for  j from i to n do
		if  A[i,j]<>0 then 
			bas:= [op(bas), v_list[j]];
			break;
		fi;
	od;
od;

bas;
end;

#=====================================================

Vessiot_basis_matrix:=proc(v_list::list(Matrix))
# v_list:  alist of Vessiot objects 
# bas:     a basis for the span of v_list
# 11/20/02:IA

local n, a,Z, A,bas,i,j,vars,w;
global _Vessiot_xlinalg_conditions;

n:=nops(v_list);
vars:=[seq(a[i],i=1..n)];

Z:= convert(eval(add(vars[i]*v_list[i], i=1..n)),set);
Z:=collect(Z,vars);


A:=Vessiot_rref(LinearAlgebra[GenerateMatrix](convert(Z,set), vars)[1]);

bas:=[];

for  i from 1 to LinearAlgebra[RowDimension](A) do 
	for  j from i to n do
		if  A[i,j]<>0 then 
			bas:= [op(bas), v_list[j]];
			break;
		fi;
	od;
od;

map(eval,bas);

end:

#==================================================


#
# MAIN ROUTINE
#



if type(a[1],Matrix) then 
	ans:=Vessiot_basis_matrix(a);
      RETURN(map(eval,ans));  
fi;

if type(a[1],Vessiot_bftv) then 
      ans:=Vessiot_basis_bftv(a);
      RETURN(ans):
fi;


end:




















canonical_subspace_basis:=proc(subspace::list({Vessiot_bftv,Matrix}),
                                   master_space::list({Vessiot_bftv,Matrix}) )
#11/20/02:IA

local L,A,B,i,v,new_subspace;  

if subspace= [] then 
	RETURN([]) 
fi;

A:=Matrix(linear_combo(subspace,master_space));
B:=Vessiot_rref(A);
new_subspace:=[];

if type(subspace[1],Vessiot_bftv) then
	for i from 1 to LinearAlgebra[RowDimension](B) do
 		v:=v_zip(convert(LinearAlgebra[Row](B,i),list),master_space,plus);
 		new_subspace:=[op(new_subspace),v];
	od;
fi;


if type(subspace[1],Matrix) then
	for i from 1 to LinearAlgebra[RowDimension](B) do
 		v:=eval(add(B[i,j]*master_space[j],j=1..nops(master_space)));
 		new_subspace:=[op(new_subspace),eval(v)];
	od;
fi;




new_subspace;
end:

#-------------------------------------


annihilator := proc(vf_list::{list(Vessiot_vect), list(Vessiot_form)})
#CLASS I PROGRAM
#12/13/02:IA 

local n, a, omega, one_forms, X, f, vec, i, eq, ans, vars, v, A, N,fr;

if nops(vf_list) =  0 then
	if nargs=2 then  
		one_forms:= args[2] 
	else 
		one_forms := frameBaseForms() 
	fi:

	RETURN(one_forms)
fi;
    
fr:=_Vessiot_current_frame_name;

if objectType(vf_list[1])= 'vect' then 
	if nargs=2 then  
		one_forms:= args[2] 
	else 
		one_forms := frameBaseForms() 
	fi:
	
	n:=nops(one_forms);
	vars := [seq(a[i], i = 1 .. n)];
	omega := v_zip(vars, one_forms, plus);
	eq := {};
	
	for v in vf_list do
        	eq := eq union coeff_set(hook(v, omega))
    	od;

      eq:=collect(eq,vars); 

	A := LinearAlgebra[GenerateMatrix](eq, vars);
    	N := Vessiot_nullspace(A[1]);
    	
      ans := {};
      
	for v in N do ans :=
      	{op(ans), v_zip(convert(v, list), one_forms, plus)}
    	od;
    	
	ans := convert(ans, list)
fi;

if objectType(vf_list[1]) ='form' then 
      if nargs=2  then 
		vec:=args[2] 
	else
		vec := frameBaseVectors()
	fi:
      
	n:=nops(vec);
	vars := [seq(a[i], i = 1 .. n)];
	X := v_zip(vars,vec, plus);
    	eq := {};
    	
	for f in vf_list do
      	eq := eq union coeff_set(hook(X, f))
    	od;
    
	eq:=collect(eq,vars); 

	A := LinearAlgebra[GenerateMatrix](eq, vars);
    	N := Vessiot_nullspace(A[1]);
    	ans := {};

	for v in N do ans :=
        	{op(ans), v_zip(convert(v, list), vec, plus)}
   	od;
    
	ans := convert(ans, list)
fi;

ans

end:


complementary_basis:=proc(small_basis,big_basis)
#CLASS II PROGRAM
#12/13/02:IA

local comp_basis,v,test,var,gen_comp_basis,i,V,w,var_set;



if nops(small_basis)=0 then 
	RETURN(big_basis)
fi;

comp_basis:=[];

for v in big_basis do
	test:=linear_combo(v, [op(small_basis), op(comp_basis)]);
      
	if test=[] and type(small_basis[1],Matrix) then  		
		comp_basis:=[op(comp_basis),eval(v)] 
      fi;
	
	if test=[] and type(small_basis[1],Vessiot_bftv) then  		
      	comp_basis:=[op(comp_basis), v];
	fi;
od;

if nargs = 2 then RETURN(comp_basis) fi;

if nargs = 3 then 
	var:=args[3];
      gen_comp_basis:=[]; 
	var_set:={};
      i:=0;
      for v in comp_basis do
      	V:=v; 
	     	for w in small_basis do
            	i:=i+1;
                  V:= V &plus  (cat(a,i) &mult w);
 			var_set:={op(var_set),cat(a,i)};
            od;      
            gen_comp_basis:=[op(gen_comp_basis), V];
            
      od;
fi;
gen_comp_basis,var_set

end:

#***************************************************







