#
# Checks.alg
#
# LieAlgCheck
#
#

LieAlgCheck:=proc()

# CLASS II PROGRAM
# 01/10/2004 IA

option `Copyright Ian M. Anderson 2004`; 

description `Check properties of Lie algebras and subalgebras.`;

local   SmodJ, Adj_rep,check,subspace, flag, subalg, S,c, A,d,ans, q,Jacobi,J,
	 g0 ,v,w,temp,range_alg, domain_alg,temp1,temp2,parameters,p,alg,m_general,vec1,vec2,b,
	fr ,n,i, j, B,C,forms,S0,t,k,form_basis,s,Z,check_homo,StructureArrayRange,StructureArrayDomain,RangeBasis,g,N,
	basis,Eq,eq,a,m,components,C1,vars, M,H,Iso;


global _Vessiot_show_intermediate_steps_flag;


if nargs=0 then 
	ERROR(cat("\n LieAlg Error:", 
          	"\n Last argument for LieAlgCheck must be:",
"\n {abelian,ideal1, ideal2, Jacobi1, Jacobi2, Lie_algebra_homomorphism1, Lie_algebra_homomorphism2",
            "\n  indecomposable, absolutely_indecomposable",
		"\n  naturally_reductive_pair1, naturally_reductive_pair2, nilpotent, graded_Lie_algebra", 
		"\n  semisimple, solvable, subalgebra1, subalgebra2, reductive_pair1, reductive_pair2,", 
		"\n  symmetric_pair1, symmetric_pair2}" ));  

fi;




check:=args[-1];


#-----------------------------------------


if check=  indecomposable then 


if nargs=1 then
	alg:= convert(_Vessiot_current_frame_name,Lie_algebra_data)
else
	alg:=args[1]
fi;



S := LieAlgebrasTools:-matrix_centralizer(ad(alg)); 
J :=LieAlgebrasTools:-Jacobson_radical(S); 
SmodJ := complementary_basis(J,S); 

if 2 < nops(SmodJ) then ans := false fi; 

if nops(SmodJ) = 2 then 
	C := linear_combo(eval(SmodJ[2]^2),[SmodJ[1], op(J)]); 
	if nops(C) = 0 then 
		RETURN(false) 
	fi;  
	ans:=Vessiot_query(C[1] < 0) 
fi; 

if nops(SmodJ) = 1 then ans := true fi; 

RETURN(ans) 
fi;

#-----------------------------------------------------


if check= absolutely_indecomposable then 


if nargs=1 then
	alg:= convert(_Vessiot_current_frame_name,Lie_algebra_data)
else
	alg:=args[1]
fi;

	Adj_rep:=ad(alg):

	C:=LieAlgebrasTools:-matrix_centralizer(Adj_rep);

	#Remove the identity from C 

	C:=[seq(C[i], i=2..nops(C))]:
 
	p:=nops(C):

	ans:= true;

	for A in C do  
	 	for B in C do 
			if  LinearAlgebra[Trace](LinearAlgebra[MatrixMatrixMultiply](A,B)) <> 0 then 
		    		ans:= false  
			fi;
	 	od; 
	od;

RETURN(ans)


fi ;


#-----------------------------------------------------------------





#.......... Jacobi1: returns true or false ...........

if check= Jacobi1 then
	 
	n:=frameBaseDimension();
	fr:=_Vessiot_current_frame_name;
      form_basis:=frameBaseForms();
	ans:=true;
      	
	for q to n do  
    		Jacobi:=ext_d(ext_d(form_basis[q]));	
    		temp:= simplify(coeff_set(Jacobi));      
    		if temp<> {0} then 
			ans:=false 
		fi;

	od;
    
	RETURN(ans);

fi;

#.......... Jacobi2: returns coeffs of d^2 ..........

if check= Jacobi2 then
	 
	n:=frameBaseDimension();
	fr:=_Vessiot_current_frame_name;
      form_basis:=frameBaseForms();

	eq:={};
	
	for q to n do  
    		Jacobi:=ext_d(ext_d(form_basis[q]));	
    		temp:=  simplify(op(coeff_set(Jacobi)));      
    		eq:={op(eq),temp}
	od;
       
	if  nargs=1 then
		ERROR(cat("\n LieAlg Error:", 
                      "\n First argument for LieAlgCheck(Jacobi2) is the set of unassigned parameters",
			    "\n appearing in structure equations." )) 	
	fi;

	
	if type(args[1],set(name))=false then
		ERROR(cat("\n LieAlg Error:", 
                      "\n First argument for LieAlgCheck(Jacobi2) is the set of unassigned parameters",
			    "\n appearing in the structure equations.")) 	
	fi;

      ans:=solve(eq, args[1]);
  
	RETURN(eq, ans)

fi;


#.......... check_subalgebra..........

if check=subalgebra1 or check=subalgebra2 then 

      

	fr:=_Vessiot_current_frame_name;
	n:= frameBaseDimension();
	g0:=frameBaseVectors();


    	if check=subalgebra1 then  
      	if nargs <> 2 then 
		ERROR(cat("\n LieAlg Error:", 
			    "\n LieAlgCheck(*,",check,") requires 1 argument:",
			    "\n a list of vectors."))
		fi;
	fi;


    	if check=subalgebra2 or check=ideal2 then  
      	if nargs <> 3 then 
		ERROR(cat("\n LieAlg Error:", 
			    "\n LieAlgCheck(*,",check,") requires 2 arguments:",
			    "\n First argument: a list of vectors defining a subspace;", 
		          "\n Second argument: the  set of parameters appearing in the subspace."))
		fi;
            vars:=args[2]
	fi;


      if type(args[1],list)=false then
		ERROR(cat("\n LieAlg Error:", 
                      "\n First argument for LieAlgCheck(*,",check,")is a list of vectors." )) 

      fi;

      subspace:=map(evalV,args[1]);
      
	if type(subspace,list(Vessiot_vect))=false then
		ERROR(cat("\n LieAlg Error:", 
                      "\n LieAlgCheck(*,",check,") requires a list of vectors." )) 
      fi;

      if nops(DiffGeom_basis(subspace)) <>   nops(subspace) then
		ERROR(cat("\n LieAlg Error:", 
                      "\n Given list of vectors for LieAlgCheck(*,",check,") not independent." )) 
      fi;
 

     

	B:=LieAlgebrasTools:-bracket_of_subspaces(subspace,subspace);

	

	S0:=subspace;
	
	#if check=subalgebra2 then 
	#	for t in args[2] do
	#	S0:=subs(t=0,S0)
	#	od;
	#fi;
	
	S0:=map(helmsimp,S0);

	C:=complementary_basis(S0,g0);

	forms:=dual_basis([op(C),op(subspace)]);
 
	eq:={}:

	for v in B do 
		for  i from 1 to nops(C) do 
      		ans:=coeff_list((hook(v,forms[i]),[[]]));
			if ans<>0 then 
				eq:={op(eq),ans}
            	fi;
		od;
	od;
	

      if check=subalgebra1 then
		if eq={} then 
			RETURN(true)
		else
			RETURN(false)
		fi;
      fi;

	if check=subalgebra2 then
		if type(vars,set(name))=false then
			ERROR(cat("\n LieAlg Error:", 
                      "\n LieAlgCheck(*,",check,") requires a set of variables",
			    "\n as the second argument." )) 
		fi;
	

		ans:=solve(eq,vars);
	
      	if ans=NULL then 
        		RETURN(false, eq)
      	else  
                  ans:=[ans]; 
			B:=[];
                  for s in ans do
				B:=[op(B),map(helmsimp,subs(s,subspace))];
	      	od;
			RETURN(true,eq, ans,B);
	      fi;
      fi;

fi;
#........... check_ideal ..........


if check =ideal1 or check=ideal2 then

	fr:=_Vessiot_current_frame_name;
	n:= frameBaseDimension();
	g0:=frameBaseVectors();



    	if check=ideal1 then  
      	if nargs <> 2 then 
		ERROR(cat("\n LieAlg Error:", 
			    "\n LieAlgCheck(*,",check,") requires 1 argument:",
			    "\n a list of vectors."))
		fi;
            vars:=args[2]
	fi;


    	if check=ideal2 then  
      	if nargs <> 3 then 
		ERROR(cat("\n LieAlg Error:", 
			    "\n LieAlgCheck(*,",check,") requires 2 arguments:",
			    "\n First argument: a list of vectors defining a subspace;", 
		          "\n Second argument: the  set of parameters appearing in the subspace."))
		fi;
            vars:=args[2]
	fi;


	if type(args[1],list)=false then
		ERROR(cat("\n LieAlg Error:", 
                      "\n First argument for LieAlgCheck(*,",check,") is a list of vectors." )) 

      fi;

 	subspace:=map(evalV,args[1]);
	
	if type(subspace,list(Vessiot_vect))=false then
		ERROR(cat("\n VESSIOT ERROR:", 
                      "\n First argument for LieAlgCheck(*,",check,")  is list of vectors." )) 
            fi;

      if nops(DiffGeom_basis(subspace)) <>   nops(subspace) then
		ERROR(cat("\n LieAlg Error:", 
                      "\n Given list of vectors for LieAlgCheck(*,",check,") not independent." )) 
      fi;
 

	B:=LieAlgebrasTools:-bracket_of_subspaces(subspace,g0);


	S0:=subspace;
	
	#if check=ideal2 then 
	#	for t in args[2] do
	#	S0:=subs(t=0,S0)
	#	od;
	#fi;
	
	S0:=map(helmsimp,S0);

	C:=complementary_basis(S0,g0);

	forms:=dual_basis([op(C),op(subspace)]);
 
	eq:={}:

	for v in B do 
		for  i from 1 to nops(C) do 
      		ans:=coeff_list((hook(v,forms[i]),[[]]));
			if ans<>0 then 
				eq:={op(eq),ans}
            	fi;
		od;
	od;
	

      if check=ideal1 then
		if eq={} then 
			RETURN(true)
		else
			RETURN(false)
		fi;
      fi;

	if check=ideal2 then
           
		if type(vars,set(name))=false then
			ERROR(cat("\n LieAlg Error:", 
                      "\n LieAlgCheck(*,",check,") requires a set of variables",
			    "\n as the second argument." )) 
		fi;
	

		ans:=solve(eq,vars);
	
      	if ans=NULL then 
        		RETURN(false, eq)
      	else 
			B:=map(helmsimp,subs(ans,subspace));
	      	RETURN(true,eq, ans,B);
	      fi;
      fi;


fi;






#..........check_semisimple:true or false ..........

if check=semisimple then
	A:=Killing();
	d:=simplify(LinearAlgebra[Determinant](A));

	if d<> 0 then 
		ans:=true 
	else 
		ans:=false 
	fi;

     RETURN(ans);

fi;


#........... check_solvable: true or false ..........

if check=solvable then 
	
	flag:=_Vessiot_show_intermediate_steps_flag;
	_Vessiot_show_intermediate_steps_flag:=false;

	A:=LieAlg_series(derived);

	_Vessiot_show_intermediate_steps_flag:=flag;

	if A[-1]=[]  then 
		ans:=true 
	else 
		ans:=false 
	fi;
        
      RETURN(ans)
fi;

#.......... check nilpotent:true or false ..........

if check=nilpotent then
	
	flag:=_Vessiot_show_intermediate_steps_flag;
	_Vessiot_show_intermediate_steps_flag:=false;
	
	A:=LieAlg_series(lower);

	_Vessiot_show_intermediate_steps_flag:=flag;

	if A[-1]=[]  then 
		ans:=true 
	else 
		ans:=false 
	fi;

	RETURN(ans)

fi:

#........... check abelian ............


if check =abelian then
	A:=derived_algebra():

	if A=[]  then 
		ans:=true 
	else 
		ans:=false 
	fi;

	RETURN(ans)
fi:

#.............. check Lie_algebra_homomorphism1,check Lie_algebra_homomorphism2 ...........    

if check=Lie_algebra_homomorphism1 or check=Lie_algebra_homomorphism2 then


#Error Messages for check Lie_algebra_homomorphism1    

if check=Lie_algebra_homomorphism1 and nargs< 3 then 
	ERROR(cat("\n LieAlg Error:", 
		    "\n LieAlgCheck(*,Lie_algebra_homomorphism1) require 3 arguments:",
                "\n Domain algebra name, range algebra name, Matrix",
                "\n Number of arguments received:",
                "\n ", nargs-1 ) )
fi;

if check=Lie_algebra_homomorphism2 and nargs<4 then 
	ERROR(cat("\n LieAlg Error:", 
		    "\n LieAlgCheck(Lie_algebra_homomorphism2) require 4 arguments:",
                "\n Domain algebra name, range algebra name, Matrix, parmeter_set",
                "\n Number of arguments received:",
                "\n ", nargs-1) )
fi;

domain_alg:=args[1];
range_alg:=args[2];
A:=args[3];

if type(domain_alg,name)=false then
	ERROR(cat("\n LieAlg Error:", 
		    "\n First argument for LieAlgCheck(*,Lie_algebra_homomorphism*)",
		    "\n must be an initialized  Lie algebra name."))
fi;

if  member(domain_alg,listOfFrameNames())=false then
	ERROR(cat("\n LieAlg Error:", 
		    "\n First argument for LieAlgCheck(*,Lie_algebra_homomorphism*)",
		    "\n must be an initialized Lie algebra frame name."))
fi;

 
if type(range_alg,name)=false then
	ERROR(cat("\n LieAlg Error:", 
		    "\n Second argument for LieAlgCheck(*,Lie_algebra_homomorphism*)",
		    "\n must be an initialized Lie algebra name."))
fi;

if  member(range_alg,listOfFrameNames())=false then
	ERROR(cat("\n LieAlg Error:", 
		    "\n Second argument for LieAlgCheck(*,Lie_algebra_homomorphism*)",
		    "\n must be an initialized Lie algebra name."))
fi;
               	
if type(A,Matrix) =false then
	ERROR(cat("\n LieAlg Error:", 
		    "\n Third argument for LieAlgCheck(*,Lie_algebra_homomorphism*)",
		    "\n must be a Matrix."))
fi;
 

n := frameBaseDimension(domain_alg);
m := frameBaseDimension(range_alg);
 
if LinearAlgebra[ColumnDimension](A) <> n then
	ERROR(cat("\n LieAlg Error:",
      "\n Number of columns of  Matrix  must equal dimension of domain algebra(first argument):", 
                "\n " , LinearAlgebra[ColumnDimension](A), ",",   n  )) 
fi;
 
               
if LinearAlgebra[RowDimension](A) <> m then
	ERROR(cat("\n LieAlg Error:",
	"\n Number of rows of  Matrix  must equal dimension of range algebra(second argument):", 
                "\n " ,LinearAlgebra[RowDimension](A), ",",   m )) 
fi;

#
# Check that  Matrix  defines a Lie algebra homomorphism
#
Z:=zero_vect([],range_alg):
check_homo:=Matrix(n,n,(i,j)->Z);
 

# this code  could be improved

StructureArrayRange:=array(1..m,1..m,1..m,sparse):
StructureArrayDomain:=array(1..n,1..n,1..n,sparse):

for s in structure_constants_list(domain_alg) do
	StructureArrayDomain[op(s[1])]:=s[2];
	StructureArrayDomain[s[1][2],s[1][1],s[1][3]]:=-s[2]

od:

 
for s in structure_constants_list(range_alg) do
	StructureArrayRange[op(s[1])]:=s[2];
	StructureArrayRange[s[1][2],s[1][1],s[1][3]]:=-s[2]

od:

RangeBasis:=frameBaseVectors(range_alg):

for i to n do 
	for j from i + 1 to n do
    		for k to m do
                
	temp1 := 
add(add(StructureArrayRange[a,b, k]*A[ a,i]*A[b,j], b = 1 .. m), a = 1 .. m);
      
	temp2 := 
add(StructureArrayDomain[i, j,c]*A[k,c], c = 1 .. n); 
                   
	check_homo[i,j]:= check_homo[i,j] &plus
(simplify(temp1 - temp2) &mult RangeBasis[k]) ;

		od
	od
od;
   
eq:=op(convert(map(coeff_set, check_homo),set));

eq:=`union`(eq);

if check= Lie_algebra_homomorphism1 then  
	if eq = {0} then 
		RETURN(true) 
	else
		RETURN(false)
	fi;
fi;


if check= Lie_algebra_homomorphism2 then 
	if type(args[4],set)= false then 
		ERROR(cat("\n LieAlg Error:", 
		    "\n Fourth argument for LieAlgCheck(*,Lie_algebra_homomorphism*)",
		    "\n must be the set of parameters appearing the Matrix."))
fi;

	ans:=solve(eq, args[4]);
	RETURN(eq, ans) 	
fi;

fi;


#.......... check reductive pair ..........

if check= reductive_pair1 or check=reductive_pair2 then 

      if check=reductive_pair1 then
		if nargs<>3 then  
			ERROR(cat("\n LieAlg Error:", 
		    "\n LieAlgCheck(*,reductive_pair1) require 2 arguments:",
                "\n First argument is a list of vectors defining a subalgebra;",
                "\n Second argument is a list of vectors defining a complementary subspace.") )
		fi;
	fi;

	if check=reductive_pair2 then
		if nargs<>4 then  
		    ERROR(cat("\n LieAlg Error:", 
		    "\n LieAlgCheck(*,reductive_pair2) require 3 arguments:",
                "\n First argument is a list of vectors defining a subalgebra;",
                "\n Second argument is a list of vectors defining a complementary subspace;",
		    "\n Third argument is the set of parameters appearing in the complement."))
		fi;

		vars:=args[3];
	fi;


	if type(args[1],list)=false then
		ERROR(cat("\n LieAlg Error:", 
                      "\n First argument for LieAlgCheck(*,",check,") is a list of vectors." )) 

      fi;

 	subalg:=map(evalV,args[1]);

	if type(subalg,list(Vessiot_vect))=false then
		ERROR(cat("\n LieAlg Error:", 
                      "\n First arguement for LieAlgCheck(*,",check,")is a list of vectors", 
                      "\n defining the subalgebra." )) 
	fi; 	
#
# NOTE  RECURSIVE CALL TO LieAlgCheck.
#	
      if LieAlgCheck(subalg,subalgebra1)= false then 
		ERROR(cat("\n LieAlg Error:", 
                      "\n List of vectors given for the first argument to", 
		          " LieAlgCheck(*,",check,")", 
                      "\n do not define a subalgebra." )) 
	fi; 	


	if type(args[2],list)=false then
		ERROR(cat("\n LieAlg Error:", 
                  "\n Second argument for LieAlgCheck(*,",check,") is a list of vectors", 
			"\n defining the subspace." ))
      fi;

	subspace:=map(evalV,args[2]);

	if type(subspace,list(Vessiot_vect))=false then
		ERROR(cat("\n LieAlg Error:", 
                      "\n Second arguement for LieAlgCheck(*,",check,") is a list of vectors",
			    "\n defining the subspace." )) 
      fi;
      
      B:= [op(subalg), op(subspace)];
      
	if  nops(DiffGeom_basis(B)) <> frameBaseDimension() or
	    nops(B) <> frameBaseDimension()  then
		 ERROR(cat("\n LieAlg Error:", 
                       "\n The two list of vectors given to LieAlgCheck(*,",check,")", 
                       "\n do not form a basis for the Lie algebra." )) 
	fi; 	

      m:=map(helmsimp,subs({seq(vars[i]=0, i=1..nops(vars))},subspace));

	q:=nops(subspace);
	p:=nops(subalg);

	alg:=frameBaseVectors();

 
	eq:={};

	for a from 1 to q do  
		for i from 1 to p do 
			vec1:=Lie_bracket(subspace[a],subalg[i]); 
			temp:=linear_combo(Lie_bracket(m[a],subalg[i]),B);
			vec2:=v_zip(temp[p+1..p+q],subspace,plus);
			eq:={op(eq),op(coeff_set(vec1 &minus vec2))};
		od;
	od;
 
    

	if check=reductive_pair1 then
		if eq={0} then 
			RETURN(true)
		else 
		 	RETURN(false)
            fi;
	fi; 
        
	if check=reductive_pair2 then
		if type(vars,set(name))=false then
			ERROR(cat("\n LieAlg Error:", 
                      "\n LieAlgCheck(*,",check,") requires a set of variables",
			    "\n as the third argument." )) 
		fi;
	
		ans:=solve(eq, vars);

		if ans=NULL then 
			RETURN(false, eq)
		else
			B:=map(helmsimp,subs(ans,subspace));
	 		RETURN(true,eq,ans, B);
            fi;
      fi;


fi;



#.......... symmetric pair ...........

if check=symmetric_pair1 or check=symmetric_pair2  then


	if check=symmetric_pair1 then	
		if nargs<>3 then  
			ERROR(cat("\n LieAlg Error:", 
		    	"\n LieAlgCheck(*,symmetric_pair1) require 2 arguments:",
                	"\n First argument is a list of vectors defining a subalgebra;",
                	"\n Second argument is a list of vectors defining a complementary subspace.") )
		fi;
	fi;


	if check=symmetric_pair2 then	
		if nargs<>4 then  
			ERROR(cat("\n LieAlg Error:", 
		    	"\n LieAlgCheck(*,symmetric_pair1) require 3 arguments:",
                	"\n First argument is a list of vectors defining a subalgebra;",
                	"\n Second argument is a list of vectors defining a complementary subspace.", 
                  "\n Third argument is the set of parameters in the complementary subspace."))
		fi;
            vars:=args[3];
	fi;

	if type(args[1],list)=false then
		ERROR(cat("\n LieAlg Error:", 
                      "\n First argument for LieAlgCheck(*,",check,")is a list of vectors." )) 

      fi;

	subalg:=map(evalV,args[1]);

	if type(subalg,list(Vessiot_vect))=false then
		ERROR(cat("\n LieAlg Error:", 
                      "\n First argument for LieAlgCheck(*,",check,")is a list of vectors", 
                      "\n defining the subalgebra." )) 
	fi;
 	
	if LieAlgCheck(subalg,subalgebra1)= false then 
		ERROR(cat("\n LieAlg Error:", 
                      "\n List of vectors given for the first argument to", 
		          " LieAlgCheck(*,",check,")", 
                      "\n do not define a subalgebra." )) 
	fi; 	

	if type(args[2],list)=false then
		ERROR(cat("\n LieAlg Error:", 
                      "\n Second argument for LieAlgCheck(*,",check,")is a list of vectors." )) 

      fi;

	subspace:=map(evalV,args[2]);
	
	if type(subspace,list(Vessiot_vect))=false then
		ERROR(cat("\n LieAlg Error:", 
                      "\n Second argument for LieAlgCheck(*,",check,")is a list of vectors.",
			    "\n defining the subspace." )) 
	fi;

 	B:= [op(subalg), op(subspace)];
      
	if  nops(DiffGeom_basis(B)) <> frameBaseDimension() or
	    nops(B) <> frameBaseDimension()  then
		 ERROR(cat("\n LieAlg Error:", 
                       "\n The two lists of vectors given to LieAlgCheck((*,",check,")", 
                       "\n do not form a basis for the Lie algebra." )) 
	fi; 	

#
#  RECURSIVE CALL!!
#
	if not LieAlgCheck(subalg,subspace,reductive_pair1) then 
		ERROR(cat("\n LieAlg Error:",
			    "\n Subalgebra, subspace pair  given to LieAlgCheck(*,",check,")do",
			    " not form a reductive pair."))
	fi;

	S:=LieAlgebrasTools:-bracket_of_subspaces(subspace, subspace);
	forms:=dual_basis([op(subspace),op(subalg)]);

      eq:={};

	for v in S do
		for i from 1 to nops(subspace) do
			ans:= coeff_list(hook(v, forms[i]), [[]]);
			if ans <> 0 then 
				eq:={op(eq),ans}
			fi;
		od;
	od;


      if check=symmetric_pair1 then
		if eq={} then 
			RETURN(true)
		else 
		 	RETURN(false)
            fi;
	fi; 
        
	if check=symmetric_pair2 then
		if type(vars,set(name))=false then
			ERROR(cat("\n LieAlg Error:", 
                      "\n LieAlgCheck(*,",check,") requires a set of variables",
			    "\n as the fourth argument." )) 
		fi;
	
		ans:=solve(eq, vars);

		if ans=NULL then 
			RETURN(false, eq)
		else
			B:=map(helmsimp,subs(ans,subspace));
	 		RETURN(true,eq,ans, B);
            fi;
      fi;


fi;


#........... naturally reductive1 ..........

if check=naturally_reductive_pair1 or check=naturally_reductive_pair2 then
	
	if check=naturally_reductive_pair1 then
	
		if nargs<>4 then  
			ERROR(cat("\n LieAlg Error:", 
		    	"\n LieAlgCheck(*,naturally_reductive_pair1) require 3 arguments:",
                	"\n First argument is a list of vectors defining a subalgebra;",
                	"\n Second argument is a list of vectors defining a complementary subspace.",
		    	"\n Third argument is a Matrix defining an inner product on the subspace."	 
			) )
		fi;
      
	fi;

	if check=naturally_reductive_pair2 then
	
		if nargs<>5 then  
			ERROR(cat("\n LieAlg Error:", 
		    	"\n LieAlgCheck(*,naturally_reductive_pair2) require 4 arguments:",
                	"\n First argument is a list of vectors defining a subalgebra;",
                	"\n Second argument is a list of vectors defining a complementary subspace;",
		    	"\n Third argument is a Matrix defining an inner product on the subspace;",
                	"\n Fourth argument is a parameter set."	 
			) )
		fi;
      
		vars:=args[4];
	fi;

	if type(args[1],list)=false then
		ERROR(cat("\n LieAlg Error:", 
                      "\n First argument for",
			    " LieAlgCheck(*,",check,") is a list of vectors." )) 
	fi;
	
	if type(args[2],list)=false then
		ERROR(cat("\n LieAlg Error:", 
                      "\n Second argument for",
			    " LieAlgCheck(*,",check,") is a list of vectors." )) 
	fi;
	

	M:=  map(evalV,args[2]);
	Iso:=map(evalV,args[1]);
	H:=args[3];

	if type(Iso,list(Vessiot_vect))=false then
		ERROR(cat("\n  LieAlg Error:", 
                      "\n  LieAlgCheck(*,",check,") requires a list of vectors", 
                      "\n defining the subalgebra as the first argument." )) 
	fi;
	
	if type(M,list(Vessiot_vect))=false then
		ERROR(cat("\n LieAlg Error:", 
                      "\n LieAlgCheck(*,",check,") requires a list of vectors.",
			    "\n defining the subspace as the second argument." )) 
	fi;

	if type(H,Matrix)=false then
		ERROR(cat("\n LieAlg Error:", 
                      "\n LieAlgCheck(*,",check,") requires a Matrix.",
			    "\n defining the inner product  as the third argument." )) 
	fi;

      if LinearAlgebra[RowDimension](H) <> nops(M) or LinearAlgebra[ColumnDimension](H) <> nops(M) then
		ERROR(cat("\n LieAlg Error:", 
                      "\n LieAlgCheck(*,",check,") requires a n x n  Matrix.",
			    "\n where n is the dimension of the subspace defined by the second argument." )) 
	fi;



      if nops(DiffGeom_basis(Iso)) <> nops(Iso) then
		ERROR(cat("\n LieAlg Error:", 
            "\n Vectors given for first argument for LieAlgCheck(*,",check,")",
		"\n must be linearly independent.")) 
	fi;
		 
      if  LieAlgCheck(Iso,subalgebra1)=false then
		ERROR(cat("\n LieAlg Error:", 
            "\n Vectors given for first argument for LieAlgCheck(*,",check,")",
		"\n must define a subalgebra.")) 
	fi;


	m:=nops(M);

	basis:=[op(M), op(Iso)];
 
      if nops(DiffGeom_basis(basis)) <> nops(basis) or
	   nops(basis) <>frameBaseDimension()  then
		ERROR(cat("\n LieAlg Error:", 
            "\n Vectors given for first and second arguments for",
		"  LieAlgCheck(*,",check,")",
		"\n must define a basis for the Lie algebra.")) 
	fi;

	if  LieAlgCheck(Iso,M,reductive_pair1)=false then
		ERROR(cat("\n LieAlg Error:", 
            "\n Vectors given for first and second argument for",
		"\n LieAlgCheck(*,",check,")",
		"\n must define a reductive pair.")) 
	fi;


	for i from 1 to m do 
 		for j from i to m do
  			components:=linear_combo(Lie_bracket(M[i], M[j]),basis);
 	 		for k from 1 to m do
  				C1[i,j,k]:=components[k];
  				C1[j,i,k]:=-components[k];
  			od;
 		od;
	od;

	Eq:={};

	for i from 1 to m do 
		for j from 1 to m do 
			for k from 1 to m do
 				eq:=sum (C1[i,j,a]*H[k,a] +C1[i,k,a]*H[j,a], a=1..m);
 				Eq:=Eq union {eq};
 			od;
		od;
	od;

      if check = naturally_reductive_pair1 then 
		if Eq= {0}then 
			RETURN(true) 
		else  
			RETURN(false) 
		fi;
      fi;
	
      if check = naturally_reductive_pair2  then
		if type(vars,set(name))=false then
			ERROR(cat("\n LieAlg Error:", 
                      "\n LieAlgCheck(*,",check,") requires a set of variables",
			    "\n as the fourth  argument." )) 
		fi;
		
		ans:=solve(Eq, vars); 
      
		if ans= NULL then 
			RETURN(false,Eq) 
		else 
			B:=map(helmsimp,subs(ans,M));
	 		RETURN(true,Eq,ans,B); 
            fi;
	fi;

	
	







fi;




if check =graded_Lie_algebra then		
		g:=args[1];
		N:=indices(g); 
		for i in N do 
			for j in N do 
				p:=op(i); 
				q:=op(j);
				A:=LieAlgebrasTools:-bracket_of_subspaces(g[op(p)],g[op(q)]); 

				if A<>[]  and member([p+q], [N]) then 
					S:=g[p+q]; 
					C:=linear_combo(A,S); 
					if C=[] then RETURN(false) fi; 
				fi;
		
				if A  <> [] and not(member([p+q], [N])) then RETURN(false) fi; 
			od; 
		od; 
      	
		RETURN(true); 
   
	fi;
#............

















ERROR(cat("\n LieAlg Error:", 
          "\n Last argument for LieAlgCheck must be:",
"\n {abelian,ideal1, ideal2, Jacobi1, Jacobi2, Lie_algebra_homomorphism1, Lie_algebra_homomorphism2",
  "\n  indecomposable, absolutely_indecomposable",
"\n  naturally_reductive_pair1, naturally_reductive_pair2, nilpotent,graded_Lie_algebra ", 
"\n  semisimple, solvable, subalgebra1, subalgebra2, reductive_pair1, reductive_pair2,", 
"\n  symmetric_pair1, symmetric_pair2}" ));  

end:

#********************************************************************