#
# VessiotConvert.alg
#
# LieAlg 
#
# convert/vector_field_system; convert/Lie_algebra_data
#


                                


#~~~~~~~~~~~~~~~~~~  convert/vector_field_system  ~~~~~~~~~~~~~~~~~~

`convert/vector_field_system`:=proc(alg_name, frame_name,ad_type)  

# CLASS II PROGRAM
# 01/10/2004 IA

option `Copyright Ian M. Anderson 2004`; 

description `compute adjoint and co-adjoint vector field representations `;

local i, j,Gamma,X,fr, n,z;

if nargs <>3 then 
	ERROR(cat("\n LieAlg Error:",
                "\n convert/vector_field_sytems requires three arguments:",
                "\n Lie algebra name, coordinate frame name, representation type" ))
fi;

if member(alg_name, listOfFrameNames())=false then
	ERROR(cat("\n LieAlg Error:",
                "\n First argument for convert/vector_field_sytems must be an",
	          "\n initialized Lie algebra name:",
                "\n ", alg_name))
fi;

if member(frame_name, listOfFrameNames())=false then
	ERROR(cat("\n LieAlg Error:",
                "\n Second argument for convert/vector_field_sytems must be an",
		    "\n initialized coordinate frame name:",
                "\n ", frame_name))
fi;

if  member(ad_type, {adjoint, coadjoint})=false then
	ERROR(cat("\n LieAlg Error:",
                "\n Third argument for convert/vector_field_sytems must be",
                " {adjoint, coadjoint}",
	          "\n ", ad_type))
fi;  


if frameBaseDimension(alg_name) <> frameBaseDimension(frame_name) then
	ERROR(cat("\n LieAlg Error:",  
		    "\n convert/vector_field_system:",
		    "\n The base dimension of the coordinate frame must equal",
		    " the dimension of the Lie algebra:", 
		    "\n ",frameBaseDimension(alg_name), ",", frameBaseDimension(frame_name) ))
fi;

n:= frameBaseDimension();

Gamma:=[];


z:=frameIndependentVariables(frame_name);


if ad_type='adjoint' then

for i from 1 to n do

	X:=zero_vect([],frame_name);
	for j from 1 to n  do
		X:=X &plus   
		( add(structure_constant(i,k,j,alg_name)  *z[k], k=1..n)  	
		&mult Vect(j,frame_name)         )
	od;

	Gamma:=[op(Gamma), X];
od;

fi;

#......................

if ad_type='coadjoint' then

	for i from 1 to n do
		X:=zero_vect([],frame_name);
			for j from 1 to n do
				X:=X &plus   
				( add(structure_constant(i,j,k,alg_name)  *z[k], k=1..n)  
				&mult Vect(j,frame_name))
			od;

	Gamma:=[op(Gamma), X];

	od;
fi;


Gamma;

end:



#~~~~~~~~~~~~~~~~~ convert/Lie_algebra_data ~~~~~~~~~~~~~~~~~~~~~~


`convert/Lie_algebra_data`:=proc(alg)

# CLASS II PROGRAM
# 01/10/2004 IA

option `Copyright Ian M. Anderson 2004`; 

description `convert various Lie algebras to Vessiot Lie algebra data structure `;


local structure_constants_array_to_Lie_algebra_data,
	matrix_algebra_to_Lie_algebra_data,
	structure_equations_to_Lie_algebra_data,
	Lie_algebra_name_to_Lie_algebra_data,
	subalgebra_pair_to_Lie_algebra_data_pair,subalgebra_to_Lie_algebra_data,
	ans,algV, alg2V, x ;



#
#  LOCAL  ROUTINES 
#


#........... structure_constants_array_to_Lie_algebra_data .........

structure_constants_array_to_Lie_algebra_data:=proc(A)

local i,j,k, size, n, str_eq,alg_name;

size:=op(2,eval(A));
n:=op(2,size[1]);

if  nops(convert('size',list)) <>3 then  
	ERROR(cat("\n LieAlg Error:", 
	          "\n convert/Lie_algebra_data(array):", 
		    "\n The array  must be of be of rank 3." ))
fi;

if  op(2,size[2]) <> n or op(2,size[3]) <> n then
	ERROR(cat("\n LieAlg Error:", 
	          "\n convert/Lie_algebra_data(array):", 
		    "\n The array  must be of equal dimensions." ))
fi;

if nargs=2 then 
	alg_name:=args[2];
	if type(alg_name,name)=false then 
		ERROR(cat("\n LieAlg Error:", 
	          	    "\n The  second argument for convert/Lie_algebra_data(array)", 
		    		"\n must be an unassigned name." ))
	fi;
else 
	alg_name:=`Koszul`
fi;
	


str_eq:=[];

for i from 1 to n do 
	for j from i+1 to n do 
		for k from  1 to n do
            if A[i,j,k]<> 0 then 
            	str_eq:=[op(str_eq),[[i,j,k], A[i,j,k]] ]
            fi;
            od;
	od;
od;

[[Lie_alg,alg_name,[n]] ,str_eq]

end:

#............  matrix_algebra_to_Lie_algebra_data ...........


matrix_algebra_to_Lie_algebra_data:=proc(basis_list)

local i,j,k,bracket_coeffs, C1, C2,C3,structure_constant_data,alg_name,zero_matrix;


if type(basis_list, list('Matrix'(square)  ) )=false then 
	ERROR(cat("\n LieAlg Error:" ,	         
		      "\n convert/Lie_algebra_data(list(Matrix)):",
		      "\n The matrices must be square.") )
fi;


for i from 2 to nops(basis_list) do 
	if LinearAlgebra[ColumnDimension](basis_list[i])<>LinearAlgebra[ColumnDimension](basis_list[1]) 	
	then ERROR(cat("\n LieAlg Error:" ,
	               "\n convert/Lie_algebra_data(list(Matrix)):",
			   "\n Each Matrix  must be of the same dimension.")) 
	fi 
od;


if nargs=2 then 
	alg_name:=args[2];
	if type(alg_name,name)=false then 
		ERROR(cat("\n LieAlg Error:", 
	          	    "\n The  second argument for convert/Lie_algebra_data(list(Matrix))", 
		    		"\n must be an unassigned name." ))
	fi;
else 
	alg_name:=`Koszul`
fi;

zero_matrix:=
Matrix(LinearAlgebra[ColumnDimension](basis_list[1]),LinearAlgebra[ColumnDimension](basis_list[1]),0);

if convert(linear_combo(zero_matrix,basis_list),set)<>{0} then
	ERROR(cat("\n LieAlg Error:" ,
	          "\n convert/Lie_algebra_data(list(Matrix)):",
		    "\n The matrices must be linearly independent."  )) 
fi;
 
structure_constant_data:=[];

for i to nops(basis_list) do
    for j from i+1 to nops(basis_list) do
       C1:= LinearAlgebra[MatrixMatrixMultiply](basis_list[i] ,basis_list[j]);
       C2:= LinearAlgebra[MatrixMatrixMultiply](basis_list[j] ,basis_list[i]);
       C3:= LinearAlgebra[MatrixAdd](C1,C2,1,-1);
       bracket_coeffs:=linear_combo(C3,basis_list);
       if bracket_coeffs=[] then 
	 	ERROR(cat("\n LieAlg Error:" ,
	          	    "\n convert/Lie_algebra_data(list(Matrix)):",
		    	    "\n The matrices  must  not closed under Matrix brackets",
		    	    "\n ", i, ",",j )) 
	 fi;
       for k to nops(bracket_coeffs) do
       	if bracket_coeffs[k]<>0 then
            	structure_constant_data:=
			[op(structure_constant_data),[[i,j,k],bracket_coeffs[k]]];
         	fi;
        	od;
    od;
od;


[[Lie_alg,alg_name,[nops(basis_list)]],structure_constant_data];

end:

#............... structure_equations_to_Lie_algebra_data ..............

structure_equations_to_Lie_algebra_data:= proc (str_eq,frame) 

local alg_name, eq, i, j, k, C, str_eq_list; 

if nargs<2 then  
	ERROR(cat("\n LieAlg Error:",
                "\n convert/Lie_algebra_data(list(name)) 2 or 3 arguments:",
                "\n list of list of structure equations, basis vectors, Lie algebra name" ))
fi;


if type(str_eq,list(equation))=false then 
	ERROR(cat("\n LieAlg Error:",
		    "\n  First argument for convert/Lie_algebra_data(list(equations)) must be",
                "\n a list of equations,", 
		    "\n ", "for example, [e1,e2] =e1" ))
fi;


if type(frame,list(name))=false then 
	ERROR(cat("\n LieAlg Error:",
		    "\n Second argument for convert/Lie_algebra_data(list(equations)) must be",
                "\n a list of unassigned names for Lie algebra basis."))
fi;


if nargs=3 then 
	alg_name:=args[3];
	if type(alg_name,name)=false then 
		ERROR(cat("\n LieAlg Error:", 
	          	    "\n The  third argument for convert/Lie_algebra_data(list(equation))", 
		    		"\n must be an unassigned name." ))
	fi;
else 
	alg_name:=`Koszul`
fi;


str_eq_list := []; 

for eq in str_eq do 
	member(lhs(eq)[1],frame,'i'); 
	member(lhs(eq)[2],frame,'j'); 
	for k to nops(frame) do 
		C := diff(rhs(eq),frame[k]); 
		if C <> 0 then 
			str_eq_list := [op(str_eq_list), [[i, j, k], C]] 
		fi 
	od 
od; 

[[Lie_alg, alg_name, [nops(frame)]], str_eq_list] 

end:

#==============================================================

Lie_algebra_name_to_Lie_algebra_data:=proc(alg_name)

if member(alg_name,listOfFrameNames())=false then
	ERROR(cat("\n LieAlg Error:",
		    "\n First argument for convert/Lie_algebra_data(name) must be an initialized",
		    "\n Lie algebra name."))

fi;

[[Lie_alg, _Vessiot_current_frame_name, [frameBaseDimension()]], structure_constants_list()]

end:

#================================================================



subalgebra_to_Lie_algebra_data := proc(subalg0)

#
# This routine   should be rewritten!
#



local i, j, n,z_vect ,struc_const, C, subalg,
	V,L,ans, k,h,alg_name,brackets,masterAlgebra,phi,A;

subalg:=map(evalV,subalg0);


if nops(DiffGeom_basis(subalg))<>nops(subalg) then
	ERROR(cat("\n LieAlg Error:", 
		    "\n Given list of vectors not linearly independent."))
fi;

masterAlgebra:=_Vessiot_current_frame_name;

n := nops(subalg);



if nargs>=2 then 
	alg_name:=args[2];
	if type(alg_name,name)=false then 
		ERROR(cat("\n LieAlg Error:", 
	          	    "\n The  second argument for convert/Lie_algebra_data(list(vectors)", 
		    		"\n must be an unassigned name." ))
	fi;
else 
	alg_name:=`Koszul`
fi;


struc_const := [];

brackets:=[seq(seq([i,j,Lie_bracket(subalg[i],subalg[j])],j=i+1..n),i=1..n)];

#..... 

z_vect:=proc(v) 
	if op(v[3])[2][1][2]=0 then 
	 	RETURN(true) 
	fi; 
false 

end;

#.....

brackets:=remove(z_vect,brackets);


if brackets<>[]  then               
	C := linear_combo(map2(op,3,brackets),subalg);
      if C=[] then 
		ERROR(cat("\n LieAlg Error:",
                      "\n List of vectors for `convert/Lie_algebra_data` do",
		          "\n not form a subalgebra."))  
      fi;


	for h from 1 to nops(brackets) do
		for k to n do                      
			if C[h][k] <> 0 then struc_const :=
                        [op(struc_const), [[brackets[h][1], brackets[h][2], k], C[h][k]]]
            	fi
		od
	od

fi;


ans:=[[Lie_alg,alg_name,[n]],struc_const];

if nargs =3 then  
 	L:=frameBaseVectors();
 	A:=LinearAlgebra[Transpose](Matrix(linear_combo(subalg,L)));
      RETURN(ans,eval(A)) 
fi;

ans;

end:


#...........  subalgebra_pair_to_Lie_algebra_data_pair ..........

subalgebra_pair_to_Lie_algebra_data_pair := proc (mother_alg0, baby_alg0) 


local h, C, D, c,alg_name, mother_alg_data,basis,ans,L,A,mother_alg, baby_alg;




mother_alg:=map(evalV,mother_alg0);

baby_alg:=map2(map,evalV,baby_alg0);



if nops(DiffGeom_basis(mother_alg))<>nops(mother_alg) then
	ERROR(cat("\n LieAlg Error:", 
		    "\n First list of vectors not linearly independent."))
fi;




C := [];
 

for h in baby_alg while true do 
       if  h =[] then 
		C:=[op(C),[]]
	else   
		x:=linear_combo(h,mother_alg);
            if x=[] then  
			ERROR(cat("\n LieAlg Error:", 
		    		    "\n Second list of vectors not in span of first set."))
            fi;   
		C := [op(C), linear_combo(h,mother_alg)] 
      fi; 
od; 




if nargs>2 then 
	alg_name:=args[3];
	if type(alg_name,name)=false then 
		ERROR(cat("\n LieAlg Error:", 
	          	    "\n The  third argument for convert/Lie_algebra_data(list(vector), list(list(vector))", 
		    		"\n must be an unassigned name." ))
	fi;
else 
	alg_name:=`Koszul`
fi;


if nargs =4 then  
 	L:=frameBaseVectors();
 	A:=LinearAlgebra[Transpose](Matrix(linear_combo(mother_alg,L)));
 	ans:=ans,A

fi;

mother_alg_data:=convert(mother_alg, Lie_algebra_data, alg_name);

Lie_algebra_init(mother_alg_data); 

basis:=frameBaseVectors();
D := []; 

for c in C while true do 
	D := [op(D), map(v_zip,c,basis,plus)] 
od; 

ans:=mother_alg_data,op(D); 


if nargs =4 then  
 	RETURN(ans,eval(A))
fi;


ans; 

end:

#........................................

#
# RETURN TO MAIN PROGRAM
#



if nargs=0 then 
	ERROR(cat("\n LieAlg Error:", 
	          "\n convert/Lie_algebra_data requires at least 1 argument. "))
fi;


if type(alg,name) then
	ans:= Lie_algebra_name_to_Lie_algebra_data(args);
      RETURN(ans)
fi;

if type(alg,array) then 
	ans:=structure_constants_array_to_Lie_algebra_data(args);
      RETURN(ans)
fi;

if type(alg,list) then
	algV:=map(evalV,alg)
fi;

if type(algV,list(Matrix)) then
	ans:= matrix_algebra_to_Lie_algebra_data(args);
      RETURN(ans)
fi;

if type(algV,list(equation)) then
	ans:= structure_equations_to_Lie_algebra_data(args);
      RETURN(ans)
fi;


if type(algV,list(Vessiot_vect)) and nargs =1 then
	ans:= subalgebra_to_Lie_algebra_data(algV);
	RETURN(ans)
fi;

if type(algV,list(Vessiot_vect)) and nargs >=2 then
	
	if type(args[2],name) then
		ans:= subalgebra_to_Lie_algebra_data(args);
		RETURN(eval(ans))
	fi;
 	
      if  type(args[2], list(list)) then
		alg2V:=map2(map,evalV,args[2]);
		
		if type(alg2V,list(list(Vessiot_vect))) =false then 
            	ERROR(cat("\n LieAlg Error:", 
				    "\n if first argument to convert/Lie_algebra_data is a",  
				    "\n list of vectors, then the second arguement must be one of:",
				    "\n {Lie algebra name, list of list of vectors}." ))	 	
		fi;
		
		ans:=subalgebra_pair_to_Lie_algebra_data_pair(args);  
		RETURN(ans)
	fi;

fi;

ERROR(cat("\n LieAlg Error:", 
	    "\n Invalid argument to convert/Lie_algebra_data:",
	    "\n First argument=initialize Lie algebra name;", 
          "\n First argument=list of matrices;",
	    "\n First argument=structure constant array;",
          "\n First argument=structure equations; Second argument=list of unassign basis names;",
          "\n First argument=list of vectors;",
          "\n First argument=list of vectors; Secong argument=list of list of vectors."))


end:




#~~~~~~~~~~~~~~~~~~~~~ matrix_to_Lie_algebra_transform ~~~~~~~~~~~~~~~~~~~

`convert/Lie_algebra_transform` := proc(A,domain_alg, range_alg)

# CLASS II PROGRAM
# 01/10/2004 IA

option `Copyright Ian M. Anderson 2004`; 

description `converts a Matrix to a transformation between Lie algebras`;

local i, j, name_list, n, m, temp1, temp2, domain_index, check,
	range_index, a, b, c, ans, k,trans_eq1, trans_eq2;


#........... Error Messages ............

if nargs<3 then 
	ERROR(cat("\n LieAlg Error:", 
		    "\n matrix_to_Lie algebra_transform require 3 arguments:",
                "\n Domain algebra name, range algebra name, Matrix",
                "\n Number of arguments received:",
                "\n ", nargs )) 
fi;

if type(domain_alg,name)=false then
	ERROR(cat("\n LieAlg Error:", 
		    "\n First argument for matrix_to_Lie algebra_transform",
		    "\n must be a Lie algebra frame name"))
fi;


if  member(domain_alg,listOfFrameNames())=false then
	ERROR(cat("\n LieAlg Error:", 
		    "\n First argument for matrix_to_Lie algebra_transform",
		    "\n must be a Lie algebra frame name:", 
                "\n ", domain_alg))
fi;


if  member(domain_alg,listOfFrameNames())=false then
	ERROR(cat("\n LieAlg Error:", 
		    "\n First argument for matrix_to_Lie algebra_transform",
		    "\n must be a Lie algebra frame name:", 
                "\n ", domain_alg))
fi;

 
if type(range_alg,name)=false then
	ERROR(cat("\n LieAlg Error:", 
		    "\n Second argument for matrix_to_Lie algebra_transform",
		    "\n must be a Lie algebra frame name"))
fi;

if  member(range_alg,listOfFrameNames())=false then
	ERROR(cat("\n LieAlg Error:", 
		    "\n Second argument for matrix_to_Lie algebra_transform",
		    "\n must be a Lie algebra frame name:",
                "\n ", range_alg ))
fi;
               	
if type(A,Matrix)=false then 
	ERROR(cat("\n LieAlg Error:", 
		    "\n Third argument for matrix_to_Lie algebra_transform",
		    "\n must be a Matrix:"))
fi;

	
n := frameBaseDimension(domain_alg);
m := frameBaseDimension(range_alg);
 
if LinearAlgebra[ColumnDimension](A) <> n then
	ERROR(cat("\n LieAlg Error:",
                "\n Number of columns of  Matrix  must equal dimension of domain algebra:", 
                "\n " ,LinearAlgebra[ColumnDimension](A), ",",   n)) ;
fi;
               
if LinearAlgebra[RowDimension](A) <> m then
	ERROR(cat("\n LieAlg Error:",
		   "\n Number of rows of  Matrix  must equal dimension of range algebra:", 
                "\n " ,LinearAlgebra[RowDimension](A), ",",   m)) 
fi;

if LieAlgCheck(domain_alg,range_alg, eval(A), Lie_algebra_homomorphism1)=false then
 	ERROR(cat("\n LieAlg Error:", 
		    "\n Matrix does not preserve Lie_brackets.",
		    "\n Lie algebra transformation not defined.")) 
fi;

trans_eq1:= 
	[seq([add(A[i, j]*cat(z,j),j = 1 .. LinearAlgebra[ColumnDimension](A)), cat(z,i)], 
      			         i = 1 .. LinearAlgebra[RowDimension](A))];


ans := _VESSIOT([[helm_map,[[domain_alg,0], [range_alg,0]],[],[A]], trans_eq1]);

end:


















#****************************************************