#
# Lie_algebra_transform.alg
#
# LieAlg
#
# Lie_algebra_transform; transformation_subalgebra;
#
#~~~~~~~~~~~~~~~~~ Lie_algebra_transform ~~~~~~~~~~~~

Lie_algebra_transform:= proc(trans,vf0)

# CLASS II PROGRAM
# 01/10/2004 IA

option `Copyright Ian M. Anderson 2004`; 

description `map a vector, form or tensor on a Lie algebra`;

local A, psi, subalg,ans,vf;


if nargs<> 2 then 
	ERROR(cat("\n LieAlg Error:",
		"\n Lie_algebra_transform requires two arguments:",
		"\n Lie algebra transform, {vector, form or tensor}:",
            "\n Number of arguments recieved:",
		"\n ", nargs))
fi;
 
if type(trans,Vessiot_transformation) =false then             
	ERROR(cat("\n LieAlg Error:", 
		"\n First argument for Lie_algebra_transform must be a transformation:",
		"\n ", objectType(trans) ))
fi;

vf:=evalV(vf0);

if type(vf,{Vessiot_vect, Vessiot_form, Vessiot_tensor}) =false then             
	ERROR(cat("\n LieAlg Error:", 
		    "\n Second argument for Lie_algebra_transform must be a:", 
	          "\n {vector, form, tensor}."))
fi;


if transformDomain(trans)[1] <> objectFrame(vf) then
	ERROR(cat("\n LieAlg Error:", 
		    "\n Lie_algebra transform requires the domain frame of first argument", 
		    "\n to agree with object frame of second argument:", 
	          "\n ",transformDomain(trans)[1], " , ",  objectFrame(vf) ))
fi;


if objectType(vf) = vect then
        ans := pushforward(trans, vf);
        RETURN(ans)
fi;
    

if objectType(vf) = form then
        ans := pullback(trans, vf);        
        RETURN(ans)
fi;


#
# This could be generalized to  handle tensors not of mixed type -- ie inverse not needed.
#

if objectType(vf)=tensor then 
	A:=Jacobian_matrix(trans);
	if _Vessiot_simplify(LinearAlgebra[Determinant](A)) = 0 then 
		ERROR(cat("\n LieAlg Error:",
			    "\n Lie_algebra_transform on tensors requires",
			    " invertible transformation." ))
	fi;
      
	psi:=inverse_transform(trans);

	with(tensors, push_pull_tensor);
 	
	ans:=push_pull_tensor(trans, psi, vf);
      RETURN(ans)    
fi;


end:

#~~~~~~~~~~~~~~~ transformation_subalgebra ~~~~~~~~~~~~~~~~~


transformation_subalgebra:=proc(phi,subalg_type)

# CLASS II PROGRAM
# 01/10/2004 IA

option `Copyright Ian M. Anderson 2004`; 

description `Compute kernel,image,inverse-image subalgebras`;





local kernel_subalgebra, image_subalgebra, inverse_image_subalgebra,subalg,
	A,N,B,K,v,c,domainAlgebra, 	imageVectors,C,S,inverseImageAlgebra, rangeAlgebra ; 

if nargs<2then 
	ERROR(cat("\n LieAlg Error:", 
		    "\n Transformation_subalgebra requires two arguments:",
		    "\n transformation, subalgebra_type={kernel,image, inverse_image}." ))
fi;


if type(phi, Vessiot_transformation) =false then 
	ERROR(cat("\n LieAlg Error:", 
		    "\n First argument for transformation_subalgebra must be a",
		    "\n Lie algebra transformation",
		    "\n ", objectType(phi) ))
fi;

 
if  member(subalg_type, {kernel, image, inverse_image}) =false then 
	ERROR(cat("\n LieAlg Error:", 
		    "\n Second argument for transformation_subalgebra must be one of",
		    "  {kernel,image, inverse_image}:",
		    "\n ",  subalg_type ))
fi;


#............. kernel_subalgebra ...........

if subalg_type='kernel' then

	domainAlgebra:=transformDomain(phi)[1];
	A:=transformJacobian(phi):
	N:=Vessiot_nullspace(A);
	B:=frameBaseVectors(domainAlgebra);

	K:=[];

	for v in N do 
		c:=convert(v,list);
		K:=[op(K), v_zip(c,B,plus)];
	od;
     
	RETURN(K);

fi;


#--------------------------------------------

if  subalg_type='inverse_image'  then
	
	if nargs<>3 then 
		ERROR(cat("\n LieAlg Error:",
			    "\n Transformation_subalgebras(inverse_image) requires 3 argument."))
	fi;

 	if type(args[3], list)=false then
		ERROR(cat("\n LieAlg Error:",
			    "\n Third argument for transformation_subalgebras(inverse_image) must be a",
			    " a list of vectors in the range Lie algebra."))
	fi;	
	
	subalg:=map(evalV,args[3]);

	
 	if type(subalg, list(Vessiot_vect))=false then
		ERROR(cat("\n LieAlg Error:",
			    "\n Third argument for transformation_subalgebras(inverse_image) must be a",
			    " a list of vectors in the range Lie algebra."))
	fi;		
		
      if objectFrame(subalg[1])<> transformRange(phi)[1] then
		ERROR(cat("\n LieAlg Error:",
			    "\n Third argument for transformation_subalgebras(inverse_image) must be a",
			    " a list of vectors in the range Lie algebra."))
	fi;		
			


	domainAlgebra:=transformDomain(phi)[1];
	change_frame_to(domainAlgebra);

	K:=transformation_subalgebra(phi,kernel);
	B:=complementary_basis(K,frameBaseVectors());

	imageVectors:=map2(Lie_algebra_transform,phi,B);

	S:=DifferentialGeometryTools:-IntersectSubspaces([subalg,imageVectors]);

	if S=[] then 
		C:=[] 
	else
		C:=linear_combo(S,imageVectors);
      fi;

	inverseImageAlgebra:=K;

	for v in C do
		inverseImageAlgebra:=[op(inverseImageAlgebra),v_zip(v,B,plus)];
	od;

	RETURN(inverseImageAlgebra)
 
fi;

#............. image ..............


if subalg_type='image' then
	rangeAlgebra:= transformRange(phi)[1];
	A:=transformJacobian(phi);
#
# NEED  VESSIOT_COLSPACE
#
	N:=LinearAlgebra[ColumnSpace](A);
	B:=frameBaseVectors(rangeAlgebra);
	K:=[];
	for v in N do 
		c:=convert(v,list);
		K:=[op(K), v_zip(c,B,plus)];
	od;
	RETURN(K);
fi;


end:

#*****************************************************