#
# MatrixAlgebras.alg
#
# LieAlg
#
# matrix_algebras
#

matrix_algebras:=proc(alg_type)


# CLASS II PROGRAM
# 01/29/2001 JJ
# 01/10/2004 IA

option `Copyright Ian M. Anderson 2004`; 

description `Computes various Matrix subalgebras of the gl(n,R)`;

local B0,full_gl,upper_gl,strictly_upper_gl,gl_subalgebra,ans,invariants;

#
# LOCAL SUBROUTINES
#


#............ full_gl ...........

full_gl:=proc(n::posint,pretty_flag)

local alg_data, fr, basis, dual_basis, index_list, list_array, i, j;

alg_data:=0;

## We check that our basis names are not assigned --- needs work! ##

if assigned(e) then 
	ERROR(cat("\n LieAlg Error:",
		    "\n e is used by gl(n) as a basis vector.",
		    "\n Unassign it before calling matrix_algebras." )) 
fi;

if assigned(epsilon) then 
	ERROR(cat("\n LieAlg Error:", 
		    "\n epsilon is used by gl(n) as a basis dual-vector.",
		    "\n Unassign it before calling matrix_algebras. ")) 
fi;

## first we define a basis ##

index_list:=[  ];
list_array:=Matrix(n,n,0);
                                                                                                             
## The idea here is to generate a list of (array) indices ##
## and an array of (list) entry numbers ##
## i.e. if [ i, j ] is the kth element of index_list then k is the i, j th element of list_array ##
## and conversely  ##
## This allows us to go back and forth between the number of the algebra element ##
## and the formula for the brackets (which is based on the indices) ##

basis:=[seq(seq(cat(e,i,j),j=1..n),i=1..n)];
dual_basis:=[seq(seq(cat(epsilon,i,j),j=1..n),i=1..n)];

for i from 1 to n do
	for j from 1 to n do
		index_list:=[ op(index_list), [ i, j ] ];
		list_array[ i, j ]:=nops(index_list);
	od
od;


unassign(op(basis), op(dual_basis));


## Print messages explaining bases ##

if _Vessiot_show_intermediate_steps_flag=true then
	print( e[ `ij` ],`Is The vector basis, where `, e[ `ij` ] , 
	      `is the Matrix with 1 in the i th row, j th column, zeros elsewhere.` );
	print( epsilon[ `ij` ] , `Is the dual basis.` );
fi;

	

## Now we define the brackets using the formula : [ eij, ehk ] = djh*eik-dki*ehj ##
## where the d is the Kronecker delta function ##

alg_data:=[  ];

for i from 1 to n^2 do
	for j from i+1 to n^2 do
	
		if index_list[ i ] [ 2 ]=index_list[ j ] [ 1 ] then
			alg_data:=[ op(alg_data), 
				    [ [ i,  j, list_array[ index_list[ i ] [ 1 ], index_list[ j ] [ 2 ] ] ], 1 ] ];
		fi;

		if index_list[ i ] [ 1 ]=index_list[ j ] [ 2 ] then
			alg_data:=[ op(alg_data), 
				    [ [ i,  j, list_array[ index_list[ j ] [ 1 ], index_list[ i ] [ 2 ] ] ], -1 ] ];	
		fi;
	od;
od;

fr:=cat(`gl`, convert(n,string),'R');
	
if nargs=2 then
	if pretty_flag=pretty then 
		fr:=cat(`gl(`, convert(n,string),`,R)`); 
	fi;
fi;

[[ Lie_alg,fr, [ nops(index_list) ] ], alg_data ],basis, dual_basis ;

end:
#......... end of full_gl ..........



#........... upper_gl ............

upper_gl:=proc(n::posint,pretty_flag)

local alg_data, fr, basis, dual_basis, index_list, list_array, i, j;

alg_data:=0;

## We check that our basis names are not assigned --- needs work! ##

if assigned(e) then 
	ERROR(cat("\n LieAlg Error:",
		    "\n e is used by gl(n) as a basis vector.",
		    "\n Unassign it before calling matrix_algebras." )) 
fi;

if assigned(epsilon) then 
	ERROR(cat("\n LieAlg Error:", 
		    "\n epsilon is used by gl(n) as a basis dual-vector.",
		    "\n Unassign it before calling matrix_algebras. ")) 
fi;


## first we define a basis ##

index_list:=[  ];
list_array:=Matrix(n,n,0);

## The idea here is to generate a list of (array) indices ##
## and an array of (list) entry numbers ##
## i.e. if [ i, j ] is the kth element of index_list then k is the i, j th element of list_array ##
## and conversely ##
## This allows us to go back and forth between the number of the algebra element ##
## and the formula for the brackets (which is based on the indices) ##

basis:=[seq(seq(cat(e,i,j),j=i..n),i=1..n)];
dual_basis:=[seq(seq(cat(epsilon,i,j),j=i..n),i=1..n)];
                                                                                                             
for i from 1 to n do
	for j from i to n do
		index_list:=[ op(index_list), [ i, j ] ];
		list_array[ i, j ]:=nops(index_list);

	od;
od;

unassign(op(basis), op(dual_basis));

## Print messages explaining bases ##

if _Vessiot_show_intermediate_steps_flag=true then
	print( e[ ij ],`Is The vector basis, where `, e[ ij ] , 
		`is the Matrix with 1 in the i th row, j th column, 0 elsewhere.` );
	print( epsilon[ ij ] , `Is the dual basis.` );
fi;

	

## Now we define the brackets using the formula : [ eij, ehk ] = djh*eik-dki*ehj ##
## where the d is the Kronecker delta function ##

alg_data:=[  ];

for i from 1 to nops(index_list) do
	for j from i+1 to nops(index_list) do
		
		if index_list[ i ] [ 2 ]=index_list[ j ] [ 1 ] then
			alg_data:=[ op(alg_data), 
				    [ [ i,  j, list_array[ index_list[ i ] [ 1 ], index_list[ j ] [ 2 ] ] ], 1 ] ];
		fi;

		if index_list[ i ] [ 1 ]=index_list[ j ] [ 2 ] then
			alg_data:=[ op(alg_data), 
				    [ [ i,  j, list_array[ index_list[ j ] [ 1 ], index_list[ i ] [ 2 ] ] ], -1 ] ];
		fi;

	od;
od;

fr:=cat(`upper_gl`, convert(n,string));

if nargs=2 then
	if pretty_flag=pretty then 
		fr:=cat(`upper_gl(`, convert(n,string),`)`); 
	fi;
fi;

[ [ Lie_alg,fr, [ nops(index_list) ] ], alg_data ],basis, dual_basis ;

end:
#.......... end of upper_gl ...........




#...........strictly_upper_gl ............

strictly_upper_gl:=proc(n::posint,pretty_flag)

local alg_data, fr, basis, dual_basis, index_list, list_array, i, j;

alg_data:=0;

## We check that our basis names are not assigned ##


if assigned(e) then 
	ERROR(cat("\n LieAlg Error:",
		    "\n e is used by gl(n) as a basis vector.",
		    "\n Unassign it before calling matrix_algebras." )) 
fi;

if assigned(epsilon) then 
	ERROR(cat("\n LieAlg Error:", 
		    "\n epsilon is used by gl(n) as a basis dual-vector.",
		    "\n Unassign it before calling matrix_algebras. ")) 
fi;


## first we define a basis ##

index_list:=[  ];
list_array:=Matrix(n,n,0);

## The idea here is to generate a list of (array) indices ##
## and an array of (list) enty numbers ##
## i.e. if [ i, j ] is the kth element of index_list then k is the i, j th element of list_array ##
## and conversely  ##
## This allows us to go back and forth between the number of the algebra element ##
## and the formula for the brackets (which is based on the indices) ##

basis:=[seq(seq(cat(e,i,j),j=i+1..n),i=1..n)];
dual_basis:=[seq(seq(cat(epsilon,i,j),j=i+1..n),i=1..n)];
                                                                                                             
for i from 1 to n-1 do
	for j from i+1 to n do
		index_list:=[ op(index_list), [ i, j ] ];
		list_array[ i, j ]:=nops(index_list);

	od;
od;

unassign(op(basis), op(dual_basis));

## Print messages explaining bases ##

if _Vessiot_show_intermediate_steps_flag=true then
	print( e[ ij ],`Is The vector basis, where `, e[ ij ] , 
		`is the Matrix with 1 in the i th row, j th column, 0 elsewhere.` );	
	print( epsilon[ ij ] , `Is the dual basis.` );
fi;
	
## Now we define the brackets using the formula : [ eij, ehk ] = djh*eik-dki*ehj ##
## where the d is the Kronecker delta function ##

alg_data:=[  ];

for i from 1 to nops(index_list) do
	for j from i+1 to nops(index_list) do
		
		if index_list[ i ] [ 2 ]=index_list[ j ] [ 1 ] then
			alg_data:=[ op(alg_data), 
				    [ [ i,  j, list_array[ index_list[ i ] [ 1 ], index_list[ j ] [ 2 ] ] ], 1 ] ];
		fi;

		if index_list[ i ] [ 1 ]=index_list[ j ] [ 2 ] then

			alg_data:=[ op(alg_data), 
			[ [ i,  j, list_array[ index_list[ j ] [ 1 ], index_list[ i ] [ 2 ] ] ], -1 ] ];
		fi;

	od;
od;

fr:=cat(`strictly_upper_gl`, convert(n,string));

if nargs=2 then
	if pretty_flag=pretty then 
	fr:=cat(`strictly_upper_gl(`, convert(n,string),`)`); 
	fi;
fi;

[ [ Lie_alg,fr, [ nops(index_list) ] ], alg_data ],basis, dual_basis ;

end:
#............ end of strictly_upper_gl ...........




#............ gl_subalgebra .............

gl_subalgebra := proc (alg_fr_name, invariants::list) 

local fr, n, coords, vect_list, i, X, vars, j, Eq,
 J, eq, A, B,N,N_temp, Ans, alg_fr, alg_vect_list, w, v; 



#
# Watch for  frame relation between alg_fr_name/invariants
#


                


change_frame_to(objectFrame(invariants[1])); 


fr := _Vessiot_current_frame_name; 
n := frameBaseDimension(fr); 
coords :=frameIndependentVariables(): 

vect_list := [];  

for i to n do 
	vect_list := ([op(vect_list), DifferentialGeometryTools:-Vect(i)]); 
od; 

X := `&mult`(0,vect_list[1]); 
vars := [seq(seq(cat(_a,i,j),j = 1 .. n),i = 1 .. n)]; 

for i to n do for j to n do 
	X := `&plus`(X,`&mult`(cat(_a,i,j)*coords[i],vect_list[j]))
od od; 

Eq := []; 

for J in invariants while true do 
	eq := op(coeff_list(Lie_derivative(X,J),all)); 
	Eq := [op(Eq), eq]
od; 

A := LinearAlgebra[GenerateMatrix](Eq,vars); 


N_temp := Vessiot_nullspace(A[1]); 
B0:=Matrix([seq(m, m=N_temp)]);
B:=LinearAlgebra[Transpose](B0);
B:=Vessiot_rref(eval(B));
N:=[];

for i from 1 to nops(N_temp) do
	N:=[op(N),LinearAlgebra[Row](B,i)];
od;

Ans := []; 


change_frame_to(alg_fr_name); 
alg_fr := _Vessiot_current_frame_name; 

alg_vect_list:=frameBaseVectors(); 

for w in N while true do 
	v := v_zip(convert(eval(w),list),alg_vect_list,plus); 
	Ans := [op(Ans), v] 
od
end: 

#............ end of gl_subalgebra .............

#
#   MAIN PROGRAM
#

if nargs< 2 then 
	ERROR(cat("\n LieAlg Error:", 
		    "\n matrix_algebras requires 2 or 3 arguments." ))
fi;



if alg_type=full then
	ans:=full_gl(args[2]);
      RETURN(ans);
fi;

if alg_type=upper then
	ans:=upper_gl(args[2]);
      RETURN(ans);
fi;

if alg_type=strictly_upper then
	ans:=strictly_upper_gl(args[2]);
      RETURN(ans);
fi;

if alg_type=sub_algebra then
	if nargs <3 then 
		ERROR(cat("\n LieAlg Error:", 
		          "\n matrix_algebras(sub_algebras) requires 2 arguments:",
		          "\n First argument: the initialized Lie algebra name for",
                      " the full Matrix algebra glnR.",
			    "\Second argument: a list of vectors, forms or tensors."	 
		))
	
fi;

	if member(args[2],listOfFrameNames())=false then 
	ERROR(cat("\n LieAlg Error:",
                "\n Second argument for matrix_algebras(sub_algebra)must be",
                "\n initialized Lie algebra."))
	fi;
		 

	if type(args[3], list(Vessiot_bftv))= false then
	ERROR(cat("\n LieAlg Error:",
                 "\n Third argument for matrix_algebras(sub_algebra)must be",
                 "\n list of  {forms, tensors, vector}."))
	fi;

	invariants:=map(evalV,args[3]);

	if frameBaseDimension(objectFrame(invariants[1]))^2 <> frameBaseDimension(args[2]) then
		ERROR(cat("\n LieAlg Error:",
                "\n Dimension mismatch between dimension of Matrix algebra and", 
		    " dimension of underlying space for the invariants."))  
fi;





	ans:=gl_subalgebra(args[2], args[3]);
      RETURN(ans)
fi;

ERROR(cat("\n LieAlg Error:",
	    "\n First argument for matrix_algebras must be one of", 
          "\n {full,upper,strictly_upper,sub_algebra}"))
end:

#**************************************************

 