#
# New_algebras_from_old.alg
#
# LieAlg
#
# quotient_algebra; complexify_Lie_algebra;
# direct_sum_of_Lie_algebras;
# direct_sum_of_Lie_algebras;
#
#

#~~~~~~~~~~~~~~~~~~~~~ quotient_algebra ~~~~~~~~~~~~~~~~~~~

quotient_algebra := proc(ideal0, complement0)

# CLASS II PROGRAM
# 01/13/2004 IA

option `Copyright Ian M. Anderson 2004`; 

description `create quotient Lie algebra data structure`;

local basis,ideal, complement,i, j, n, struc_const, C, alg_name,k ,ans,A, B,P,m;
    

if nargs< 2 then 
	ERROR(cat("\n LieAlg Error:",
	          "\n quotient_algebra requires 2 to 4  arguements:",
                "\n list of vectors(ideal), complementary list of vectors, Lie algebra name"))
fi;

if type(ideal0,list)=false then
      ERROR(cat("\n LieAlg Error:",
 		    "\n First argument for quotient algebra must be a list of vectors."))
fi;
 	
ideal:=map(evalV,ideal0);

if not type(ideal, list(Vessiot_vect))  then
	ERROR(cat("\n LieAlg Error:",
 		    "\n First argument for quotient_algebra must be a list of vectors."))
fi;
 
if nops(DiffGeom_basis(ideal)) <> nops(ideal) then  
	ERROR(cat("\n LieAlg Error:",
                "\n First list of vectors(ideal) for quotient_algebra not linearly independent."));
fi;

if not LieAlgCheck(ideal,ideal1)  then 
	ERROR(cat("\n LieAlg Error:",
 		    "\n First list of vectors(ideal) for quotient_algera do not form an ideal." ));
fi:


if type(complement0,list)=false then
      ERROR(cat("\n LieAlg Error:",
 		    "\n Second argument for quotient algebra must be a list of vectors."))
fi;
 	
complement:=map(evalV,complement0);

if not type(complement, list(Vessiot_vect)) then
	ERROR(cat("\n LieAlg Error:",
 		    "\n Second argument for quotient_algebra must be a list of vectors."))
fi;

m:=nops(ideal);
n := nops(complement);
    
basis := [op(complement),op(ideal)];
    
if nops(DiffGeom_basis(basis)) <> frameBaseDimension() or
   m +n <> frameBaseDimension() then  
	ERROR(cat("\n LieAlg Error:",  
		    "\n Vectors in second list do not complete vectors in ",
		    "\n first list to a Lie algebra basis."));
fi;

if nargs>=3 then 
	alg_name:=args[3];
	if type(alg_name,name)=false then 
		ERROR(cat("\n LieAlg Error:", 
	          	    "\n The  third argument for quotient_algebra", 
		    		"\n must be an unassigned name." ))
	fi;
else 
	alg_name:=`Koszul`
fi;
	
#
# Main Procedure
#

struc_const := [];
    
for i  to n do 
	for j from i + 1 to n do
      	C := linear_combo(Lie_bracket(basis[i], basis[j]),
                basis);
            for k to n do
                    if C[k] <> 0 then struc_const :=
                        [op(struc_const), [[i, j, k], C[k]]]
                    fi
            od   
	od
od;
    
ans:= [[Lie_alg,alg_name,[n]],struc_const];

if nargs=4 then  
   A:=LinearAlgebra[Transpose](Matrix(linear_combo(basis,frameBaseVectors())));
   B:= LinearAlgebra[IdentityMatrix](n,n+m); 
   P:= LinearAlgebra[MatrixMatrixMultiply](B, LinearAlgebra[MatrixInverse](A) );
   ans:= ans,P
fi;

ans;

end:



#~~~~~~~~~~~~~~~~~~~~ complexify_Lie_algebra ~~~~~~~~~~~~~~~~~~~~

complexify_Lie_algebra:=proc(N)

# CLASS II PROGRAM
# 01/13/2004 IA

option `Copyright Ian M. Anderson 2004`; 

description `create complexified Lie algebra data structure`;

local n, Lie_alg_data,Lie_alg_name,structure_constant_list,a1,a2,a3,a4,v;

if nargs <1 then 
	ERROR(cat("\n LieAlg Error:",
		    "\n complexify_Lie_algebra requires 1 or 2 arguments:",
		    "\n {Lie algebra data structure, initialized Lie algebra name}", 
	          " Lie algebra name."))  
fi;


if type(N,Vessiot_Lie_alg) then
	Lie_alg_data:=N;
else
   	if member(N,listOfFrameNames()) then 
		Lie_alg_data:=convert(N,Lie_algebra_data)
	else
		ERROR(cat("\n LieAlg Error:",
		    	    "\n First argument for complexify_Lie_algebra must be:",
		    	    "\n {Lie algebra data structure, initialized Lie algebra name}.")) 
	fi;
fi;

	
n:=Lie_alg_data[1][3][1]:

if nargs=2 then 

	if type(args[2],name)=false then 
		ERROR(cat("\n LieAlg Error:", 
	          	    "\n The  second argument for complexify_Lie_algebra", 
		    	    "\n must be an unassigned name." ))
	fi;

	Lie_alg_name:=args[2] 
else 
	Lie_alg_name:=cat(Lie_alg_data[1][2],`C`);
fi;



structure_constant_list:=[];

for v in Lie_alg_data[2] do
	a1:=v;
	a2:=[[v[1][1], v[1][2]+n, v[1][3]+n],v[2]];a3:=[[v[1][1]+n, v[1][2], v[1][3]+n],v[2]];
	a4:=[[v[1][1]+n, v[1][2]+n, v[1][3]],-v[2]];
	structure_constant_list:=[op(structure_constant_list),a1,a2,a3,a4];
od;

[[Lie_alg,Lie_alg_name,[2*n]], structure_constant_list];

end:



#~~~~~~~~~~~~~~~~~~~~ direct_sum_of_algebras ~~~~~~~~~~~~~~~~~~

direct_sum_of_algebras:=proc(list_of_alg) 


# CLASS I PROGRAM
# 01/13/2004 IA

option `Copyright Ian M. Anderson 2004`; 

description `create a direct sum Lie algebra data structure`;


local n,Lie_alg_name, N,A,i, old_structure_constants, new_structure_constants,c, frame;

if nargs =0 then 
	ERROR(cat("\n LieAlg Error:",
		     "\n direct_sum_of_algebras requires 1 or 2 arguments:",
		     "\n {list of Lie algebra names, list of Lie algebra data structures},", 
		     " Lie algebra name."))
fi;

if type(list_of_alg,list(name)) then 
	old_structure_constants:=map(structure_constants_list,list_of_alg)
else 
	if type(list_of_alg, list(Vessiot_Lie_alg)) then
		old_structure_constants:=
		[seq(list_of_alg[i][2], i =1..nops(list_of_alg))]
      else
      	ERROR(cat("\n LieAlg Error:",
		     "\n First argument for direct_sum_of_algebras must be:",
		     "\n {list of Lie algebra names, list of Lie algebra data structures},", 
		     " Lie algebra name."))
	fi;
fi;


if nargs=2 then 
	if type(args[2],name)=false then 
		ERROR(cat("\n LieAlg Error:", 
	          	    "\n The  second argument for direct_sum_of_algebras", 
		    	    "\n must be an unassigned name." ))
	fi;
	Lie_alg_name:=args[2] 
else 
	Lie_alg_name:=`Koszul`;
fi;

n:=0;
N:=0; 

new_structure_constants:=[];

for  i from 1 to  nops(old_structure_constants)  do 
	A:=old_structure_constants[i];
	for c in A do
		new_structure_constants:=
		[op(new_structure_constants),[[c[1][1]+N,c[1][2]+N,c[1][3]+N],c[2]] ];
	od; 

	if type(list_of_alg,list(name)) then 
		n:=frameBaseDimension(list_of_alg[i]) 
	else 
		n:= list_of_alg[i][1][3][1]; 
	fi;

	N:=N+n;
od;

[[Lie_alg,Lie_alg_name,[N]], new_structure_constants];

end:



#~~~~~~~~~~~~~~~~~~ semi_direct_sum of algebras ~~~~~~~~~~~~~~~~~~~~


LieAlg[semi_direct_sum_of_algebras]:=proc(alg1, alg2,L)

# CLASS I PROGRAM
# 01/13/2004 IA

option `Copyright Ian M. Anderson 2004`; 

description `create a semi-direct product  Lie algebra data structure`;

local n1,n2,i,j,struct_const,structure_constants1,Lie_alg_name, 
	structure_constants2,name1,name2,alg_name,term, A,B,M, k,l;

if nargs <3 then  
	ERROR(cat("\n LieAlg Error:",
		     "\n semi_direct_product_of_algebras requires 3 or 4 arguments:",
		     "\n initialized Lie algebra name, initialized Lie algebra name, ",
		     " Vessiot_transform, Lie algebra name; or ", 
		     "\n Lie algebra data structure, Lie algebra data structure,", 
		     " array, Lie algebra name." )) 
fi;

if type(alg1,{name, Vessiot_Lie_alg})=false then
	ERROR(cat("\n LieAlg Error:",
		     "\n First argument for semi_direct_product_of_algebras must be:",
		     "\n {list of Lie algebra names, list of Lie algebra data structures}."))
fi;
 

if type(alg2,{name, Vessiot_Lie_alg})=false then
	ERROR(cat("\n LieAlg Error:",
		     "\n Second argument for semi_direct_product_of_algebras must be:",
		     "\n {list of Lie algebra names, list of Lie algebra data structures}.")) 
fi;

if type(L,{Vessiot_transformation,Matrix})=false then
	ERROR(cat("\n LieAlg Error:",
		     "\n Third argument for semi_direct_product_of_algebras must be:",
		     "\n {Vessiot_transformation, MAtrix}."))
fi;

if nargs=4 then 
	if type(args[4],name)=false then 
		ERROR(cat("\n LieAlg Error:", 
	          	    "\n The  second argument for direct_sum_of_algebras", 
		    	    "\n must be an unassigned name." ))
	fi;
	Lie_alg_name:=args[4] 
else 
	Lie_alg_name:=`Koszul`;
fi;
 
if type(alg1,name) then 
	if member(alg1, listOfFrameNames())=false  then
		ERROR(cat("\n LieAlg Error:",
			    "\n First argument for semi_direct_product_of_Lie_algebras is", 
			    "  not an initialized Lie algebra."))
      fi;
                   		     		
	if member(alg2, listOfFrameNames())=false  then
		ERROR(cat("\n LieAlg Error:",
		     "\n  Since the first argument for a semi_direct_product_of_algebras is an ",
		     "\n  initialized Lie algebra, the second arguement must be an", 
       		     " initialized Lie algebra.")) 
            fi;
	
	if type(L, Vessiot_transformation)=false then
		ERROR(cat("\n LieAlg Error:",
		     "\n Since the first argument for  semi_direct_product_of_algebras is an ",
		     "\n  initialized Lie algebra, the third argument must be an", 
		     " Lie algebra transformation from the first algebra to the second.")) 
	fi;       
	
	if transformDomain(L)<>alg1 or transformRange(L) <> alg2 then 
            ERROR(cat("\n LieAlg Error:",
			    "\n Since the first argument for  semi_direct_product_of_algebras is an ",
		     	    "\n  initialized Lie algebra, the third arguement must be an", 
		          " Lie algebra transformation fromthe first algebra to the second."))
	fi; 		

 	structure_constants1:=structure_constants_list(alg1);                  
	structure_constants2:=structure_constants_list(alg2);            	
	n1:=frameBaseDimension(alg1);
      n2:=frameBaseDimension(alg2); 
      name1:=alg1;
      name2:=alg2;
      M:=Matrix(n2,n1, (i,j)-> coeff_list(Lie_algebra_transform(
	L,vect(z.j,name1)),[[i]]))
fi;
 

if type(alg1, Vessiot_Lie_alg) then

	if type(alg2, Vessiot_Lie_alg)=false  then
		ERROR(cat("\n LieAlg Error:",
		     	     "\n  Since the first argument for a semi_direct_product_of_algebras is a ",
		     	     "\n  Lie algebra data structure, the second arguement must be an", 
		     	     " Lie algebra data structure.")) 
	fi;

	if type(L, Matrix)=false then
		ERROR(cat("\n LieAlg Error:",
		     "\n Since the first argument for  semi_direct_product_of_algebras is an ",
		     "\n Lie algebra data structure, the third argument must be an array."))
      fi;
 
      n1:=alg1[1][3][1];
      n2:=alg2[1][3][1];
	
	if LinearAlgebra[ColumnDimension](L) <>n1 or LinearAlgebra[RowDimension](L) <> n2 then 
            ERROR(cat("\n LieAlg Error:",
			    "\n Since the first argument for  semi_direct_product_of_algebras is an ",
		     	    "\n initialized Lie algebra, the third argument must be an array", 
		          "\n with ", n2, " rows and ", n1, " columns.")) 		
      fi;


 	structure_constants1:=alg1[2];
 	structure_constants2:=alg2[2];
      
      name1:=alg1[1][2];
      name2:=alg2[1][2];
      M:=L;
fi;
 
  
 struct_const:=structure_constants1;
  
 for  term in structure_constants2 do
  	struct_const:=[op(struct_const), [ [term[1][1]+n1,term[1][2]+n1, 	term[1][3] +n1], term[2]] ]
  	od;
  
 for i from 1 to n1 do 
 	for j from 1 to n2 do 
      	for k from 1 to n2 do 
      	B:=add( M[l,i]*structure_constant(l,j,k,alg2), l=1..n2);
 		if B <>0 then 
  			struct_const:=[op(struct_const), [[i,j+n1, k+n1],B]];
 		fi;
 		od;
  	od;
  od;
 
 [[Lie_alg,Lie_alg_name,[n1+n2]],struct_const];
 
 end: 





#**************************************************