#
# algebras_and_ideals.alg
#
# LieAlg
#
# center; rad; derived_algebra; normalizer; stablizer; 
# centralizer;
#
#


#~~~~~~~~~~~~~~~~~ center ~~~~~~~~~~~~~~~~

center := proc()

# CLASS II PROGRAM
# 01/09/2004:IA

# The  center is found by solving  the linear system C_{ij}^k a^i=0. 

option `Copyright Ian M. Anderson 2004`; 

description `Compute the center of a Lie algebra`;

local A,C,n, i,j,k,get_center,cent_bas,ans;


optionFrameCheck(0);

n:=frameBaseDimension();


for k from 1 to n do 
	C[k]:=Matrix(n,n,0);
	for i from 1 to n do 
		for j from i+1 to n do
			C[k][i,j]:=structure_constant(i,j,k);
			C[k][j,i]:=-C[k][i,j]
		od;
	od;

	if k = 1 then  
		A:= C[1] else
		A:=Matrix([[A],[C[k]] ]);
	fi;

od;

cent_bas:= Vessiot_nullspace(A);

ans:=[seq(v_zip( convert(cent_bas[i],list), frameBaseVectors(),plus),i=1..nops(cent_bas))]

end:


#~~~~~~~~~~~~~~~~~~~ rad ~~~~~~~~~~~~~~~~~~~

rad:= proc()

#CLASS II PROGRAM
#01/09/2004:IA

local r,g0,d1,vars,A,i,a,Z,eq_list,v,soln,fr,n,ans;

option `Copyright Ian M. Anderson 2004`; 

description `Compute the radical of a Lie algebra.`;


fr:=_Vessiot_current_frame_name;
n:=frameBaseDimension();

g0 := frameBaseVectors();
d1:=LieAlgebrasTools:-bracket_of_subspaces(g0,g0);

vars:=[seq(a[i],i=1..n)];
Z:=v_zip(vars,g0,plus);

eq_list:={};

for v in d1 do
 	eq_list:= eq_list union  {Killing(Z,v)}
od;

A:=LinearAlgebra[GenerateMatrix](eq_list,vars);
soln:=Vessiot_nullspace(A[1]);

[seq(v_zip( convert(soln[i],list),g0,plus),i=1..nops(soln))]
end:



#~~~~~~~~~~~~~~~~~derived_algebra~~~~~~~~~~~~~~~~~~ 

derived_algebra:=proc()

#CLASS II PROGRAM
#01/09/2004:IA

option `Copyright Ian M. Anderson 2004`; 

description `Compute the derived algegra of a Lie algebra.`;

local fr, n, ans,g0;

fr:=_Vessiot_current_frame_name;
n:= frameBaseDimension();

g0:=frameBaseVectors();


ans:=LieAlgebrasTools:-bracket_of_subspaces(g0,g0);

end:

#~~~~~~~~~~~~~~~~~  normalizer  ~~~~~~~~~~~~~~~~

normalizer:=proc(sub_alg0)

#CLASS II PROGRAM
#01/09/2004:IA

option `Copyright Ian M. Anderson 2004`; 

description "Compute the normalizer of a subalgebra h in a Lie algebra or subalgebra";

local i,j, n, k, _a, Z, A,B,eq_list,sols,C,v,vars,form,g0, fr,sub_alg;

if nargs=0 then 
	ERROR(cat("\n LieAlg Error:",  
                "\n Normalizer requires a list of vectors for the first argument."));
fi:

if  not type(sub_alg0,list)  then 
	ERROR(cat("\n LieAlg Error:",  
                "\n normalizer requires a list of vectors for the first argument."));
fi:

sub_alg:=map(evalV,sub_alg0);


if  not type(sub_alg,list(Vessiot_vect))  then 
	ERROR(cat("\n LieAlg Error:",  
                "\n normalizer requires a list of vectors for the first argument."));
fi:

if  not LieAlgCheck(sub_alg,subalgebra1)  then 
	ERROR(cat("\n LieAlg Error:",  
                "\n First list of vectors do not form a subalgebra."));
fi:

if nargs=2 then 
	
	if  not type(args[2],list)  then 
	ERROR(cat("\n LieAlg Error:",  
                "\n normalizer requires a list of vectors for the second optional argument."));
	fi:

	g0:=map(evalV,args[2]);
	
	if  not type(g0,list(Vessiot_vect))  then 
		ERROR(cat("\n LieAlg Error:",  
                	    "\n normalizer requires a list of vectors for the",
                      " second optional argument." ));
	fi:

	if  not LieAlgCheck(g0,subalgebra1)  then 
		ERROR(cat("\n LieAlg Error:",  
                "\n Second  optional list of vectors for normalizer do not form a subalgebra."));
	fi:

	if linear_combo(sub_alg,g0)=[] then 
		ERROR(cat("\n LieAlg Error:",  
                	    "\n Span of second optional list of vectors for normalizer must", 
			    " contain first list." ));
	fi:


else
	g0:=frameBaseVectors();
fi;

fr:=_Vessiot_current_frame_name;
n:= nops(g0);

k:= nops(sub_alg);


vars:=[seq(_a[i],i=1..n)];
Z:=v_zip(vars,g0,plus);
 
A:=annihilator(sub_alg);
 
eq_list:={};
 
for i from 1 to k do 
	for form in A do
	eq_list:=eq_list union 	{coeff_list(hook(Lie_bracket(Z,sub_alg[i]),form),[[]])};
 	od; 
od;

B:=LinearAlgebra[GenerateMatrix](eq_list,vars);
sols:= Vessiot_nullspace(B[1]):
 
C:=[];
for v in sols do 
 	C:= [op(C), v_zip(convert(v,list),g0,plus)];
od;

C;
end:

#~~~~~~~~~~~~~~~~~~~~~~ stabilizer ~~~~~~~~~~~~~~~~~~~~~~~~~


stabilizer:= proc(A, subalg0)


#CLASS II PROGRAM
#01/09/2004:IA


local vars, alpha, v, fr, psi, Omega, P, eq,subalg,soln,s,B, ans;
description "";   

fr:= _Vessiot_current_frame_name;


if nargs <>2 then
	ERROR(cat("\n LieAlg ERROR:", 
                "\n Arguments for stablizer are ",
		    "{Matrix,Lie algebra transformation},list of vectors.")) 
fi;

 
if  type(A,Matrix) then
	psi:= matrix_to_Lie_algebra_transform(fr, fr, A);
else   if type(A,Vessiot_transformation) then 
      psi:=A;
     else
	ERROR(cat("\n LieAlg ERROR:", 
                "\n stablizer  requires a Matrix or  Lie algebra transformation as first argument.")) 
fi;
fi;


if  not type(subalg0,list)  then 
	ERROR(cat("\n LieAlg Error:",  
                "\n Stablizer requires a list of vectors for the second argument."));
fi:

subalg:=map(evalV,subalg0);

if  not type(subalg,list(Vessiot_vect))  then 
	ERROR(cat("\n LieAlg Error:",  
                "\n Stablizer requires a list of vectors for the second argument."));
fi:

vars:= indets(convert(A, set));
Omega:= annihilator(subalg);

P := map2(pt_pushforward, psi, subalg);

eq:= {seq(seq(coeff_list(hook(v, alpha), [[]]), v = P),
        alpha = Omega)};

soln := [solve(eq, vars)];

ans:=[];

for s in soln do
 	B:=eval(map2(subs, s, A));
      if type(A,Matrix) then 
		if LinearAlgebra[Determinant](B)<>0  then 
			ans:=[op(ans),eval(B)] 
		fi;
	fi;
      
	if type(B,Vessiot_transformation) then 
		if transformation_subalgebra(B,kernel)=[] then
			ans:=[op(ans),B] 
		fi;
	fi;
od;

ans
   
end:

#-------------------------------------------------

centralizer := proc(sub_set0)

#CLASS II PROGRAM
#01/09/2004:IA

local i,j, n, _a, k, Z,sub_set, A,eq_list,sols,C,v,vars,g0, fr;

description "Compute the centralizer of a set of vectors.";

optionalFrameCheck(1);

if nargs=0  then 
	ERROR(cat("\n LieAlg Error:",  
                "\n centralizer requires a list of vectors for the first argument."));
fi:


if  not type(sub_set0,list)  then 
	ERROR(cat("\n LieAlg Error:",  
                "\n centralizer requires a list of vectors for the first argument."));
fi:


sub_set:=map(evalV,sub_set0);

if  not type(sub_set,list(Vessiot_vect))  then 
	ERROR(cat("\n LieAlg Error:",  
                "\n centralizer requires a list of vectors for the first argument."));
fi:


if nargs=2 then 
	
	if  not type(args[2],list)  then 
	ERROR(cat("\n LieAlg Error:",  
                "\n centralizer requires a list of vectors for the second optional argument."));
	fi:

	g0:=map(evalV,args[2]);
	
	if  not type(g0,list(Vessiot_vect))  then 
		ERROR(cat("\n LieAlg Error:",  
                	    "\n centralizer requires a list of vectors for the",
                      " second optional argument." ));
	fi:

	if  not LieAlgCheck(g0,subalgebra1)  then 
		ERROR(cat("\n LieAlg Error:",  
                "\n Second  optional list of vectors for centralizer do not form a subalgebra."));
	fi:

	if linear_combo(sub_set,g0)=[] then 
		ERROR(cat("\n LieAlg Error:",  
                	    "\n Span of second optional list of vectors for centralizer", 
			    " must contain first list." ));
	fi:


else
	g0:=frameBaseVectors();
fi;



fr:=_Vessiot_current_frame_name;
n:= nops(g0);

k:= nops(sub_set);

vars:=[seq(_a[i],i=1..n)];
Z:=v_zip(vars,g0,plus);

eq_list:={};

for i from 1 to k do 
	eq_list:=eq_list union coeff_set(Lie_bracket(Z,sub_set[i]));
od;

A:=LinearAlgebra[GenerateMatrix](eq_list,vars);
sols:= Vessiot_nullspace(A[1]):

C:=[];

for v in sols do 
	C:= [op(C), v_zip(convert(v,list),g0,plus)];
od;

C;
end:

#********************************************************************