#
# Cohomology.alg
#
# LieAlg
#
# relative_chains, cohomology
#



#~~~~~~~~~~~~~~~~~~ relative_chains ~~~~~~~~~~~~~~~~~~~

RelativeChains := proc (sub_alg0)

# CLASS II PROGRAM
# 01/10/2004 IA

option `Copyright Ian M. Anderson 2004`; 

description `calculates relative chains (forms) with respect to a subalgebra`;
 
local p, degree, ans,n, q,complement,form_basis,sub_alg,ann_form,relative_chains_in_degree; 

n:=frameBaseDimension();

if  nargs=0 then 
	ERROR(cat("\n LieAlg Error:",
                "\n First argument for relative_chains is a list of vectors."))
fi;

if  type(sub_alg0, list)=false then 
	ERROR(cat("\n LieAlg Error:",
                "\n First argument for relative_chains is a list of vectors."))
fi;

sub_alg:=map(evalV,sub_alg0);

if type(sub_alg, list(Vessiot_vect))=false then 
	ERROR(cat("\n LieAlg Error:",
                "\n First argument for relative_chains is a list of vectors."))
fi;

if nops(sub_alg) >0  then
	if  nops(DiffGeom_basis(sub_alg)) <> nops(sub_alg) then 
		ERROR(cat("\n LieAlg Error:",
                "\n List of  vectors  for relative_chains are not independent."))
	fi;
fi;


p := nops(sub_alg); 

if nargs=1 then 
	degree:='all';
	complement:=complementary_basis(sub_alg,[seq(DifferentialGeometryTools:-Vect(i),i=1..n)]);
fi;
	
if nargs=2 then
      complement:=complementary_basis(sub_alg,[seq(DifferentialGeometryTools:-Vect(i),i=1..n)]);
	degree:=args[2]
fi;

if nargs=3 then 
	complement:=args[3]; 
	degree:=args[2]
fi;


form_basis:=dual_basis([op(sub_alg), op(complement)]);

#
# LOCAL SUBROUTINE
#

relative_chains_in_degree:= proc(sub_alg, q,form_basis)

local eqs, v, rel_chain_coeff, ans, a, c, n, m,fr,k, 
	i, j, C, coframe, omega, tau,vars,b,alpha,w1,w2,inv_forms,ann_forms;
     
if q = 0 then RETURN([scalar_form()]) fi;
 
n := frameBaseDimension();
m:= n -nops(sub_alg);
fr:=_Vessiot_current_frame_name;
 
#with(DG_Utilities,generate_forms_in_degree,annihilator); 


ann_form:=annihilator(sub_alg,form_basis);

inv_forms:=generate_forms_in_degree(q,ann_form);
  

for v in sub_alg do
    	vars:=[seq(a[j], j=1..nops(inv_forms))];        
	omega:=v_zip(vars,inv_forms,plus); 
           
	eqs := coeff_set(Lie_derivative(v, omega));
      if eqs<>{0} then    
		C:= LinearAlgebra[GenerateMatrix](eqs,vars );
     		rel_chain_coeff := Vessiot_nullspace(C[1]);
 		if  nops(rel_chain_coeff)=0 then 
				inv_forms:=[zero_form(q)]
		else        
			inv_forms:=[];
         		for c in rel_chain_coeff do
         			tau:=helmsimp(subs({seq(a[j]=c[j],j=1..nops(vars))}, omega));
 				inv_forms := [op(inv_forms), tau]
         		od;
     
		fi;
	fi;

od;
 
inv_forms
 
end:

#

if degree='all' then 
	ans:=[seq(relative_chains_in_degree(sub_alg,q,form_basis),q = 0 .. n-p)]
else
	ans:= relative_chains_in_degree(sub_alg,degree,form_basis)       
fi;

ans;
end:


#~~~~~~~~~~~~~~~~~ Lie_algebra_cohomology  ~~~~~~~~~~~~~~~~~~~~~


Cohomology:=proc(chains) 


# CLASS II PROGRAM
# 01/10/2004 IA

option `Copyright Ian M. Anderson 2004`; 

description `Compute cohomology from list of subspaces`; 

local d_mat_1, d_mat_2,q, n,top_degree,degree, 
	relative_ext_d_matrix,Lie_algebra_cohomology_in_degree,  coh;

if nargs=0 then 
 	ERROR(cat("\n LieAlg Error:",
		    "\n Argument for Lie_algebra_cohomology must a list of list of chains in",
		    "\n increasing order of degree, starting with  degree 0."))	
fi;

if  chains=[[]] or type(chains, list(list(Vessiot_form))) =false then 
	ERROR(cat("\n LieAlg Error:",
		    "\n Argument for Lie_algebra_cohomology must a list of list of chains in", 
                "\n increasing order of degree, starting with  degree 0."))	
fi;

#
# Coulf be more messages -- but   lets rewrite  routines first !!!
#

#
# LOCAL ROUTINES
#


#........... relative_ext_d_matrix ...........

relative_ext_d_matrix := proc (rel_chain1, rel_chain2) 

local m, A,i; 
m := nops(rel_chain1);  
      
if map(coeff_list,rel_chain2,all)=[[0]]  then 
	A:=Matrix(m,1,0) 
	
else
	A:=Matrix([seq(linear_combo(ext_d(rel_chain1[i]),rel_chain2)	,i = 1 .. m)]); 
fi;

LinearAlgebra[Transpose](A)
end:

#........... Lie_algebra_cohomology_in_degree ..........

Lie_algebra_cohomology_in_degree :=proc(d_mat_1,d_mat_2,q,q_chains)
local A, B, C,Co,c,r,s,i,j,b,pans, ans;
if q = 1 then 
 	A:=convert(Vessiot_nullspace(d_mat_2),list);
 	if nops(A)=0 then  
		ans:=[zero_form(1)] 
	else 
 		ans:=[];
		for i from 1 to nops(A) do  
 			ans :=[op(ans), v_zip(convert(A[i],list),q_chains,plus)];
		od;
 	fi;
fi;

if q > 1 then 
 	A:=convert(Vessiot_nullspace(d_mat_2),list);
 	r:=nops(A);
 	if r=0  then 
		ans:= [zero_form(q)] 
 	else 
 		B:=LinearAlgebra[Transpose](d_mat_1);
 		C:=Matrix([seq(LinearAlgebra[MatrixVectorMultiply](B,A[i]),i=1..r)]);
 		Co:=convert(Vessiot_nullspace(C),list);
 		s:=nops(Co);
 		if s=0 then 
			ans:=[zero_form(q)] 
		else
 			pans:=[seq(eval(sum('Co[i][j]*A[j]','j'=1..r)),i=1..s)];
 			ans:=[];
 			for i from 1 to s do 
				c:=convert(pans[i],list); 
 				ans :=[op(ans),v_zip(c,q_chains,plus)]
			od;
		fi;
	fi;
fi;

RETURN(ans);

end:
 

#
#  MAIN PROGRAM 
#

n:=nops(chains)-1;
top_degree:= formDegree(chains[n+1][1]);
coh:=[];
d_mat_1:=relative_ext_d_matrix(chains[1],chains[2]);
 	
for q from 1 to n do 
	
	if q<n then 
		d_mat_2:=relative_ext_d_matrix(chains[q+1],chains[q+2]) 
	else 
		d_mat_2:=eval(Matrix(1,1,[0])) 
	fi;
 	
	coh:=[op(coh),Lie_algebra_cohomology_in_degree
           (d_mat_1,d_mat_2,q,chains[q+1])]; 
 	
	d_mat_1:=eval(d_mat_2);
od;

coh
end;

#********************************************************