#  cor_der.vss
#
#  directional_covariant_derivative, covariantDerivativeProcedure, covariant_derivative
#
#


directional_covariant_derivative := proc(X::Vessiot_vect, 
T::Vessiot_tensor, nabla::Vessiot_connection)

        
local ans, Y, N,covariantDerivativeProcedure;

VessiotFrameCheck(X);


if objectFrame(X) <> objectFrame(T)  then
	ERROR(cat("\n VESSIOT ERROR:",  
                "\n vector frame and tensor frame are different\n" ,
                objectFrame(X), objectFrame(T) ))  
fi;

if objectFrame(X) <> objectFrame(nabla)  then
	ERROR(cat("\n VESSIOT ERROR:",  
                "\n vector frame and connection frame are different \n", 
                  objectFrame(X), objectFrame(nabla) )) 
fi;

if objectFrame(T) <> objectFrame(nabla)  then
	ERROR(cat("\n VESSIOT ERROR:", 
                "\n tensor frame and connection frame are different", 
                objectFrame(T),  objectFrame(nabla) ))  
fi;

VessiotFrameCheck(X);

N:=convert(nabla,Vessiot_tensor);

covariantDerivativeProcedure:=proc(term,flag,X,nabla)

local ans,Y,Z, alpha,t,comp,C, termX,termN;


if flag = `coeff` 
	then 
      comp:=[ [ [], Lie_derivative(X,term)] ];
      ans:=_VESSIOT([[tensor,_Vessiot_current_frame_name,[[],[] ]],  comp ]);   
	RETURN(ans) 
fi;


if flag= 'con_bas'  
	then
      comp:=[];
      for termX in op(X)[2] do 
		for termN in op(nabla)[2] do 
            if termX[1][1]=termN[1][3] and termN[1][1]=term 
			then
            	C:=[[ termN[1][2] ], termN[2]*termX[2] ];
            	comp:= [op(comp),C];
            fi; 
     		od;
	od;

ans:=_VESSIOT([[tensor,objectFrame(nabla),[[con_bas],[] ]], comp]); 

RETURN(ans);
fi;

if flag= 'cov_bas'  
	then
	comp:=[];
      for termX in op(X)[2] do 
		for termN in op(nabla)[2] do 
            if termX[1][1]=termN[1][3] and termN[1][2]=term 
			then
            	C:=[[ termN[1][1] ], -termN[2]*termX[2] ];
            	comp:= [op(comp),C];
            fi; 
     		od;
	od;

ans:=_VESSIOT([[tensor,objectFrame(nabla),[[cov_bas],[] ]], comp]); 

RETURN(ans)

fi;

ERROR(`VESSIOT ERROR: covariant_derivative  only works at present on tensors of type cov_bas or con_bas`);

end:




derivation[tensor](covariantDerivativeProcedure,T,zero_tens(tensorType(T)), X, N);

end:

#-----------------------------------------------------------



#
#----------------------------------------------------------------------------
#


covariant_derivative:=proc(T::Vessiot_tensor, nabla::Vessiot_connection)
        
local ans,n,var,X,alpha ,directions; global tau_tensor, tau_mono;

tau_tensor:=0;
tau_mono:=0;

if objectFrame(T) <> objectFrame(nabla)  
	then
	ERROR(`VESSIOT ERROR: tensor frame`, objectFrame(T), `and 	connection 	frame`, objectFrame(nabla), `are different.`)  
fi;

if objectFrame(T) <> _Vessiot_current_frame_name 
	then
	change_frame_to(objectFrame(T))
fi;

n:= frameBaseDimension();

ans:=DifferentialGeometryTools:-zero_tens(tensorType(T)) &tensor DifferentialGeometryTools:-Form(1);

for   var  from 1 to n do
	X:=DifferentialGeometryTools:-Vect(var);
	alpha:=DifferentialGeometryTools:-Form(var);
      ans:= ans &plus   (directional_covariant_derivative(X,T,nabla) &tensor alpha)
od;

ans:=helmsimp(ans);


ans;


end:
