#
#  Curvature Tensor 
#
#  tensors  
#
#  curvature_tensor,Ricci_tensor, Ricci_scalar, Weyl_tensor, Cotton_tensor
#
#


curvature_tensor:= proc(a_connection::Vessiot_connection)

local n,f,Gamma,Omega,A,B,R,i,j,C;

VessiotFrameCheck(a_connection);

n:=frameBaseDimension();

C:=convert(a_connection,Vessiot_tensor);


#

f:=proc(i,j) 
	local A,B,M; 
	A:=convert(frameBaseVectors()[j], Vessiot_tensor);
	B:=convert(frameBaseForms()[i],   Vessiot_tensor);

	M:=contract_indices( A&tensor B,  C, [[1,1],[2,2]]);

	convert(M,Vessiot_form);

end:

# Change connection to Matrix-valued one form and compute Matrix-valued curvature two form.

Gamma:=Matrix(n,n,f):

Omega:=(map(ext_d, Gamma)) &mplus   (Gamma &mwedge Gamma);

Omega:=map(convert, Omega,Vessiot_tensor);


# Change curvature two form to tensor

A:=convert(frameBaseVectors()[1],Vessiot_tensor);
B:=convert(frameBaseForms()[1],Vessiot_tensor);



R:= 0&mult (B &tensor A &tensor B &tensor B);

for i from 1 to n do
for j from 1 to n do


A:=convert(frameBaseVectors()[i],Vessiot_tensor);
B:=convert(frameBaseForms()[j],Vessiot_tensor);

R:=R &plus  (B&tensor A &tensor Omega[i,j])
od;
od;

(-1) &mult R;

end:





#-----------------------------------------------------


Ricci_tensor:=proc(curv_tensor::Vessiot_tensor)
#CLASS II PROGRAM
#01/15/03:IA 
         
local indxType,  indxType1, indxType2;

indxType:=tensorIndexType(curv_tensor);

indxType1:=map(tensorIndexPart1, indxType);
indxType2:=map(tensorIndexPart2, indxType);

if indxType1 <>[cov,con,cov,cov] then 
	ERROR(cat("\n VESSIOT ERROR:",
                "\n a tensor of type [cov,con,cov,cov]  is required",
                 indxType1))
fi;

if  nops(convert(indxType2,set))>1 then
	ERROR(cat("\n VESSIOT ERROR:",
                "\n a tensor indices of more than one spatial type",
                indxType2))
fi;

contract_indices(curv_tensor, [[2,4]]);

end:

#-----------------------------------------------------------


Ricci_scalar:=proc(a_metric::Vessiot_inv_metric,curv_tensor::Vessiot_tensor)
#CLASS II PROGRAM
#01/15/03:IA       
#    a_metric: a  contravariant metric. 
#    curv_tensor1: a type [-1,1,-1,-1] curvature tensor.

local R, S;

R:=Ricci_tensor(curv_tensor);
S:=raise_lower_indices(a_metric,R,[1]);
contract_indices(S,[[1,2]]);

end:

#---------------------------------------------------

Einstein_tensor:=proc(a_metric::Vessiot_tensor,curv_tensor::Vessiot_tensor)
      
local R,S,  Sg, Rcon, ans;

R:=Ricci_tensor(curv_tensor);
S:=Ricci_scalar(a_metric,curv_tensor);       
Sg:=DiffGeomMain:-Tensor(a_metric,S);

Rcon:=raise_lower_indices(a_metric,R, [1,2]);
       
ans:= DiffGeomMain:-Minus(Rcon, DiffGeomMain:-Mult( 1/2 , Sg) );       
        
end:

#---------------------------------------------------------------

Weyl_tensor:=proc(metric::Vessiot_bftv,curvature::Vessiot_bftv)

local Ricci,S,n, t1,t2, t3; 

n:=frameBaseDimension();

if nargs=3 then 
	Ricci:=args[3] 
else 
	Ricci:=Ricci_tensor(curvature) 
fi;

if nargs=4 then 
	S:=args[4] 
else 
	S:=Ricci_scalar(inverse_metric(metric),curvature)
fi;

t1:= raise_lower_indices(metric,curvature, [2]);

t2:= rearrange_indices(metric &tensor Ricci, [1,4,3,2]);
t2:= (4/(n-2)) &mult symm_skew_indices(symm_skew_indices(t2, [1,2],skew),[3,4],skew);

t3:=rearrange_indices(metric &tensor metric, [1,4,3,2]);
t3:=( -2/((n-1)*(n-2)))  &mult  (S &tensor  symm_skew_indices(t3,[1,2],skew));


t1  &plus t2 &plus t3;
end:

#-----------------------------------------

Cotton_tensor:=proc(metric,offel,curvature)

local n, Ricci, h, S,t1, t2, t3,t4;

n:=frameBaseDimension();
h:=inverse_metric(metric);

if n<>3  then 
	ERROR(cat("\n VESSIOT ERROR:",
                "\n The number of independent variables must be 3"))
fi;

if nargs=4 then 
	Ricci:=args[4] 
else 
	Ricci:=Ricci_tensor(curvature) 
fi;

if nargs=5 then 
	S:=args[5] 
else 
	S:=Ricci_scalar(h,curvature)
fi;

t1:= Ricci &minus ( (1/4) &mult ( metric &tensor S)); 
t2:= covariant_derivative(t1,offel );
t3:= raise_lower_indices(h,t2, [2]);
t4:= contract_indices(permutation_symbol(con_bas) &tensor t3, [[2,4], [3,6]]);

t4;
end:

