#
# rearrange.vss
#  
# tensors
#
# rearrange_indices; symm_skew_indices
#

rearrange_indices := proc(tensor1::Vessiot_tensor,new_order::list)
# CLASS I PROGRAM
# 05/01/99:CH
# 01/04/03:IA

#    This routine rearranges the order of Vessiot tensor indices.
#     
#    new_order: a list of n distinct integers, permuted to represent then new index 
#               order given the original indx order was [1,2,3...,n].

local ans, indexType, tensor_order, tensor_components,new_type, new_attributes,
          new_comps, new_index,  term; 

VessiotFrameCheck(tensor1);

indexType:=tensorIndexType(tensor1);
tensor_order:=nops(indexType):
tensor_components:=objectComponents(tensor1);

if  convert(new_order,set) <> {$1..tensor_order} then
	ERROR(cat("\n VESSIOT ERROR:", 
                "\n new index order must be a rearrangement of 1 --", tensor_order))
fi;

new_type:=map(op,new_order,indexType);
new_attributes:=[tensor,objectFrame(tensor1), [new_type,tensorDensityType(tensor1)]];

new_comps:=[];

for term in tensor_components do
	new_index:=map(op,new_order,term[1]);
      new_comps:=[op(new_comps),[new_index,term[2]]];
od;

ans:=_VESSIOT([new_attributes,new_comps]);
       
ans;

end:

#----------------------------------------------------------------


symm_skew_indices:=proc(tensor1,a_perm_list::list,pm)
# CLASS II PROGRAM
# 05/01/99:CH
# 01/04/03:IA     

#    This routine symmetrizes/or skewtries a given tensor on a set of indices.
#    perm_list: a list indx slots over which symmetrization is to occur.
#    pm = symm for symmetrize, skew for  skew_symmetrize.

local indexType, indxList,n,p,i, extend_perm,
      perm_list, perm_list1, perm_list2, B, T, permSign;

permSign[symm] :=proc(sigma) 1 end;
permSign[skew]:=proc(sigma) sgn_perm(sigma) end:

VessiotFrameCheck(tensor1);

indexType:=tensorIndexType(tensor1);
p:=nops(indexType):

if p<= 1 then
	RETURN(tensor1)
fi;

if a_perm_list = ['all'] then
	perm_list:=[$1..p]
else
 	perm_list:=a_perm_list
fi;
 
indxList:=map(op,perm_list,indexType);

if convert(indxList,set)<> {indxList[1]} then
	ERROR(cat("\n VESSIOT ERROR:",
	          "\n symmetrization can only occur over indices of the same type."))
fi;
 
n:=nops(perm_list);

perm_list1:=combinat[permute](perm_list);

#internal program
#Example 
#if perm_list1:=[[2,3],[3,2]] and B:=[1,2,3,4] then
#   perm_list2:=[[1,2,3,4],[1,3,2,4]]
#

extend_perm:=proc(A1,A2,B) 
local i, C; C:=B;

for i from 1 to nops(A1) do 
	C[A1[i]] :=A2[i] 
od; 
C;
end;

B:=[$1..p];

perm_list2:=  map(extend_perm, perm_list1,perm_list ,B);

T:=zero_tens([indexType,tensorDensityType(tensor1)]);

for i from 1 to nops(perm_list2) do
	T:=DiffGeomMain:-Plus(T,
         DiffGeomMain:-Mult(permSign[pm](perm_list2[i]), rearrange_indices(tensor1,perm_list2[i]) ));
od;

T:=  DiffGeomMain:-Mult(1/n!,  T);

end:
